@@ -10,16 +10,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -40,7 +40,7 @@ type body_flag =
    NoBody
  | OptBody
  | ColonBody
- | OptStringBody
+ | ArrayBody
 
 (*
  * Function applications can be tagged as Lazy or Eager.
@@ -71,6 +71,7 @@ type exp =
  | QuoteExp        of exp list * loc
  | QuoteStringExp  of char * exp list * loc
  | SequenceExp     of exp list * loc
+ | ArrayExp        of exp list * loc
  | ApplyExp        of apply_strategy * var * exp list * loc
  | SuperApplyExp   of apply_strategy * var * var * exp list * loc
  | MethodApplyExp  of apply_strategy * var list * exp list * loc
