@@ -10,16 +10,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -97,6 +97,11 @@ let rec pp_print_exp buf e =
          List.iter (fun e ->
                fprintf buf "@ %a" pp_print_exp e) el;
          fprintf buf ")@]"
+    | ArrayExp (el, _) ->
+         fprintf buf "@[<hv 3>(array";
+         List.iter (fun e ->
+               fprintf buf "@ %a" pp_print_exp e) el;
+         fprintf buf ")@]"
     | ApplyExp (LazyApply, v, [], _) ->
          fprintf buf "$%a" pp_print_symbol v
     | ApplyExp (s, v, el, _) ->
