@@ -12,16 +12,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -38,16 +38,18 @@ open Lm_string_set
 
 open Omake_ir
 open Omake_env
+open Omake_pos
 open Omake_eval
 open Omake_wild
 open Omake_node
 open Omake_exec
 open Omake_value
 open Omake_state
-open Omake_cache_type
 open Omake_builtin
+open Omake_cache_type
 open Omake_builtin_util
 open Omake_builtin_type
+open Omake_value_type
 
 module Pos = MakePos (struct let name = "Omake_builtin_arith" end)
 open Pos
