@@ -10,7 +10,8 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2003-2007 Mojave Group, Caltech
+ * Copyright (C) 2003-2007 Mojave Group, Calufornia Institute of Technology and
+ * HRL Laboratories, LLC
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,7 +32,7 @@
  * linked executables.  See the file LICENSE.OMake for more details.
  *
  * Author: Jason Hickey @email{jyh@cs.caltech.edu}
- * Modified By: Aleksey Nogin @email{nogin@cs.caltech.edu}
+ * Modified By: Aleksey Nogin @email{nogin@cs.caltech.edu}, @email{anogin@hrl.com}
  * @end[license]
  *)
 open Lm_printf
@@ -42,6 +43,8 @@ open Lm_string_set
 
 open Omake_ir
 open Omake_env
+open Omake_var
+open Omake_pos
 open Omake_eval
 open Omake_wild
 open Omake_node
@@ -55,6 +58,7 @@ open Omake_builtin
 open Omake_builtin_type
 open Omake_builtin_util
 open Omake_command_type
+open Omake_value_type
 
 module Pos = MakePos (struct let name = "Omake_builtin_base" end)
 open Pos
@@ -395,20 +399,20 @@ let if_fun venv pos loc args =
  *)
 let rec eval_match_cases1 compare venv pos loc s cases =
    match cases with
-      (v, pattern, e) :: cases ->
+      (v, pattern, el, export) :: cases ->
          if Lm_symbol.eq v case_sym then
             let pattern = string_of_value venv pos pattern in
                match compare venv pos loc pattern s with
                   Some venv ->
-                     eval_body_value venv pos e
+                     eval_sequence_export_exp venv pos el export
                 | None ->
                      eval_match_cases1 compare venv pos loc s cases
          else if Lm_symbol.eq v default_sym then
-            eval_body_value venv pos e
+            eval_sequence_export_exp venv pos el export
          else
             raise (OmakeException (loc_pos loc pos, StringVarError ("unknown case", v)))
     | [] ->
-         ValNone
+         venv, ValNone
 
 let rec eval_match_cases2 compare venv pos loc s cases =
    match cases with
@@ -416,13 +420,13 @@ let rec eval_match_cases2 compare venv pos loc s cases =
          let pattern = string_of_value venv pos pattern in
             (match compare venv pos loc pattern s with
                 Some venv ->
-                   eval_body_value venv pos e
+                   eval_body_exp venv pos ValNone e
               | None ->
                    eval_match_cases2 compare venv pos loc s cases)
     | [v] ->
          raise (OmakeException (loc_pos loc pos, StringValueError ("match requires an odd number of arguments", v)))
     | [] ->
-         ValNone
+         venv, ValNone
 
 let eval_match_exp compare venv pos loc args =
    let pos = string_pos "eval_match_exp" pos in
@@ -434,7 +438,7 @@ let eval_match_exp compare venv pos loc args =
             let s = string_of_value venv pos arg in
                eval_match_cases2 compare venv pos loc s rest
        | [] ->
-            ValNone
+            venv, ValNone
 
 let switch_fun =
    let compare venv _ _ s1 s2 =
@@ -448,10 +452,10 @@ let switch_fun =
 let match_fun =
    let compare venv pos loc s1 s2 =
       let lex =
-         try lexer_of_string s1
-         with Failure err ->
-            let msg = sprintf "Mailformed regular expression '%s'" s1 in
-               raise (OmakeException (loc_pos loc pos, StringStringError (msg, err)))
+         try lexer_of_string s1 with
+            Failure err ->
+               let msg = sprintf "Malformed regular expression '%s'" s1 in
+                  raise (OmakeException (loc_pos loc pos, StringStringError (msg, err)))
       in
       let channel = Lm_channel.of_string s2 in
          match Lexer.search lex channel with
@@ -511,7 +515,7 @@ let match_fun =
  * Temporary type for evaluating try blocks.
  *)
 type try_exp =
-   TrySuccessExp of value
+   TrySuccessExp of ( (venv * value) )
  | TryFailureExp of pos * obj * exn
 
 (*
@@ -527,9 +531,9 @@ let object_of_omake_exception venv pos exp =
       pp_print_exn stdstr exp;
       flush_stdstr ()
    in
-   let obj = venv_find_object_or_empty venv ScopeGlobal runtime_exception_sym in
-   let obj = venv_add_field obj pos_sym (ValString pos) in
-   let obj = venv_add_field obj message_sym (ValString exp) in
+   let obj = venv_find_object_or_empty venv runtime_exception_var in
+   let obj = venv_add_field_internal obj pos_sym (ValString pos) in
+   let obj = venv_add_field_internal obj message_sym (ValString exp) in
    let obj = venv_add_class obj runtime_exception_sym in
       obj
 
@@ -542,45 +546,45 @@ let object_of_uncaught_exception venv pos exn =
       pp_print_pos stdstr pos;
       flush_stdstr ()
    in
-   let obj = venv_find_object_or_empty venv ScopeGlobal runtime_exception_sym in
-   let obj = venv_add_field obj pos_sym (ValString pos) in
-   let obj = venv_add_field obj message_sym (ValString (Printexc.to_string exn)) in
+   let obj = venv_find_object_or_empty venv runtime_exception_var in
+   let obj = venv_add_field_internal obj pos_sym (ValString pos) in
+   let obj = venv_add_field_internal obj message_sym (ValString (Printexc.to_string exn)) in
    let obj = venv_add_class obj runtime_exception_sym in
       obj
 
 (*
  * Exception handling.
  *)
-let rec eval_finally_case venv pos cases =
+let rec eval_finally_case venv pos result cases =
    match cases with
-      (v, _, e) :: cases when Lm_symbol.eq v finally_sym ->
-         ignore (eval_body_value venv pos e)
+      (v, _, e, export) :: cases when Lm_symbol.eq v finally_sym ->
+         eval_sequence_export venv pos result e export
     | _ :: cases ->
-         eval_finally_case venv pos cases
+         eval_finally_case venv pos result cases
     | [] ->
-         ()
+         venv, result
 
 (*
  * We have successfully evaluated a CatchCase.
  * Search for the following WhenCases.
  * If a WhenCase fails, go to the following catch case.
  *)
-let rec eval_catch_rest venv_orig venv pos obj result cases =
+let rec eval_catch_rest venv pos obj result cases =
    match cases with
-      (v, s, e) :: cases when Lm_symbol.eq v when_sym ->
+      (v, s, e, export) :: cases when Lm_symbol.eq v when_sym ->
          let b = bool_of_value venv pos s in
             if b then
-               let venv, result = eval_body_exp venv pos result e in
-                  eval_catch_rest venv_orig venv pos obj result cases
+               let venv, result = eval_sequence_export venv pos result e export in
+                  eval_catch_rest venv pos obj result cases
             else
-               eval_exception venv_orig pos obj cases
+               eval_exception venv pos obj cases
     | _ ->
-         Some result
+         Some (venv, result)
 
-and eval_catch_case venv_orig pos v obj e cases =
-   let venv = venv_add_var venv_orig ScopeGlobal pos v (ValObject obj) in
-   let venv, result = eval_body_exp venv pos ValNone e in
-      eval_catch_rest venv_orig venv pos obj result cases
+and eval_catch_case venv pos v obj e cases export =
+   let venv = venv_add_var venv v (ValObject obj) in
+   let venv, result = eval_sequence_export_exp venv pos e export in
+      eval_catch_rest venv pos obj result cases
 
 (*
  * Find the first CatchCase that matches the object,
@@ -588,13 +592,15 @@ and eval_catch_case venv_orig pos v obj e cases =
  *)
 and eval_exception venv pos obj cases =
    match cases with
-      (v, s, e) :: cases ->
+      (v, s, e, export) :: cases ->
          if Lm_symbol.eq v when_sym then
             eval_exception venv pos obj cases
          else if Lm_symbol.eq v finally_sym then
             None
          else if Lm_symbol.eq v default_sym || venv_instanceof obj v then
-            eval_catch_case venv pos (Lm_symbol.add (string_of_value venv pos s)) obj e cases
+            (* FIXME: BUG: JYH: this binding occurence should be fixed *)
+            let v = VarThis (loc_of_pos pos, Lm_symbol.add (string_of_value venv pos s)) in
+               eval_catch_case venv pos v obj e cases export
          else
             eval_exception venv pos obj cases
     | [] ->
@@ -616,7 +622,7 @@ let try_fun venv pos loc args =
    in
    let e =
       try
-         let result = eval_body_value venv pos e in
+         let result = eval_body_value_env venv pos e in
             TrySuccessExp result
       with
          OmakeException (pos, exp) as exn ->
@@ -648,11 +654,11 @@ let try_fun venv pos loc args =
        | TrySuccessExp _ ->
             e
    in
-      eval_finally_case venv pos cases;
       match e with
-         TrySuccessExp result ->
-            result
+         TrySuccessExp ((venv, result)) ->
+            eval_finally_case venv pos result cases
        | TryFailureExp (_, _, exn) ->
+            ignore (eval_finally_case venv pos ValNone cases);
             raise exn
 
 (*
@@ -705,7 +711,7 @@ let raise_fun venv pos loc args =
  * Non-zero values indicate abnormal termination.
  * \end{doc}
  *)
-let exit_fun venv pos loc args =
+let exit_aux f venv pos loc args =
    let pos = string_pos "exit" pos in
    let code =
       flush stdout;
@@ -724,7 +730,13 @@ let exit_fun venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
    in
-      raise (ExitException (loc_pos loc pos, code))
+      f loc pos code
+
+let exit_fun =
+   exit_aux (fun loc pos code -> raise (ExitException (loc_pos loc pos, code)))
+
+let exit_parent_fun =
+   exit_aux (fun loc pos code -> raise (ExitParentException (loc_pos loc pos, code)))
 
 (*
  * Check whether a variable is defined.
@@ -746,6 +758,13 @@ let exit_fun venv pos loc args =
  *        X = a b c
  *        export
  * \end{verbatim}
+ *
+ * It is acceptable to use qualified names.
+ *
+ * \begin{verbatim}
+ *     $(defined X.a.b)
+ *     $(defined public.X)
+ * \end{verbatim}
  * \end{doc}
  *)
 let defined venv pos loc args =
@@ -753,11 +772,11 @@ let defined venv pos loc args =
       match args with
          [arg] ->
             let args = strings_of_value venv pos arg in
-            let b =
-               List.for_all (fun s ->
-                     venv_defined venv ScopeGlobal (Lm_symbol.add s)) args
-            in
-               val_of_bool b
+            let b = List.for_all (fun s -> defined_sym venv pos loc s) args in
+               if b then
+                  val_true
+               else
+                  val_false
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
@@ -875,7 +894,7 @@ let setenv venv pos loc args =
             let v = string_of_value venv pos arg1 in
             let s = string_of_value venv pos arg2 in
             let venv = venv_setenv venv (Lm_symbol.add v) s in
-               ValEnv (venv, ExportAll)
+               venv, ValData s
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
@@ -903,7 +922,7 @@ let unsetenv venv pos loc args =
                List.fold_left (fun venv v ->
                      venv_unsetenv venv (Lm_symbol.add v)) venv vars
             in
-               ValEnv (venv, ExportAll)
+               venv, ValNone
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
@@ -1010,14 +1029,19 @@ let get_registry venv pos loc args =
  *     foo_1 = abc
  *     X = $(getvar $(NAME)_1)
  * \end{verbatim}
+ *
+ * It is acceptable to use qualified names.
+ *
+ * \begin{verbatim}
+ *     $(getvar X.a.b)
+ * \end{verbatim}
  * \end{doc}
  *)
 let getvar venv pos loc args =
    let pos = string_pos "getvar" pos in
       match args with
          [arg] ->
-            let s = string_of_value venv pos arg in
-               venv_find_var venv ScopeGlobal pos loc (Lm_symbol.add s)
+            get_sym venv pos loc (string_of_value venv pos arg)
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
@@ -1038,15 +1062,21 @@ let getvar venv pos loc args =
  *    NAME = X
  *    setvar($(NAME), abc)
  * \end{verbatim}
+ *
+ * It is acceptable to use qualified names.
+ *
+ * \begin{verbatim}
+ *     setvar(public.X, abc)
+ * \end{verbatim}
  * \end{doc}
  *)
 let setvar venv pos loc args =
    let pos = string_pos "setvar" pos in
       match args with
          [arg1; arg2] ->
-            let v = Lm_symbol.add (string_of_value venv pos arg1) in
-            let venv = venv_add_var venv ScopeGlobal pos v arg2 in
-               ValEnv (venv, ExportAll)
+            let s = string_of_value venv pos arg1 in
+            let venv = add_sym venv pos loc s arg2 in
+               venv, arg2
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
@@ -1446,6 +1476,35 @@ let string venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
+(*
+ * \begin{doc}
+ * \fun{string-length}
+ *
+ * \begin{verbatim}
+ *    $(string-length sequence) : Int
+ *       sequence : Sequence
+ * \end{verbatim}
+ *
+ * The \verb+string-lenght+ returns a length (number of characters) in
+ * its argument. If the argument is a sequence, it flattens it, so \verb+$(string-length sequence)+
+ * is equivalent to \verb+$(string-length $(string sequence))+.
+ * \end{doc}
+ *)
+let string_length venv pos loc args =
+   let pos = string_pos "string-length" pos in
+      match args with
+         [arg] ->
+            let args = strings_of_value venv pos arg in
+            let len =
+               if args = [] then
+                  0
+               else
+                  List.fold_left (fun i s -> i + 1 + String.length s) (-1) args
+            in
+               ValInt len
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
+
 (*
  * \begin{doc}
  * \sixfuns{string-escaped}{ocaml-escaped}{html-escaped}{html-pre-escaped}{c-escaped}{id-escaped}
@@ -2385,8 +2444,7 @@ let system venv pos loc args =
    let pos = string_pos "system" pos in
       match args with
          [arg] ->
-            let _, result = eval_shell_exp venv pos loc arg in
-               result
+            eval_shell_exp venv pos loc arg
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
@@ -2458,7 +2516,7 @@ let shella venv pos loc args =
 
 let shell_code venv pos loc args =
    let pos = string_pos "shell-code" pos in
-   let venv = venv_add_var venv ScopeGlobal pos abort_on_command_error_sym val_false in
+   let venv = venv_add_var venv abort_on_command_error_var val_false in
    let _, result =
       match args with
          [arg] ->
@@ -2489,7 +2547,7 @@ let shell_code venv pos loc args =
  * A = 2
  * B = 2
  * C = 2
- * export $(SAVE_ENV)
+ * export($(SAVE_ENV))
  * println($A $B $C)
  * \end{verbatim}
  * will print \verb+1 1 2+.
@@ -2501,12 +2559,22 @@ let shell_code venv pos loc args =
 let export venv pos loc args =
    let pos = string_pos "export" pos in
       match args with
-         [] ->
-            ValEnv (venv, ExportAll)
-       | [(ValEnv _) as result] ->
-            result
-       | [arg] ->
-            eval_export_args venv pos arg
+         [ValOther (ValEnv (hand, export))] ->
+            let venv_new = venv_find_environment venv pos hand in
+            let venv = add_exports venv venv_new pos export in
+               venv, ValNone
+       | [vars] ->
+            let exports =
+               List.map (function
+                  ".PHONY" -> ExportPhonies
+                | ".RULE" -> ExportRules
+                | v -> ExportVar (VarGlobal (loc, Lm_symbol.add v))) (strings_of_value venv pos vars)
+            in
+            let hand = venv_add_environment venv in
+               venv, ValOther (ValEnv (hand, ExportList exports))
+       | [] ->
+            let hand = venv_add_environment venv in
+               venv, ValOther (ValEnv (hand, ExportAll))
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityRange (0, 1), List.length args)))
 
@@ -2582,9 +2650,9 @@ let export venv pos loc args =
  *)
 let rec eval_while_cases venv pos loc orig_cases arg cases =
    match cases with
-      (v, pattern, e) :: cases ->
+      (v, pattern, e, export) :: cases ->
          if Lm_symbol.eq v case_sym && bool_of_value venv pos pattern || Lm_symbol.eq v default_sym then
-            let venv, _ = eval_body_value_env venv pos e in
+            let venv, _ = eval_sequence_exp venv pos e in
                while_loop venv pos loc orig_cases arg
          else
             eval_while_cases venv pos loc orig_cases arg cases
@@ -2613,7 +2681,7 @@ let while_fun venv pos loc args =
          Break (_, venv) ->
             venv
    in
-      ValEnv (venv, ExportAll)
+      venv, ValNone
 
 (*
  * \begin{doc}
@@ -2714,6 +2782,7 @@ let () =
        (* String operations *)
        true,  "string",                string,              ArityExact 1;
        true,  "string-escaped",        string_escaped,      ArityExact 1;
+       true,  "string-length",         string_length,       ArityExact 1;
        true,  "ocaml-escaped",         ocaml_escaped,       ArityExact 1;
        true,  "c-escaped",             c_escaped,           ArityExact 1;
        true,  "id-escaped",            id_escaped,          ArityExact 1;
@@ -2733,15 +2802,13 @@ let () =
        (* System operations *)
        true,  "getenv",                getenv,              ArityRange (1, 2);
        true,  "defined-env",           defined_env,         ArityExact 1;
-       true,  "setenv",                setenv,              ArityExact 2;
-       true,  "unsetenv",              unsetenv,            ArityExact 1;
        true,  "exit",                  exit_fun,            ArityRange (0, 1);
+       true,  "exit-parent",           exit_parent_fun,     ArityRange (0, 1);
        true,  "raise",                 raise_fun,           ArityExact 1;
        true,  "get-registry",          get_registry,        ArityRange (3, 4);
 
        (* Normal variables *)
        true,  "getvar",                getvar,              ArityExact 1;
-       true,  "setvar",                setvar,              ArityExact 2;
 
        (* Logic *)
        true,  "not",                   not_fun,             ArityExact 1;
@@ -2749,9 +2816,6 @@ let () =
        false, "and",                   and_fun,             ArityAny;
        true,  "equal",                 equal,               ArityExact 2;
        true,  "if",                    if_fun,              ArityRange (2, 3);
-       false, "try",                   try_fun,             ArityExact 2;
-       false, "switch",                switch_fun,          ArityAny;
-       false, "match",                 match_fun,           ArityAny;
        true,  "defined",               defined,             ArityExact 1;
 
        (* List operations *)
@@ -2776,22 +2840,31 @@ let () =
        true,  "set-diff",              set_diff,            ArityExact 2;
 
        (* Shell command *)
-       true,  "system",                system,              ArityExact 1;
        true,  "shell",                 shell,               ArityExact 1;
        true,  "shella",                shella,              ArityExact 1;
        true,  "shell-code",            shell_code,          ArityExact 1;
 
-       true,  "export",                export,              ArityRange (0, 1);
-
-       false, "while",                 while_fun,           ArityExact 2;
        true,  "break",                 break,               ArityExact 0;
 
        true,  "random",                random,              ArityExact 0;
        true,  "random-init",           random_init,         ArityExact 1]
    in
+   let builtin_kfuns =
+      [true,  "setenv",                setenv,              ArityExact 2;
+       true,  "unsetenv",              unsetenv,            ArityExact 1;
+       true,  "setvar",                setvar,              ArityExact 2;
+       false, "switch",                switch_fun,          ArityAny;
+       false, "match",                 match_fun,           ArityAny;
+       false, "while",                 while_fun,           ArityExact 2;
+       false, "try",                   try_fun,             ArityExact 2;
+       true,  "export",                export,              ArityExact 0;
+       true,  "system",                system,              ArityExact 1;
+     ]
+   in
    let builtin_info =
       { builtin_empty with builtin_vars = builtin_vars;
-                           builtin_funs = builtin_funs
+                           builtin_funs = builtin_funs;
+                           builtin_kfuns = builtin_kfuns
       }
    in
       register_builtin builtin_info
