@@ -43,19 +43,22 @@ open Lm_string_set
 
 open Omake_ir
 open Omake_env
+open Omake_var
+open Omake_pos
 open Omake_eval
 open Omake_node
 open Omake_rule
 open Omake_value
 open Omake_target
 open Omake_symbol
+open Omake_builtin
 open Omake_node_sig
 open Omake_build_util
 open Omake_builtin_util
 open Omake_builtin_type
-open Omake_builtin
 open Omake_build_type
 open Omake_cache_type
+open Omake_value_type
 
 module Pos = MakePos (struct let name = "Omake_builtin_file" end)
 open Pos
@@ -494,7 +497,7 @@ let which venv pos loc args =
    let pos = string_pos "which" pos in
       match args with
          [arg] ->
-            let path = venv_find_var venv ScopeGlobal pos loc path_sym in
+            let path = venv_find_var venv pos loc path_var in
             let path = Omake_eval.path_of_values venv pos (values_of_value venv pos path) "." in
             let cache = venv_cache venv in
             let path = Omake_cache.ls_exe_path cache path in
@@ -532,7 +535,7 @@ let where venv pos loc args =
          [arg] ->
             (match strings_of_value venv pos arg with
                 [arg] ->
-                   let path = venv_find_var venv ScopeGlobal pos loc path_sym in
+                   let path = venv_find_var venv pos loc path_var in
                    let path = Omake_eval.path_of_values venv pos (values_of_value venv pos path) "." in
                    let cache = venv_cache venv in
                    let path = Omake_cache.ls_exe_path cache path in
@@ -540,16 +543,15 @@ let where venv pos loc args =
                    let res = List.map (fun v -> ValNode v) res in
                    let res =
                       try
-                         let obj = venv_find_var_exn venv ScopeGlobal shell_object_sym in
+                         let obj = venv_find_var_exn venv shell_object_var in
                             match eval_single_value venv pos obj with
                                ValObject obj ->
-                                  let v = venv_find_field_exn obj (Lm_symbol.add arg) in
+                                  let v = venv_find_field_internal_exn obj (Lm_symbol.add arg) in
                                   let kind =
                                      match eval_value venv pos v with
                                         ValPrim _ ->
                                            "Shell object method (a built-in function)"
-                                      | ValFun _
-                                      | ValFunValue _ ->
+                                      | ValFun _ ->
                                            "Shell object method (an omake function)"
                                       | _ ->
                                            "Shell object method"
@@ -895,11 +897,11 @@ let stat_reset venv pos loc args =
  *     if $(not $(defined FORCE_REALCLEAN))
  *         FORCE_REALCLEAN = false
  *         export
- *     
+ *
  *     distclean: clean
  *         cvs_realclean $(if $(FORCE_REALCLEAN), -f) -i .omakedb -i .omakedb.lock
  * \end{verbatim}
- * 
+ *
  * You can add more files that you want to always keep (such as configuration files) with the -i option.
  *
  * Similarly, if you use Subversion, you utilize the \verb+build/svn_realclean.om+ script that comes with \OMake:
@@ -908,9 +910,9 @@ let stat_reset venv pos loc args =
  *     if $(not $(defined FORCE_REALCLEAN))
  *         FORCE_REALCLEAN = false
  *         export
- *         
+ *
  *     open build/svn_realclean
- *     
+ *
  *     distclean: clean
  *         svn_realclean $(if $(FORCE_REALCLEAN), -f) -i .omakedb -i .omakedb.lock
  * \end{verbatim}
@@ -1724,7 +1726,7 @@ let mkdir venv pos loc args =
  * \obj{Stat}
  *
  * The \verb+Stat+ object represents an information about a filesystem node,
- * as returned by the \verb+stat+ and \verb+lstat+ functions. 
+ * as returned by the \verb+stat+ and \verb+lstat+ functions.
  * It contains the following fields.
  *
  * \begin{description}
@@ -1781,8 +1783,8 @@ let clip_size i =
       ValInt (Int64.to_int i)
 
 let create_stat_obj obj stat =
-   let obj = venv_add_field obj st_dev_sym (ValInt stat.Unix.LargeFile.st_dev) in
-   let obj = venv_add_field obj st_ino_sym (ValInt stat.Unix.LargeFile.st_ino) in
+   let obj = venv_add_field_internal obj st_dev_sym (ValInt stat.Unix.LargeFile.st_dev) in
+   let obj = venv_add_field_internal obj st_ino_sym (ValInt stat.Unix.LargeFile.st_ino) in
    let kind =
       match stat.Unix.LargeFile.st_kind with
          Unix.S_REG -> "REG"
@@ -1793,21 +1795,21 @@ let create_stat_obj obj stat =
        | Unix.S_FIFO -> "FIFO"
        | Unix.S_SOCK -> "SOCK"
    in
-   let obj = venv_add_field obj st_kind_sym (ValString kind) in
-   let obj = venv_add_field obj st_perm_sym (ValInt stat.Unix.LargeFile.st_perm) in
-   let obj = venv_add_field obj st_nlink_sym (ValInt stat.Unix.LargeFile.st_nlink) in
-   let obj = venv_add_field obj st_uid_sym   (ValInt stat.Unix.LargeFile.st_uid) in
-   let obj = venv_add_field obj st_gid_sym   (ValInt stat.Unix.LargeFile.st_gid) in
-   let obj = venv_add_field obj st_rdev_sym  (ValInt stat.Unix.LargeFile.st_rdev) in
-   let obj = venv_add_field obj st_size_sym  (clip_size stat.Unix.LargeFile.st_size) in
-   let obj = venv_add_field obj st_atime_sym (ValFloat stat.Unix.LargeFile.st_atime) in
-   let obj = venv_add_field obj st_mtime_sym (ValFloat stat.Unix.LargeFile.st_mtime) in
-   let obj = venv_add_field obj st_ctime_sym (ValFloat stat.Unix.LargeFile.st_ctime) in
+   let obj = venv_add_field_internal obj st_kind_sym (ValString kind) in
+   let obj = venv_add_field_internal obj st_perm_sym (ValInt stat.Unix.LargeFile.st_perm) in
+   let obj = venv_add_field_internal obj st_nlink_sym (ValInt stat.Unix.LargeFile.st_nlink) in
+   let obj = venv_add_field_internal obj st_uid_sym   (ValInt stat.Unix.LargeFile.st_uid) in
+   let obj = venv_add_field_internal obj st_gid_sym   (ValInt stat.Unix.LargeFile.st_gid) in
+   let obj = venv_add_field_internal obj st_rdev_sym  (ValInt stat.Unix.LargeFile.st_rdev) in
+   let obj = venv_add_field_internal obj st_size_sym  (clip_size stat.Unix.LargeFile.st_size) in
+   let obj = venv_add_field_internal obj st_atime_sym (ValFloat stat.Unix.LargeFile.st_atime) in
+   let obj = venv_add_field_internal obj st_mtime_sym (ValFloat stat.Unix.LargeFile.st_mtime) in
+   let obj = venv_add_field_internal obj st_ctime_sym (ValFloat stat.Unix.LargeFile.st_ctime) in
       ValObject obj
 
 let stat_aux stat_fun venv pos loc args =
    let pos = string_pos "stat" pos in
-   let obj = venv_find_object_or_empty venv ScopeGlobal stat_object_sym in
+   let obj = venv_find_object_or_empty venv stat_object_var in
       match args with
          [arg] ->
             let args = values_of_value venv pos arg in
@@ -2215,7 +2217,6 @@ let cp_file info file1 file2 =
                Unix.close fdr;
                raise exn
       in
-      let () =
          try
             let max = 4096 in
             let buffer = String.create max in
@@ -2229,18 +2230,17 @@ let cp_file info file1 file2 =
                         write 0;
                         loop ()
             in
-               loop ()
+               loop ();
+               Unix.close fdr;
+               Unix.close fdw;
+               try Unix.chmod file2 mode with
+                  Unix.Unix_error _ ->
+                     ()
          with
             exn ->
                Unix.close fdr;
                Unix.close fdw;
                raise exn
-         in
-            Unix.close fdr;
-            Unix.close fdw;
-            try Unix.chmod file2 mode with
-               Unix.Unix_error _ ->
-                  ()
 
 let rec cp_rec info file1 file2 =
    if is_dir file1 then
@@ -2722,7 +2722,7 @@ let vmount venv pos loc args =
    let venv = venv_mount venv mount_flags src dst in
       if List.mem MountForce local_flags then
          vmount_touch_files venv pos src dst;
-      ValEnv (venv, ExportAll)
+      venv, ValNone
 
 (*
  * \begin{doc}
@@ -2835,12 +2835,17 @@ let () =
        true, "find-targets-in-path-optional", find_targets_in_path_optional, ArityExact 2;
        true, "find-ocaml-targets-in-path-optional", find_ocaml_targets_in_path_optional, ArityExact 2;
 
-       true, "vmount",                  vmount,                   ArityRange (2, 3);
        true, "add-project-directories", add_project_directories,  ArityExact 1;
        true, "remove-project-directories", remove_project_directories,  ArityExact 1]
    in
+   let builtin_kfuns =
+      [true, "vmount",                  vmount,                   ArityRange (2, 3);
+      ]
+   in
    let builtin_info =
-      { builtin_empty with builtin_funs = builtin_funs }
+      { builtin_empty with builtin_funs = builtin_funs;
+                           builtin_kfuns = builtin_kfuns
+      }
    in
       register_builtin builtin_info
 
