@@ -38,6 +38,8 @@ open Lm_location
 
 open Omake_ir
 open Omake_env
+open Omake_var
+open Omake_pos
 open Omake_eval
 open Omake_node
 open Omake_value
@@ -46,6 +48,8 @@ open Omake_parser
 open Omake_printf
 open Omake_symbol
 open Omake_builtin
+open Omake_value_type
+open Omake_value_print
 open Omake_builtin_type
 open Omake_builtin_util
 
@@ -114,14 +118,14 @@ let open_in_string venv pos loc args =
 
 (*
  * \begin{doc}
- * \twofuns{open-out-string}{out-string}
+ * \twofuns{open-out-string}{out-contents}
  * The \verb+open-out-string+ creates a channel that writes to a
  * string instead of a file.  The string may be retrieved with the
- * \verb+out-string+ function.
+ * \verb+out-contents+ function.
  *
  * \begin{verbatim}
  *    $(open-out-string) : Channel
- *    $(out-string chan) : String
+ *    $(out-contents chan) : String
  *        chan : OutChannel
  * \end{verbatim}
  * \end{doc}
@@ -167,7 +171,7 @@ let out_contents venv pos loc args =
  * \item[r] Open the file for reading; it is an error if the file does not exist.
  * \item[w] Open the file for writing; the file is created if it does not exist.
  * \item[a] Open the file in append mode; the file is created if it does not exist.
- * \item[+] Open the file for both reading an writing.
+ * \item[+] Open the file for both reading and writing.
  * \item[t] Open the file in text mode (default).
  * \item[b] Open the file in binary mode.
  * \item[n] Open the file in nonblocking mode.
@@ -303,10 +307,11 @@ let close venv pos loc args =
 
 (*
  * \begin{doc}
- * \fun{read}
+ * \twofuns{read}{input-line}
  *
  * \begin{verbatim}
  *    $(read channel, amount) : String
+ *    $(input-line channel) : String
  *       channel : InChannel
  *       amount  : Int
  *    raises RuntimeException
@@ -314,8 +319,9 @@ let close venv pos loc args =
  *
  * The \verb+read+ function reads up to \verb+amount+
  * bytes from an input channel, and returns
- * the data that was read.  If an end-of-file condition is reached,
- * the function raises a \verb+RuntimeException+ exception.
+ * the data that was read. The \verb+input-line+ function reads a line from the file and returns the line read, without
+ * the line terminator. If an end-of-file condition is reached, both functions raise a \verb+RuntimeException+
+ * exception.
  * \end{doc}
  *)
 let read venv pos loc args =
@@ -333,11 +339,29 @@ let read venv pos loc args =
             in
                if count = amount then
                   ValData s
+               else if count = 0 then
+                  raise (UncaughtException (pos, End_of_file))
                else
                   ValData (String.sub s 0 count)
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
+let input_line venv pos loc args =
+   let pos = string_pos "input-line" pos in
+      match args with
+         [fd] ->
+            let fd = channel_of_value venv pos fd in
+            let s =
+               try Lm_channel.input_line fd with
+                  Sys_error _
+                | End_of_file
+                | Invalid_argument _ as exn ->
+                     raise (UncaughtException (pos, exn))
+            in
+                  ValData s
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
+
 (*
  * \begin{doc}
  * \fun{write}
@@ -546,6 +570,27 @@ let flush venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
+(*
+ * \begin{doc}
+ * \fun{channel-name}
+ *
+ * \begin{verbatim}
+ *    $(channel-name channel...) : String
+ *       channel : Channel
+ * \end{verbatim}
+ *
+ * The \verb+channel-name+ function returns the name that is associated with the channel.
+ * \end{doc}
+ *)
+let channel_name venv pos loc args =
+   let pos = string_pos "channel-name" pos in
+      match args with
+         [arg] ->
+            let fd = channel_of_value venv pos arg in
+               ValData (Lm_channel.name fd)
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
+
 (*
  * \begin{doc}
  * \fun{dup}
@@ -720,9 +765,9 @@ let pipe venv pos loc args =
             let write = Lm_channel.create "<writepipe>" Lm_channel.PipeChannel OutChannel false (Some fd_write) in
             let fd_read = ValChannel (InChannel, venv_add_channel venv read) in
             let fd_write = ValChannel (OutChannel, venv_add_channel venv write) in
-            let obj = venv_find_object_or_empty venv ScopeGlobal pipe_object_sym in
-            let obj = venv_add_field obj read_sym fd_read in
-            let obj = venv_add_field obj write_sym fd_write in
+            let obj = venv_find_object_or_empty venv pipe_object_var in
+            let obj = venv_add_field_internal obj read_sym fd_read in
+            let obj = venv_add_field_internal obj write_sym fd_write in
                ValObject obj
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 0, List.length args)))
@@ -813,10 +858,10 @@ let select venv pos loc args =
             let rfd = reintern_channel rfd in
             let wfd = reintern_channel wfd in
             let efd = reintern_channel efd in
-            let obj = venv_find_object_or_empty venv ScopeGlobal select_object_sym in
-            let obj = venv_add_field obj read_sym  (ValArray rfd) in
-            let obj = venv_add_field obj write_sym (ValArray wfd) in
-            let obj = venv_add_field obj error_sym (ValArray efd) in
+            let obj = venv_find_object_or_empty venv select_object_var in
+            let obj = venv_add_field_internal obj read_sym  (ValArray rfd) in
+            let obj = venv_add_field_internal obj write_sym (ValArray wfd) in
+            let obj = venv_add_field_internal obj error_sym (ValArray efd) in
                ValObject obj
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 4, List.length args)))
@@ -1333,7 +1378,7 @@ let getc venv pos loc args =
    let arg =
       match args with
          [] ->
-            venv_find_var venv ScopeGlobal pos loc stdin_sym
+            venv_find_var venv pos loc stdin_var
        | [arg] ->
             arg
        | _ ->
@@ -1374,7 +1419,7 @@ let gets venv pos loc args =
    let arg =
       match args with
          [] ->
-            venv_find_var venv ScopeGlobal pos loc stdin_sym
+            venv_find_var venv pos loc stdin_var
        | [arg] ->
             arg
        | _ ->
@@ -1415,7 +1460,7 @@ let fgets venv pos loc args =
    let arg =
       match args with
          [] ->
-            venv_find_var venv ScopeGlobal pos loc stdin_sym
+            venv_find_var venv pos loc stdin_var
        | [arg] ->
             arg
        | _ ->
@@ -1481,12 +1526,12 @@ let fprint venv pos loc args =
 
 let print venv pos loc args =
    let pos = string_pos "print" pos in
-   let stdout_fd = venv_find_var venv ScopeGlobal pos loc stdout_sym in
+   let stdout_fd = venv_find_var venv pos loc stdout_var in
       fprint venv pos loc (stdout_fd :: args)
 
 let eprint venv pos loc args =
    let pos = string_pos "eprint" pos in
-   let stderr_fd = venv_find_var venv ScopeGlobal pos loc stderr_sym in
+   let stderr_fd = venv_find_var venv pos loc stderr_var in
       fprint venv pos loc (stderr_fd :: args)
 
 let fprintln venv pos loc args =
@@ -1495,12 +1540,12 @@ let fprintln venv pos loc args =
 
 let println venv pos loc args =
    let pos = string_pos "println" pos in
-   let stdout_fd = venv_find_var venv ScopeGlobal pos loc stdout_sym in
+   let stdout_fd = venv_find_var venv pos loc stdout_var in
      fprintln venv pos loc (stdout_fd :: args)
 
 let eprintln venv pos loc args =
    let pos = string_pos "eprintln" pos in
-   let stderr_fd = venv_find_var venv ScopeGlobal pos loc stderr_sym in
+   let stderr_fd = venv_find_var venv pos loc stderr_var in
       fprintln venv pos loc (stderr_fd :: args)
 
 (*
@@ -1555,12 +1600,12 @@ let fprintv venv pos loc args =
 
 let printv venv pos loc args =
    let pos = string_pos "printv" pos in
-   let stdout_fd = venv_find_var venv ScopeGlobal pos loc stdout_sym in
+   let stdout_fd = venv_find_var venv pos loc stdout_var in
       fprintv venv pos loc (stdout_fd :: args)
 
 let eprintv venv pos loc args =
    let pos = string_pos "eprintv" pos in
-   let stderr_fd = venv_find_var venv ScopeGlobal pos loc stderr_sym in
+   let stderr_fd = venv_find_var venv pos loc stderr_var in
       fprintv venv pos loc (stderr_fd :: args)
 
 let fprintvln venv pos loc args =
@@ -1569,12 +1614,12 @@ let fprintvln venv pos loc args =
 
 let printvln venv pos loc args =
    let pos = string_pos "printvln" pos in
-   let stdout_fd = venv_find_var venv ScopeGlobal pos loc stdout_sym in
+   let stdout_fd = venv_find_var venv pos loc stdout_var in
      fprintvln venv pos loc (stdout_fd :: args)
 
 let eprintvln venv pos loc args =
    let pos = string_pos "eprintvln" pos in
-   let stderr_fd = venv_find_var venv ScopeGlobal pos loc stderr_sym in
+   let stderr_fd = venv_find_var venv pos loc stderr_var in
       fprintvln venv pos loc (stderr_fd :: args)
 
 (************************************************************************
@@ -1591,7 +1636,7 @@ struct
         print_channel : Lm_channel.t
       }
 
-   type value = Omake_env.value
+   type value = Omake_value_type.value
 
    (*
     * Create the buffers and channels.
@@ -1795,7 +1840,7 @@ let printf_fun venv pos loc args =
    let pos = string_pos "printf" pos in
       match args with
          fmt :: args ->
-            let stdout = venv_find_var venv ScopeGlobal pos loc stdout_sym in
+            let stdout = venv_find_var venv pos loc stdout_var in
             let stdout = channel_of_value venv pos stdout in
                fprintf_aux venv pos loc stdout fmt args
        | _ ->
@@ -1805,7 +1850,7 @@ let eprintf_fun venv pos loc args =
    let pos = string_pos "eprintf" pos in
       match args with
          fmt :: args ->
-            let stderr = venv_find_var venv ScopeGlobal pos loc stderr_sym in
+            let stderr = venv_find_var venv pos loc stderr_var in
             let stderr = channel_of_value venv pos stderr in
                fprintf_aux venv pos loc stderr fmt args
        | _ ->
@@ -1838,6 +1883,36 @@ let sprintf_fun venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
+(*
+ * \begin{doc}
+ * \subsection{Miscellaneous functions}
+ * \subsubsection{set-channel-line}
+ *
+ * \begin{verbatim}
+ *     set-channel-line(channel, filename, line)
+ *         channel : Channel
+ *         filename : File
+ *         line : int
+ * \end{verbatim}
+ *
+ * Set the line number information for the channel.
+ * \end{doc}
+ *)
+let set_channel_line_fun venv pos loc args =
+   let pos = string_pos "set-channel-line" pos in
+   let chan, file, line =
+      match args with
+         [chan; file; line] ->
+            chan, file, line
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 3, List.length args)))
+   in
+   let chan = channel_of_value venv pos chan in
+   let file = string_of_value venv pos file in
+   let line = int_of_value venv pos line in
+      Lm_channel.set_line chan file line;
+      ValNone
+
 (************************************************************************
  * Tables.
  *)
@@ -1856,11 +1931,13 @@ let () =
        true, "fopen",                 fopen,                ArityExact 2;
        true, "close",                 close,                ArityExact 1;
        true, "read",                  read,                 ArityExact 2;
+       true, "input-line",            input_line,           ArityExact 1;
        true, "write",                 write,                ArityRange (2, 4);
        true, "lseek",                 lseek,                ArityExact 3;
        true, "rewind",                rewind,               ArityExact 1;
        true, "tell",                  tell,                 ArityExact 1;
        true, "flush",                 flush,                ArityExact 1;
+       true, "channel-name",          channel_name,         ArityExact 1;
        true, "dup",                   dup,                  ArityExact 1;
        true, "dup2",                  dup2,                 ArityExact 2;
        true, "set-nonblock-mode",     set_nonblock_mode,    ArityExact 2;
@@ -1892,7 +1969,8 @@ let () =
        true, "printf",                printf_fun,           ArityAny;
        true, "eprintf",               eprintf_fun,          ArityAny;
        true, "fprintf",               fprintf_fun,          ArityAny;
-       true, "sprintf",               sprintf_fun,          ArityAny]
+       true, "sprintf",               sprintf_fun,          ArityAny;
+       true, "set-channel-line",      set_channel_line_fun, ArityExact 3]
    in
    let builtin_info =
       { builtin_empty with builtin_vars = builtin_vars;
@@ -1901,12 +1979,9 @@ let () =
    in
       register_builtin builtin_info
 
-(*!
- * @docoff
- *
+(*
  * -*-
  * Local Variables:
- * Caml-master: "compile"
  * End:
  * -*-
  *)
