@@ -85,7 +85,8 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2004-2006 Mojave Group, Caltech
+ * Copyright (C) 2004-2007 Mojave Group, California Institute of Technology, and
+ * HRL Laboratories, LLC
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -106,7 +107,7 @@
  * linked executables.  See the file LICENSE.OMake for more details.
  *
  * Author: Jason Hickey @email{jyh@cs.caltech.edu}
- * Modified By: Aleksey Nogin @email{nogin@cs.caltech.edu}
+ * Modified By: Aleksey Nogin @email{nogin@metaprl.org}, @email{anogin@hrl.com}
  * @end[license]
  *)
 open Lm_debug
@@ -117,6 +118,8 @@ open Lm_symbol
 
 open Omake_ir
 open Omake_env
+open Omake_var
+open Omake_pos
 open Omake_eval
 open Omake_node
 open Omake_value
@@ -125,6 +128,8 @@ open Omake_parser
 open Omake_printf
 open Omake_symbol
 open Omake_builtin
+open Omake_value_type
+open Omake_value_print
 open Omake_builtin_util
 open Omake_builtin_type
 
@@ -254,8 +259,7 @@ let grep_flags pos loc s =
 
 let grep venv pos loc args =
    let pos = string_pos "grep" pos in
-   let outv = venv_find_var venv ScopeGlobal pos loc stdout_sym in
-   let outx = channel_of_value venv pos outv in
+   let outx = channel_of_var venv pos loc stdout_var in
    let flags, pattern, files =
       match args with
          [pattern] ->
@@ -279,7 +283,7 @@ let grep venv pos loc args =
    let flags, files =
       match files with
          [] ->
-            flags, [venv_find_var venv ScopeGlobal pos loc stdin_sym]
+            flags, [venv_find_var venv pos loc stdin_var]
        | [_] ->
             flags, files
        | _::_::_ ->
@@ -502,7 +506,7 @@ let scan_args venv pos loc args =
    let cases, options, files =
       match args with
          [ValCases cases] ->
-            cases, ValNone, venv_find_var venv ScopeGlobal pos loc stdin_sym
+            cases, ValNone, venv_find_var venv pos loc stdin_var
        | [ValCases cases; files] ->
             cases, ValNone, files
        | [ValCases cases; options; files] ->
@@ -522,7 +526,7 @@ let scan venv pos loc args =
 
    (* Get lexers for all the cases *)
    let cases, def =
-      List.fold_left (fun (cases, def) (v, test, body) ->
+      List.fold_left (fun (cases, def) (v, test, body, export) ->
             if Lm_symbol.eq v case_sym then
                let s = string_of_value venv pos test in
                let cases =
@@ -531,7 +535,7 @@ let scan venv pos loc args =
                            Some _ ->
                               raise (OmakeException (loc_pos loc pos, StringVarError ("duplicate case", v)))
                          | None ->
-                              body)
+                              body, export)
                in
                   cases, def
             else if Lm_symbol.eq v default_sym then
@@ -539,7 +543,7 @@ let scan venv pos loc args =
                   Some _ ->
                      raise (OmakeException (loc_pos loc pos, StringError "duplicate default case"))
                 | None ->
-                     cases, Some body
+                     cases, Some (body, export)
             else
                raise (OmakeException (loc_pos loc pos, StringVarError ("unknown case", v)))) (SymbolTable.empty, None) cases
    in
@@ -571,15 +575,12 @@ let scan venv pos loc args =
           | [] ->
                def
       in
-      let result =
          match body with
-            Some body ->
-               eval_body_value venv pos body
+            Some (body, export) ->
+               let venv_new, _ = eval_sequence_exp venv pos body in
+                  add_exports venv venv_new pos export
           | None ->
-               ValNone
-      in
-      let venv, _ = add_exports venv pos result in
-         venv
+               venv
    in
 
    (* Read the file a line at a time *)
@@ -615,7 +616,7 @@ let scan venv pos loc args =
        | [] ->
             venv
    in
-      ValEnv (file_loop venv files, ExportAll)
+      file_loop venv files, ValNone
 
 (*
  * \begin{doc}
@@ -723,24 +724,25 @@ let scan venv pos loc args =
  *)
 let rec awk_eval_cases venv pos loc break line cases =
    match cases with
-      (None, body) :: cases ->
-         let result = eval_body_value venv pos body in
-         let venv, _ = add_exports venv pos result in
+      (None, body, export) :: cases ->
+         let venv_new, _ = eval_sequence_exp venv pos body in
+         let venv = add_exports venv venv_new pos export in
             if break then
                venv
             else
                awk_eval_cases venv pos loc break line cases
-    | (Some lex, body) :: cases ->
+    | (Some lex, body, export) :: cases ->
          let channel = Lm_channel.of_string line in
-         let result, stop =
+         let venv, stop =
             match Lexer.search lex channel with
                Some (_, _, _, _, args) ->
                   let venv = venv_add_match_args venv args in
-                     eval_body_value venv pos body, break
+                  let venv_new, _ = eval_sequence_exp venv pos body in
+                  let venv = add_exports venv venv_new pos export in
+                     venv, break
              | None ->
-                  ValNone, false
+                  venv, false
          in
-         let venv, _ = add_exports venv pos result in
             if stop then
                venv
             else
@@ -755,7 +757,7 @@ let awk_args venv pos loc args =
    let pos = string_pos "awk_args" pos in
       match args with
          [ValCases cases] ->
-            cases, [venv_find_var venv ScopeGlobal pos loc stdin_sym]
+            cases, [venv_find_var venv pos loc stdin_var]
        | [ValCases cases; files] ->
             cases, values_of_value venv pos files
        | _ ->
@@ -765,7 +767,7 @@ let awk_option_args venv pos loc args =
    let pos = string_pos "awk_args" pos in
       match args with
          [ValCases cases] ->
-            cases, "", [venv_find_var venv ScopeGlobal pos loc stdin_sym]
+            cases, "", [venv_find_var venv pos loc stdin_var]
        | [ValCases cases; files] ->
             cases, "", values_of_value venv pos files
        | [ValCases cases; options; files] ->
@@ -804,12 +806,12 @@ let awk venv pos loc args =
 
    (* Separator expressions *)
    let rs =
-      try string_of_value venv pos (venv_find_var_exn venv ScopeGlobal rs_sym) with
+      try string_of_value venv pos (venv_find_var_exn venv rs_var) with
          Not_found ->
             "\r|\n|\r\n"
    in
    let fs =
-      try string_of_value venv pos (venv_find_var_exn venv ScopeGlobal fs_sym) with
+      try string_of_value venv pos (venv_find_var_exn venv fs_var) with
          Not_found ->
             "[ \t]+"
    in
@@ -828,7 +830,7 @@ let awk venv pos loc args =
 
    (* Get lexers for all the cases *)
    let cases =
-      List.map (fun (v, test, body) ->
+      List.map (fun (v, test, body, export) ->
             if Lm_symbol.eq v case_sym then
                let s = string_of_value venv pos test in
                let _, lex =
@@ -837,9 +839,9 @@ let awk venv pos loc args =
                         let msg = sprintf "Malformed regular expression '%s'" s in
                            raise (OmakeException (loc_pos loc pos, StringStringError (msg, err)))
                in
-                  Some lex, body
+                  Some lex, body, export
             else if Lm_symbol.eq v default_sym then
-               None, body
+               None, body, export
             else
                raise (OmakeException (loc_pos loc pos, StringVarError ("unknown case", v)))) cases
    in
@@ -859,7 +861,7 @@ let awk venv pos loc args =
    in
 
    (* Read the file a line at a time *)
-   let rec line_loop venv inx =
+   let rec line_loop venv inx lineno =
       match Lexer.searchto rs_lex inx with
          Lexer.LexEOF ->
             venv
@@ -868,16 +870,18 @@ let awk venv pos loc args =
             (* Split into words *)
             let words = collect_words line in
             let venv = venv_add_match venv line words in
+            let venv = venv_add_var venv fnr_var (ValInt lineno) in
             let venv = awk_eval_cases venv pos loc break line cases in
-               line_loop venv inx
+               line_loop venv inx (lineno + 1)
    in
    let rec file_loop venv args =
       match args with
          arg :: args ->
             let inp, close_in = in_channel_of_any_value venv pos arg in
             let inx = venv_find_channel venv pos inp in
+            let venv = venv_add_var venv filename_var (ValData (Lm_channel.name inx)) in
             let venv =
-               try line_loop venv inx with
+               try line_loop venv inx 1 with
                   exn when close_in ->
                      venv_close_channel venv pos inp;
                      raise exn
@@ -893,7 +897,7 @@ let awk venv pos loc args =
          Break (_, venv) ->
             venv
    in
-      ValEnv (venv, ExportAll)
+      venv, ValNone
 
 (*
  * \begin{doc}
@@ -978,7 +982,7 @@ let rec subst_eval_case venv pos loc buf channel lex options body =
          Buffer.add_string buf skipped
     | Lexer.LexMatched (_, _, skipped, matched, args) ->
          let venv' = venv_add_match venv matched args in
-         let result = eval_body_value venv' pos body in
+         let _, result = eval_sequence_exp venv' pos body in
             Buffer.add_string buf skipped;
             Buffer.add_string buf (string_of_value venv pos result);
             if (options land subst_global_opt) <> 0 then
@@ -997,12 +1001,12 @@ let subst_eval_line venv pos loc line cases =
 let fsubst venv pos loc args =
    let pos = string_pos "fsubst" pos in
    let cases, files = awk_args venv pos loc args in
-   let outp = prim_channel_of_var venv pos loc stdout_sym in
+   let outp = prim_channel_of_var venv pos loc stdout_var in
    let outx = venv_find_channel venv pos outp in
 
    (* Record separator *)
    let rs =
-      try string_of_value venv pos (venv_find_var_exn venv ScopeGlobal rs_sym) with
+      try string_of_value venv pos (venv_find_var_exn venv rs_var) with
          Not_found ->
             "\r|\n|\r\n"
    in
@@ -1015,7 +1019,7 @@ let fsubst venv pos loc args =
 
    (* Get lexers for all the cases *)
    let cases =
-      List.map (fun (v, test, body) ->
+      List.map (fun (v, test, body, _) ->
             let args = values_of_value venv pos test in
             let pattern, options =
                match args with
@@ -1076,13 +1080,13 @@ let fsubst venv pos loc args =
        | [] ->
             ()
    in
-   let return =
-      try file_loop files; ValNone with
+   let venv =
+      try file_loop files; venv with
          Break (_, venv) ->
-            ValEnv (venv, ExportAll)
+            venv
    in
       Lm_channel.flush outx;
-      return
+      venv, ValNone
 
 (*
  * \begin{doc}
@@ -1144,7 +1148,7 @@ let lex venv pos loc args =
 
    (* Get lexers for all the cases *)
    let lex, cases, _ =
-      List.fold_left (fun (lex, cases, index) (v, test, body) ->
+      List.fold_left (fun (lex, cases, index) (v, test, body, export) ->
             let args = values_of_value venv pos test in
             let pattern =
                match args with
@@ -1168,7 +1172,7 @@ let lex venv pos loc args =
                         let msg = sprintf "Malformed regular expression '%s'" pattern in
                            raise (OmakeException (loc_pos loc pos, StringStringError (msg, err)))
                in
-               let cases = SymbolTable.add cases action_sym body in
+               let cases = SymbolTable.add cases action_sym (body, export) in
                   lex, cases, succ index) (lex, SymbolTable.empty, 0) cases
    in
 
@@ -1178,15 +1182,15 @@ let lex venv pos loc args =
          if Lm_symbol.eq action_sym eof_sym then
             venv
          else
-            let venv = venv_add_match venv lexeme args in
-            let venv = venv_add_var venv ScopeProtected pos loc_sym (ValOther (ValLocation lexeme_loc)) in
-            let body =
+            let venv_new = venv_add_match venv lexeme args in
+            let venv_new = venv_add_var venv_new parse_loc_var (ValOther (ValLocation lexeme_loc)) in
+            let body, export =
                try SymbolTable.find cases action_sym with
                   Not_found ->
                      raise (Invalid_argument "lex")
             in
-            let result = eval_body_value venv pos body in
-            let venv, _ = add_exports venv pos result in
+            let venv_new, _ = eval_sequence_exp venv_new pos body in
+            let venv = add_exports venv venv_new pos export in
                input_loop venv inx
    in
    let rec file_loop venv files =
@@ -1196,14 +1200,14 @@ let lex venv pos loc args =
             let inx = venv_find_channel venv pos inp in
             let venv =
                try input_loop venv inx with
-                  (Break _ | Return _ | UncaughtException _ ) as exn ->
+                  (Break _ | Return _ ) as exn ->
                      if close_in then
                         venv_close_channel venv pos inp;
                      raise exn
                 | exn ->
                      if close_in then
                         venv_close_channel venv pos inp;
-                     raise (UncaughtException (pos, exn))
+                     raise_uncaught_exception pos exn
             in
                if close_in then
                   venv_close_channel venv pos inp;
@@ -1216,7 +1220,7 @@ let lex venv pos loc args =
          Break (_, venv) ->
             venv
    in
-      ValEnv (venv, ExportAll)
+      venv, ValNone
 
 (*
  * \begin{doc}
@@ -1264,7 +1268,7 @@ let lex_search venv pos loc args =
 
    (* Get lexers for all the cases *)
    let lex, cases, default, _ =
-      List.fold_left (fun (lex, cases, default, index) (v, test, body) ->
+      List.fold_left (fun (lex, cases, default, index) (v, test, body, export) ->
             let args = values_of_value venv pos test in
             let pattern =
                match args with
@@ -1281,10 +1285,10 @@ let lex_search venv pos loc args =
                            let msg = sprintf "Malformed regular expression '%s'" pattern in
                               raise (OmakeException (loc_pos loc pos, StringStringError (msg, err)))
                   in
-                  let cases = SymbolTable.add cases action_sym body in
+                  let cases = SymbolTable.add cases action_sym (body, export) in
                      lex, cases, default, succ index
                else if Lm_symbol.eq v default_sym then
-                  lex, cases, Some body, index
+                  lex, cases, Some (body, export), index
                else
                   raise (OmakeException (loc_pos loc pos, StringVarError ("unknown case", v)))) (**)
          (Lexer.empty, SymbolTable.empty, None, 0) cases
@@ -1296,12 +1300,11 @@ let lex_search venv pos loc args =
          "", _
        | _, None ->
             venv
-       | _, Some body ->
-            let venv = venv_add_match venv lexeme [] in
-            let venv = venv_add_var venv ScopeProtected pos loc_sym (ValOther (ValLocation lexeme_loc)) in
-            let result = eval_body_value venv pos body in
-            let venv, _ = add_exports venv pos result in
-               venv
+       | _, Some (body, export) ->
+            let venv_new = venv_add_match venv lexeme [] in
+            let venv_new = venv_add_var venv_new parse_loc_var (ValOther (ValLocation lexeme_loc)) in
+            let venv_new, _ = eval_sequence_exp venv_new pos body in
+               add_exports venv venv_new pos export
    in
 
    (* Process the files *)
@@ -1316,15 +1319,15 @@ let lex_search venv pos loc args =
             let venv = skip venv lexeme_loc skipped in
 
             (* Process the matched text *)
-            let venv = venv_add_match venv lexeme args in
-            let venv = venv_add_var venv ScopeProtected pos loc_sym (ValOther (ValLocation lexeme_loc)) in
-            let body =
+            let venv_new = venv_add_match venv lexeme args in
+            let venv_new = venv_add_var venv_new parse_loc_var (ValOther (ValLocation lexeme_loc)) in
+            let body, export =
                try SymbolTable.find cases action_sym with
                   Not_found ->
                      raise (Invalid_argument "lex")
             in
-            let result = eval_body_value venv pos body in
-            let venv, _ = add_exports venv pos result in
+            let venv_new, _ = eval_sequence_exp venv_new pos body in
+            let venv = add_exports venv venv_new pos export in
                input_loop venv inx
    in
 
@@ -1336,14 +1339,14 @@ let lex_search venv pos loc args =
             let inx = venv_find_channel venv pos inp in
             let venv =
                try input_loop venv inx with
-                  (Break _ | Return _ | UncaughtException _ ) as exn ->
+                  (Break _ | Return _) as exn ->
                      if close_in then
                         venv_close_channel venv pos inp;
                      raise exn
                 | exn ->
                      if close_in then
                         venv_close_channel venv pos inp;
-                     raise (UncaughtException (pos, exn))
+                     raise_uncaught_exception pos exn
             in
                if close_in then
                   venv_close_channel venv pos inp;
@@ -1356,7 +1359,7 @@ let lex_search venv pos loc args =
          Break (_, venv) ->
             venv
    in
-      ValEnv (venv, ExportAll)
+      venv, ValNone
 
 (*
  * \begin{doc}
@@ -1526,7 +1529,7 @@ let lex_search venv pos loc args =
 let lex_rule venv pos loc args =
    let pos = string_pos "lex-rule" pos in
       match args with
-         [_; action; _; pattern; _; ValBody (_, SequenceExp (_, body))] ->
+         [_; action; _; pattern; _; ValBody (_, body, export)] ->
             let lexer = current_lexer venv pos in
             let action_name = string_of_value venv pos action in
             let action_sym = Lm_symbol.add action_name in
@@ -1539,10 +1542,10 @@ let lex_rule venv pos loc args =
             in
 
             (* Add the method *)
-            let body = SequenceExp (loc, body) in
-            let venv = venv_add_var venv ScopeProtected pos action_sym (ValFun (ArityExact 0, venv_get_env venv, [], body)) in
-            let venv = venv_add_var venv ScopeProtected pos builtin_sym (ValOther (ValLexer lexer)) in
-               ValEnv (venv, ExportAll)
+            let action_var = VarThis (loc, action_sym) in
+            let venv = venv_add_var venv action_var (ValFun (ArityExact 0, venv_get_env venv, [], body, export)) in
+            let venv = venv_add_var venv builtin_field_var (ValOther (ValLexer lexer)) in
+               venv, ValNone
 
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 6, List.length args)))
@@ -1569,15 +1572,15 @@ let lex_engine venv pos loc args =
                   venv_close_channel venv pos inp
             in
             let venv = venv_add_match venv lexeme args in
-            let venv = venv_add_var venv ScopeProtected pos loc_sym (ValOther (ValLocation lexeme_loc)) in
-            let action = venv_find_var venv ScopeProtected pos loc action in
+            let venv = venv_add_var venv parse_loc_var (ValOther (ValLocation lexeme_loc)) in
+            let action = venv_find_var venv pos loc (VarThis (loc, action)) in
                eval_apply venv pos loc action []
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
 (*
  * \begin{doc}
- * \subsection{Parser}
+ * \obj{Parser}
  *
  * The \verb+Parser+ object provides a facility for syntactic analysis based
  * on context-free grammars.
@@ -1736,7 +1739,7 @@ let lex_engine venv pos loc args =
  *)
 let parse_start venv pos loc args =
    let pos = string_pos "parse-start" pos in
-   let parser = current_parser venv pos in
+   let parse = current_parser venv pos in
    let args =
       match args with
          [arg] ->
@@ -1744,14 +1747,14 @@ let parse_start venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
    in
-   let parser =
-      List.fold_left (fun parser s ->
-            Parser.add_start parser (Lm_symbol.add s)) parser args
+   let parse =
+      List.fold_left (fun parse s ->
+            Parser.add_start parse (Lm_symbol.add s)) parse args
    in
 
    (* Redefine the parser *)
-   let venv = venv_add_var venv ScopeProtected pos builtin_sym (ValOther (ValParser parser)) in
-      ValEnv (venv, ExportAll)
+   let venv = venv_add_var venv builtin_field_var (ValOther (ValParser parse)) in
+      venv, ValNone
 
 (*
  * Precedence operations.
@@ -1759,48 +1762,48 @@ let parse_start venv pos loc args =
 let parse_prec venv pos loc args assoc =
    let pos = string_pos "parse-prec" pos in
    let this = venv_this venv in
-   let parser = current_parser venv pos in
-   let parser, level, args =
+   let parse = current_parser venv pos in
+   let parse, level, args =
       match args with
          [before; args] ->
             let current_prec = Lm_symbol.add (string_of_value venv pos before) in
             let level =
-               try Parser.find_prec parser current_prec with
+               try Parser.find_prec parse current_prec with
                   Not_found ->
                      raise (OmakeException (loc_pos loc pos, StringVarError ("no such precedence", current_prec)))
             in
-            let parser, level = Parser.create_prec_lt parser level assoc in
-               parser, level, args
+            let parse, level = Parser.create_prec_lt parse level assoc in
+               parse, level, args
        | [args] ->
-            let current_prec = Lm_symbol.add (string_of_value venv pos (venv_find_field this pos current_prec_sym)) in
+            let current_prec = Lm_symbol.add (string_of_value venv pos (venv_find_field_internal this pos current_prec_sym)) in
             let level =
-               try Parser.find_prec parser current_prec with
+               try Parser.find_prec parse current_prec with
                   Not_found ->
                      raise (OmakeException (loc_pos loc pos, StringVarError ("current precedence is not found", current_prec)))
             in
-            let parser, level = Parser.create_prec_gt parser level assoc in
-               parser, level, args
+            let parse, level = Parser.create_prec_gt parse level assoc in
+               parse, level, args
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityRange (1, 2), List.length args)))
    in
    let args = strings_of_value venv pos args in
-   let parser =
-      List.fold_left (fun parser s ->
-            Parser.add_prec parser level (Lm_symbol.add s)) parser args
+   let parse =
+      List.fold_left (fun parse s ->
+            Parser.add_prec parse level (Lm_symbol.add s)) parse args
    in
 
    (* Reset the current precedence *)
    let venv =
       match args with
          arg :: _ ->
-            venv_add_var venv ScopeProtected pos current_prec_sym (ValString arg)
+            venv_add_var venv current_prec_field_var (ValString arg)
        | [] ->
             venv
    in
 
    (* Redefine the parser *)
-   let venv = venv_add_var venv ScopeProtected pos builtin_sym (ValOther (ValParser parser)) in
-      ValEnv (venv, ExportAll)
+   let venv = venv_add_var venv builtin_field_var (ValOther (ValParser parse)) in
+      venv, ValNone
 
 let parse_left venv pos loc args =
    let pos = string_pos "parse-left" pos in
@@ -1844,23 +1847,23 @@ let prec_option venv pos loc options =
  *)
 let action_sym = Lm_symbol.add "action"
 
-let find_action_name venv =
-   Lm_symbol.new_name action_sym (venv_defined venv ScopeProtected)
+let find_action_name venv loc =
+   Lm_symbol.new_name action_sym (fun v -> venv_defined venv (VarThis (loc, v)))
 
 (*
  * Add a parser clause.
  *)
 let parse_rule venv pos loc args =
    let pos = string_pos "parse-rule" pos in
-   let action, head, rhs, options, body =
+   let action, head, rhs, options, body, export =
       match args with
-         [_; action; head; rhs; ValMap options; ValBody (_, SequenceExp (_, body))] ->
+         [_; action; head; rhs; ValMap options; ValBody (_, body, export)] ->
             let action = string_of_value venv pos action in
             let head = string_of_value venv pos head in
                if head = "" then   (* Action name was omitted *)
-                  find_action_name venv, Lm_symbol.add action, rhs, options, body
+                  find_action_name venv loc, Lm_symbol.add action, rhs, options, body, export
                else
-                  Lm_symbol.add action, Lm_symbol.add head, rhs, options, body
+                  Lm_symbol.add action, Lm_symbol.add head, rhs, options, body, export
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 6, List.length args)))
    in
@@ -1874,61 +1877,40 @@ let parse_rule venv pos loc args =
       match body with
          _ :: _ ->
             let body =
-               SequenceExp (**)
-                  (loc, (LetVarExp (loc, ScopeProtected, val_sym, VarDefNormal, ConstString (loc, "")) :: body))
+               LetVarExp (loc, VarThis (loc, val_sym), [], VarDefNormal, ConstString (loc, "")) :: body
             in
-               venv_add_var venv ScopeProtected pos action (ValFun (ArityExact 0, venv_get_env venv, [], body))
+               venv_add_var venv (VarThis (loc, action)) (ValFun (ArityExact 0, venv_get_env venv, [], body, export))
        | [] ->
             venv
    in
 
    (* Add back the parser *)
-   let venv = venv_add_var venv ScopeProtected pos builtin_sym (ValOther (ValParser par)) in
-      ValEnv (venv, ExportAll)
+   let venv = venv_add_var venv builtin_field_var (ValOther (ValParser par)) in
+      venv, ValNone
 
 (*
  * Perform the lexing.
  *)
-let parser_add_matches par args =
-   let par, _ =
-      List.fold_left (fun (par, i) arg ->
-            let v = Lm_symbol.add (string_of_int i) in
-            let par = venv_add_field par v arg in
-               par, succ i) (par, 1) args
-   in
-      par
-
 let parse_engine venv pos loc args =
    let pos = string_pos "parse-engine" pos in
       match args with
          [start] ->
             let dfa = current_parser venv pos in
             let start = Lm_symbol.add (string_of_value venv pos start) in
-            let lexer = venv_find_var venv ScopeProtected pos loc lexer_sym in
+            let lexer = venv_find_var venv pos loc lexer_field_var in
             let lexer = eval_object venv pos lexer in
             let parser_obj = venv_this venv in
             let lex (venv, parser_obj, lexer) =
-               let lex = venv_find_field lexer pos lex_sym in
+               let lex = venv_find_field_internal lexer pos lex_sym in
                let venv = venv_with_object venv lexer in
-               let result = eval_apply venv pos loc lex [] in
-
-               (* If the lex action resulted in an export, thread the lexer object *)
-               let venv, lexer, obj =
-                  match result with
-                     ValEnv _ ->
-                        let venv, _ = add_exports venv pos result in
-                        let lexer = venv_this venv in
-                           venv, lexer, lexer
-                   | _ ->
-                        let obj = eval_object venv pos result in
-                           venv, lexer, obj
-               in
+               let venv, result = eval_apply venv pos loc lex [] in
+               let obj = eval_object venv pos result in
                   try
-                     let lex_loc = venv_find_field_exn obj loc_sym in
+                     let lex_loc = venv_find_field_internal_exn obj loc_sym in
                      let lex_loc = loc_of_value venv pos lex_loc in
-                     let name = venv_find_field_exn obj name_sym in
+                     let name = venv_find_field_internal_exn obj name_sym in
                      let name = Lm_symbol.add (string_of_value venv pos name) in
-                     let value = venv_find_field_exn obj val_sym in
+                     let value = venv_find_field_internal_exn obj val_sym in
                         name, lex_loc, (venv, parser_obj, lexer), value
                   with
                      Not_found ->
@@ -1946,33 +1928,11 @@ let parse_engine venv pos loc args =
             in
             let eval (venv, parser_obj, lexer) action loc args =
                let pos = loc_pos loc pos in
-               let parser_obj = parser_add_matches parser_obj args in
-               let action = venv_find_field parser_obj pos action in
+               let venv = venv_add_match_values venv args in
+               let action = venv_find_field_internal parser_obj pos action in
                let venv = venv_with_object venv parser_obj in
-               let venv = venv_add_var venv ScopeGlobal pos loc_sym (ValOther (ValLocation loc)) in
-               let result = eval_apply venv pos loc action [] in
-
-               (* If the parse action resulted in an export, thread the parser object *)
-               let venv, parser_obj, result =
-                  match result with
-                     ValEnv _ ->
-                        let venv, _ = add_exports venv pos result in
-                        let parser_obj = venv_this venv in
-                        let result =
-                           try venv_find_field_exn parser_obj val_sym with
-                              Not_found ->
-                                 let print_error buf =
-                                    fprintf buf "@[<v 3>The parser returned a malformed object.\
-@ You used a parser action that ended in an 'export' and you did not define\
-@ the 'value' field in the parser object\
-@ %a@]" pp_print_value (ValObject parser_obj)
-                                 in
-                                    raise (OmakeException (pos, LazyError print_error))
-                        in
-                           venv, parser_obj, result
-                   | _ ->
-                        venv, parser_obj, result
-               in
+               let venv = venv_add_var venv parse_loc_var (ValOther (ValLocation loc)) in
+               let venv, result = eval_apply venv pos loc action [] in
                   (venv, parser_obj, lexer), result
             in
             let _, value =
@@ -1992,23 +1952,29 @@ let () =
       [true, "grep",                  grep,                 ArityRange (1, 3);
        true, "builtin-grep",          builtin_grep,         ArityExact 1;
        true, "cat",                   cat,                  ArityExact 1;
-       true, "lex-rule",              lex_rule,             ArityRange (3, 4);
+       true, "parse-engine",          parse_engine,         ArityExact 1;
+       true, "parse-build",           parse_build,          ArityExact 1;
+      ]
+   in
+   let builtin_kfuns =
+      [true, "lex-rule",              lex_rule,             ArityRange (3, 4);
        true, "lex-engine",            lex_engine,           ArityExact 1;
        true, "parse-rule",            parse_rule,           ArityRange (3, 5);
        true, "parse-start",           parse_start,          ArityExact 1;
-       true, "parse-engine",          parse_engine,         ArityExact 1;
        true, "parse-left",            parse_left,           ArityExact 1;
        true, "parse-right",           parse_right,          ArityExact 1;
        true, "parse-nonassoc",        parse_nonassoc,       ArityExact 1;
-       true, "parse-build",           parse_build,          ArityExact 1;
        true, "scan",                  scan,                 ArityRange (1, 3);
        true, "awk",                   awk,                  ArityExact 3;
        true, "fsubst",                fsubst,               ArityExact 3;
        true, "lex",                   lex,                  ArityExact 3;
-       true, "lex-search",            lex_search,           ArityExact 3]
+       true, "lex-search",            lex_search,           ArityExact 3;
+      ]
    in
    let builtin_info =
-      { builtin_empty with builtin_funs = builtin_funs }
+      { builtin_empty with builtin_funs = builtin_funs;
+                           builtin_kfuns = builtin_kfuns
+      }
    in
       register_builtin builtin_info
 
