@@ -36,6 +36,7 @@ open Lm_string_set
 
 open Omake_ir
 open Omake_env
+open Omake_pos
 open Omake_eval
 open Omake_wild
 open Omake_node
@@ -43,11 +44,12 @@ open Omake_exec
 open Omake_value
 open Omake_state
 open Omake_symbol
+open Omake_builtin
 open Omake_build_type
 open Omake_cache_type
-open Omake_builtin
 open Omake_builtin_type
 open Omake_builtin_util
+open Omake_value_type
 
 module Pos = MakePos (struct let name = "Omake_builtin_object" end)
 open Pos
@@ -69,7 +71,7 @@ let extends_fun venv pos loc args =
          venv_include_object venv obj
    in
    let venv = List.fold_left extend_arg venv args in
-      ValEnv (venv, ExportAll)
+      venv, ValNone
 
 (*
  * Get the object form of a value.
@@ -86,34 +88,6 @@ let object_fun venv pos loc args =
    let values = List.map (fun v -> ValObject (eval_object venv pos v)) values in
       concat_array values
 
-(*
- * Import an object from a file.
- * And add that object by its name to the environment.
- *
- * The argument may be a file or an object,
- * but if it is an object, this does nothing.
- *)
-let import_fun venv pos loc args =
-   let pos = string_pos "import" pos in
-   let import_val venv v =
-      match eval_value venv pos v with
-         ValObject _ ->
-            venv
-       | _ ->
-            let name = string_of_value venv pos v in
-            let obj = object_of_file venv pos loc name in
-            let name = Filename.basename name in
-            let name = Lm_filename_util.root name in
-            let name = Lm_symbol.add name in
-               venv_add_var venv ScopeGlobal pos name (ValObject obj)
-   in
-   let import_arg venv arg =
-      let values = values_of_value venv pos arg in
-         List.fold_left import_val venv values
-   in
-   let venv = List.fold_left import_arg venv args in
-      ValEnv (venv, ExportAll)
-
 (************************************************************************
  * Object operations.
  *)
@@ -128,7 +102,7 @@ let object_mem venv pos loc args =
             let obj = eval_object venv pos arg in
             let s = string_of_value venv pos v in
             let v = Lm_symbol.add s in
-               val_of_bool (venv_object_mem obj v)
+               val_of_bool (venv_defined_field venv obj v)
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
@@ -139,7 +113,7 @@ let object_find venv pos loc args =
             let obj = eval_object venv pos arg in
             let s = string_of_value venv pos v in
             let v = Lm_symbol.add s in
-               venv_find_field obj pos v
+               venv_find_field venv obj pos v
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
@@ -153,7 +127,8 @@ let object_add venv pos loc args =
             let obj = eval_object venv pos arg in
             let s = string_of_value venv pos v in
             let v = Lm_symbol.add s in
-               ValObject (venv_add_field obj v x)
+            let venv, obj = venv_add_field venv obj pos v x in
+               venv, ValObject obj
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
@@ -180,15 +155,17 @@ let object_map venv pos loc args =
             let obj = eval_object venv pos arg in
             let _, _, f = eval_fun venv pos fun_val in
                f, obj
-       | [arg; param1; param2; ValBody (env, body)] ->
+       | [arg; param1; param2; ValBody (env, body, export)] ->
             let params =
                [Lm_symbol.add (string_of_value venv pos param1);
                 Lm_symbol.add (string_of_value venv pos param2)]
             in
             let obj = eval_object venv pos arg in
             let f venv pos loc args =
-               let venv = venv_add_args venv pos loc env params args in
-                  eval venv body
+               let venv_new = venv_add_args_hack venv pos loc env params args in
+               let venv_new, result = eval_sequence_exp venv_new pos body in
+               let venv = add_exports venv venv_new pos export in
+                  venv, result
             in
                f, obj
        | _ ->
@@ -196,17 +173,32 @@ let object_map venv pos loc args =
    in
 
    (* If the body exports the environment, preserve it across calls *)
-   let venv', obj =
-      venv_object_fold (fun (venv, obj) v x ->
-            let result = f venv pos loc [ValString (Lm_symbol.to_string v); x] in
-            let obj = venv_add_field obj v result in
-            let venv, result = add_exports venv pos result in
+   let venv, obj =
+      venv_object_fold_internal (fun (venv, obj) v x ->
+            let venv, result = f venv pos loc [ValString (Lm_symbol.to_string v); x] in
+            let obj = venv_add_field_internal obj v result in
                venv, obj) (venv, obj) obj
    in
-      if venv' == venv then
-         ValObject obj
+      venv, ValObject obj
+
+(*
+ * instanceof predicate.
+ *)
+let object_instanceof venv pos loc args =
+   let pos = string_pos "instanceof" pos in
+   let obj, v =
+      match args with
+         [obj; v] ->
+            obj, v
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
+   in
+   let obj = eval_object venv pos obj in
+   let v = Lm_symbol.add (string_of_value venv pos v) in
+      if venv_instanceof obj v then
+         val_true
       else
-         ValEnv (venv', ExportAll)
+         val_false
 
 (************************************************************************
  * Map operations.
@@ -217,7 +209,7 @@ let object_map venv pos loc args =
  *)
 let map_of_object venv pos obj =
    try
-      match venv_find_field_exn obj map_sym with
+      match venv_find_field_internal_exn obj map_sym with
          ValMap map ->
             map
        | _ ->
@@ -231,7 +223,7 @@ let map_of_value venv pos arg =
       map_of_object venv pos obj
 
 let wrap_map obj map =
-   ValObject (venv_add_field obj map_sym (ValMap map))
+   ValObject (venv_add_field_internal obj map_sym (ValMap map))
 
 (*
  * Field membership.
@@ -308,7 +300,7 @@ let map_map venv pos loc args =
             let map = map_of_object venv pos obj in
             let _, _, f = eval_fun venv pos fun_val in
                f, obj, map
-       | [arg; param1; param2; ValBody (env, body)] ->
+       | [arg; param1; param2; ValBody (env, body, export)] ->
             let params =
                [Lm_symbol.add (string_of_value venv pos param1);
                 Lm_symbol.add (string_of_value venv pos param2)]
@@ -316,8 +308,10 @@ let map_map venv pos loc args =
             let obj = eval_object venv pos arg in
             let map = map_of_object venv pos obj in
             let f venv pos loc args =
-               let venv = venv_add_args venv pos loc env params args in
-                  eval venv body
+               let venv_new = venv_add_args_hack venv pos loc env params args in
+               let venv_new, result = eval_sequence_exp venv_new pos body in
+               let venv = add_exports venv venv_new pos export in
+                  venv, result
             in
                f, obj, map
        | _ ->
@@ -325,17 +319,13 @@ let map_map venv pos loc args =
    in
 
    (* If the body exports the environment, preserve it across calls *)
-   let venv', map =
+   let venv, map =
       venv_map_fold (fun (venv, map) v x ->
-            let result = f venv pos loc [v; x] in
+            let venv, result = f venv pos loc [v; x] in
             let map = venv_map_add map pos v result in
-            let venv, result = add_exports venv pos result in
                venv, map) (venv, map) map
    in
-      if venv' == venv then
-         wrap_map obj map
-      else
-         ValEnv (venv', ExportAll)
+      venv, wrap_map obj map
 
 (*
  * Get an array of keys of the map.
@@ -429,8 +419,7 @@ let sequence_length venv pos loc args =
                      venv_map_length map
                 | ValObject obj ->
                      venv_object_length obj
-                | ValNone
-                | ValEnv _ ->
+                | ValNone ->
                      0
                 | ValInt _
                 | ValFloat _
@@ -450,7 +439,7 @@ let sequence_length venv pos loc args =
                      List.length (values_of_value venv pos arg)
                 | ValArray a ->
                      List.length a
-                | ValFun (arity, _, _, _)
+                | ValFun (arity, _, _, _, _)
                 | ValFunValue (arity, _, _, _)
                 | ValPrim (arity, _, _) ->
                      (match arity with
@@ -466,13 +455,15 @@ let sequence_length venv pos loc args =
                 | ValCases cases ->
                      List.length cases
                 | ValClass _
-                | ValOther _ ->
+                | ValOther _
+                | ValVar _ ->
                      0
-                | ValKey _
+                | ValKeyApply _
                 | ValApply _
-                | ValImplicit _
+                | ValMaybeApply _
                 | ValSuperApply _
-                | ValMethodApply _ ->
+                | ValMethodApply _
+                | ValDelayed _ ->
                      raise (Invalid_argument "Omake_builtin_base.length")
             in
                ValInt len
@@ -494,12 +485,12 @@ let sequence_nth venv pos loc args =
                  | ValFun _
                  | ValFunValue _
                  | ValPrim _
-                 | ValKey _
+                 | ValKeyApply _
                  | ValApply _
-                 | ValImplicit _
+                 | ValMaybeApply _
                  | ValSuperApply _
                  | ValMethodApply _
-                 | ValEnv _
+                 | ValDelayed _
                  | ValMap _
                  | ValObject _ ->
                       raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", i)))
@@ -511,7 +502,8 @@ let sequence_nth venv pos loc args =
                  | ValChannel _
                  | ValClass _
                  | ValCases _
-                 | ValOther _ ->
+                 | ValOther _
+                 | ValVar _ ->
                       if i = 0 then
                          arg
                       else
@@ -554,6 +546,79 @@ let sequence_nth venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
+(*
+ * Subrange.
+ *)
+let sequence_sub venv pos loc args =
+   let pos = string_pos "sub" pos in
+      match args with
+         [arg; off; len] ->
+            let off = int_of_value venv pos off in
+            let len = int_of_value venv pos len in
+            let obj = eval_object venv pos arg in
+            let arg = eval_object_value venv pos obj in
+               (match arg with
+                   ValNone
+                 | ValFun _
+                 | ValFunValue _
+                 | ValPrim _
+                 | ValKeyApply _
+                 | ValApply _
+                 | ValMaybeApply _
+                 | ValSuperApply _
+                 | ValMethodApply _
+                 | ValDelayed _
+                 | ValMap _
+                 | ValObject _
+                 | ValInt _
+                 | ValFloat _
+                 | ValNode _
+                 | ValDir _
+                 | ValBody _
+                 | ValChannel _
+                 | ValClass _
+                 | ValCases _
+                 | ValOther _
+                 | ValVar _ ->
+                      raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", off)))
+                 | ValData s ->
+                      let length = String.length s in
+                         if off < 0 || len < 0 || off + len >= length then
+                            raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", off)));
+                         ValData (String.sub s off len)
+                 | ValQuote vl ->
+                      let s = string_of_quote venv pos None vl in
+                      let length = String.length s in
+                         if off < 0 || len < 0 || off + len >= length then
+                            raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", off)));
+                         ValData (String.sub s off len)
+                 | ValQuoteString (c, vl) ->
+                      let s = string_of_quote venv pos (Some c) vl in
+                      let length = String.length s in
+                         if off < 0 || len < 0 || off + len >= length then
+                            raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", off)));
+                         ValData (String.sub s off len)
+
+                 | ValSequence _
+                 | ValString _ ->
+                      let values = values_of_value venv pos arg in
+                      let length = List.length values in
+                         if off < 0 || len < 0 || off + len >= length then
+                            raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", off)));
+                         ValArray (Lm_list_util.sub values off len)
+                 | ValArray values ->
+                      let length = List.length values in
+                         if off < 0 || len < 0 || off + len >= length then
+                            raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", off)));
+                         ValArray (Lm_list_util.sub values off len)
+                 | ValRules values ->
+                      let length = List.length values in
+                         if off < 0 || len < 0 || off + len >= length then
+                            raise (OmakeException (loc_pos loc pos, StringIntError ("out of bounds", off)));
+                         ValRules (Lm_list_util.sub values off len))
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 3, List.length args)))
+
 (*
  * Reverse the elements in the sequence.
  *)
@@ -568,12 +633,12 @@ let sequence_rev venv pos loc args =
                  | ValFun _
                  | ValFunValue _
                  | ValPrim _
-                 | ValKey _
+                 | ValKeyApply _
                  | ValApply _
-                 | ValImplicit _
+                 | ValMaybeApply _
                  | ValSuperApply _
                  | ValMethodApply _
-                 | ValEnv _
+                 | ValDelayed _
                  | ValMap _
                  | ValObject _
                  | ValInt _
@@ -583,7 +648,8 @@ let sequence_rev venv pos loc args =
                  | ValBody _
                  | ValChannel _
                  | ValClass _
-                 | ValOther _ ->
+                 | ValOther _
+                 | ValVar _ ->
                       arg
                  | ValData s1 ->
                       let len = String.length s1 in
@@ -673,12 +739,14 @@ let foreach_fun venv pos loc args =
             let args = values_of_value venv pos arg in
             let _, _, f = eval_fun venv pos fun_val in
                f, args
-       | [ValBody (env, body); param; arg] ->
+       | [ValBody (env, body, export); param; arg] ->
             let params = [Lm_symbol.add (string_of_value venv pos param)] in
             let args = values_of_value venv pos arg in
             let f venv pos loc args =
-               let venv = venv_add_args_hack venv pos loc env params args in
-                  eval venv body
+               let venv_new = venv_add_args_hack venv pos loc env params args in
+               let venv_new, result = eval_sequence_exp venv_new pos body in
+               let venv = add_exports venv venv_new pos export in
+                  venv, result
             in
                f, args
        | _ ->
@@ -686,20 +754,173 @@ let foreach_fun venv pos loc args =
    in
 
    (* If the body exports the environment, preserve it across calls *)
-   let venv', values =
+   let venv, values =
       try
          List.fold_left (fun (venv, values) v ->
-               let result = f venv pos loc [v] in
-               let venv, result = add_exports venv pos result in
+               let venv, result = f venv pos loc [v] in
                   venv, result :: values) (venv, []) args
       with
          Break (_, venv) ->
             venv, []
    in
-      if venv' == venv then
-         ValArray (List.rev values)
+      venv, ValArray (List.rev values)
+
+(*
+ * \begin{doc}
+ * \section{Boolean tests}
+ *
+ * \fun{sequence-forall}
+ *
+ * The \verb+forall+ function tests whether a predicate halds for each
+ * element of a sequence.
+ *
+ * \begin{verbatim}
+ *     $(sequence-forall <fun>, <args>)
+ *
+ *     sequence-forall(<var> => ..., <args>)
+ *        <body>
+ * \end{verbatim}
+ * \end{doc}
+ *)
+let forall_fun venv pos loc args =
+   let pos = string_pos "sequence-forall" pos in
+   let f, args =
+      match args with
+         [fun_val; arg] ->
+            let args = values_of_value venv pos arg in
+            let _, _, f = eval_fun venv pos fun_val in
+               f, args
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
+   in
+   let rec test venv args =
+      match args with
+         arg :: args ->
+            let venv, result = f venv pos loc [arg] in
+               if bool_of_value venv pos result then
+                  test venv args
+               else
+                  venv, false
+       | [] ->
+            venv, true
+   in
+   let venv, x = test venv args in
+   let x =
+      if x then
+         val_true
       else
-         ValEnv (venv', ExportAll)
+         val_false
+   in
+      venv, x
+
+(*
+ * \begin{doc}
+ * \subsection{sequence-exists}
+ *
+ * The \verb+exists+ function tests whether a predicate holds for
+ * some element of a sequence.
+ *
+ * \begin{verbatim}
+ *     $(sequence-exists <fun>, <args>)
+ *
+ *     sequence-exists(<var> => ..., <args>)
+ *        <body>
+ * \end{verbatim}
+ * \end{doc}
+ *)
+let exists_fun venv pos loc args =
+   let pos = string_pos "sequence-exists" pos in
+   let f, args =
+      match args with
+         [fun_val; arg] ->
+            let args = values_of_value venv pos arg in
+            let _, _, f = eval_fun venv pos fun_val in
+               f, args
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
+   in
+
+   (* If the body exports the environment, preserve it across calls *)
+   let rec test venv args =
+      match args with
+         arg :: args ->
+            let venv, result = f venv pos loc [arg] in
+               if bool_of_value venv pos result then
+                  venv, true
+               else
+                  test venv args
+       | [] ->
+            venv, false
+   in
+   let venv, x = test venv args in
+   let x =
+      if x then
+         val_true
+      else
+         val_false
+   in
+      venv, x
+
+(*
+ * \begin{doc}
+ * \fun{sequence-sort}
+ *
+ * The \verb+sort+ function sorts the elements in an array,
+ * given a comparison function.  Given two elements (x, y),
+ * the comparison should return a negative number if x < y;
+ * a positive number if x > y; and 0 if x = y.
+ *
+ * \begin{verbatim}
+ *     $(sequence-sort <fun>, <args>)
+ *
+ *     sort(<var>, <var> => ..., <args>)
+ *        <body>
+ * \end{verbatim}
+ * \end{doc}
+ *)
+let sort_fun venv pos loc args =
+   let pos = string_pos "sequence-sort" pos in
+   let f, args =
+      match args with
+         [fun_val; arg] ->
+            let args = values_of_value venv pos arg in
+            let _, _, f = eval_fun venv pos fun_val in
+               f, args
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
+   in
+
+   (* If the body exports the environment, preserve it across calls *)
+   let compare v1 v2 =
+      let _, x = f venv pos loc [v1; v2] in
+         int_of_value venv pos x
+   in
+   let args = List.sort compare args in
+      venv, ValArray args
+
+(*
+ * \begin{doc}
+ * \fun{compare}
+ *
+ * The \verb+compare+ function compares two values (x, y) generically
+ * returning a negative number if x < y;
+ * a positive number if x > y; and 0 if x = y.
+ *
+ * \begin{verbatim}
+ *     $(compare x, y) : Int
+ * \end{verbatim}
+ * \end{doc}
+ *)
+let compare_fun venv pos loc args =
+   let pos = string_pos "compare" pos in
+   let x, y =
+      match args with
+         [x; y] ->
+            x, y
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
+   in
+      ValInt (ValueCompare.compare x y)
 
 (************************************************************************
  * Define the functions.
@@ -711,30 +932,39 @@ let foreach_fun venv pos loc args =
  *)
 let () =
    let builtin_funs =
-      [true, "extends",              extends_fun,         ArityAny;
-       true, "object",               object_fun,          ArityExact 1;
-       true, "foreach",              foreach_fun,         ArityExact 2;
-       true, "obj-add",              object_add,          ArityExact 3;
+      [true, "object",               object_fun,          ArityExact 1;
        true, "obj-find",             object_find,         ArityExact 2;
        true, "obj-mem",              object_mem,          ArityExact 2;
        true, "obj-length",           object_length,       ArityExact 1;
-       true, "obj-map",              object_map,          ArityRange (3, 4);
+       true, "obj-instanceof",       object_instanceof,   ArityExact 2;
        true, "map-add",              map_add,             ArityExact 3;
        true, "map-find",             map_find,            ArityExact 2;
        true, "map-mem",              map_mem,             ArityExact 2;
        true, "map-length",           map_length,          ArityExact 1;
-       true, "map-map",              map_map,             ArityRange (3, 4);
        true, "map-remove",           map_remove,          ArityExact 1;
        true, "map-keys",             map_keys,            ArityExact 1;
        true, "map-values",           map_values,          ArityExact 1;
-       true, "sequence-map",         foreach_fun,         ArityRange (2, 3);
        true, "sequence-length",      sequence_length,     ArityExact 1;
        true, "sequence-nth",         sequence_nth,        ArityExact 1;
        true, "sequence-rev",         sequence_rev,        ArityExact 1;
+       true, "sequence-sub",         sequence_sub,        ArityExact 3;
        true,  "create-map",          create_map,          ArityAny;
-       false, "create-lazy-map",     create_map,          ArityAny]
+       false, "create-lazy-map",     create_map,          ArityAny;
+       true, "compare",              compare_fun,         ArityExact 2;
+      ]
+   in
+   let builtin_kfuns =
+      [true, "obj-add",              object_add,          ArityExact 3;
+       true, "extends",              extends_fun,         ArityAny;
+       true, "foreach",              foreach_fun,         ArityExact 2;
+       true, "obj-map",              object_map,          ArityRange (3, 4);
+       true, "map-map",              map_map,             ArityRange (3, 4);
+       true, "sequence-map",         foreach_fun,         ArityRange (2, 3);
+       true, "sequence-forall",      forall_fun,          ArityExact 2;
+       true, "sequence-exists",      exists_fun,          ArityExact 2;
+       true, "sequence-sort",        sort_fun,            ArityExact 2;
+      ]
    in
-
    let builtin_vars =
       ["empty-map",        (fun _ -> ValMap venv_map_empty)]
    in
@@ -744,7 +974,7 @@ let () =
        "Float",            value_sym, ValFloat 0.0;
        "String",           value_sym, ValNone;
        "Array",            value_sym, ValArray [];
-       "Fun",              value_sym, ValFunValue (ArityExact 0, venv_empty_env, [], ValNone);
+       "Fun",              value_sym, ValFun (ArityExact 0, venv_empty_env, [], [], ExportNone);
        "Rule",             value_sym, ValRules [];
        "File",             value_sym, ValNone;
        "Dir",              value_sym, ValNone;
@@ -777,6 +1007,7 @@ let () =
    in
    let builtin_info =
       { builtin_empty with builtin_funs = builtin_funs;
+                           builtin_kfuns = builtin_kfuns;
                            builtin_vars = builtin_vars;
                            builtin_objects = builtin_objects;
                            pervasives_objects = pervasives_objects
