@@ -16,16 +16,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -42,6 +42,7 @@ open Lm_string_set
 
 open Omake_ir
 open Omake_env
+open Omake_pos
 open Omake_eval
 open Omake_wild
 open Omake_node
@@ -52,6 +53,7 @@ open Omake_value
 open Omake_state
 open Omake_symbol
 open Omake_builtin
+open Omake_value_type
 open Omake_builtin_type
 open Omake_builtin_util
 open Omake_command_type
@@ -158,8 +160,9 @@ let set_options venv pos loc args =
    let pos = string_pos "OMakeFlags" pos in
       match args with
          [arg] ->
-            let s = string_of_value venv pos arg in
-               ValEnv (venv_set_options venv loc pos s, ExportAll)
+            let argv = strings_of_value venv pos arg in
+            let venv = venv_set_options venv loc pos argv in
+               venv, ValNone
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
@@ -290,7 +293,7 @@ let define_command_vars venv pos loc args =
       match args with
          []
        | [_] ->
-            ValEnv (venv_add_command_defs venv, ExportAll)
+            venv_add_command_defs venv, ValNone
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityRange (0, 1), List.length args)))
 
@@ -300,15 +303,20 @@ let define_command_vars venv pos loc args =
 let () =
    let builtin_funs =
       [true,  "OMakeVersion",          check_version,       ArityRange (1, 2);
-       true,  "OMakeFlags",            set_options,         ArityExact 1;
        true,  "cmp-versions",          cmp_version,         ArityExact 2;
-       true,  "DefineCommandVars",     define_command_vars, ArityRange (0, 1)]
+      ]
+   in
+   let builtin_kfuns =
+      [true,  "OMakeFlags",            set_options,         ArityExact 1;
+       true,  "DefineCommandVars",     define_command_vars, ArityRange (0, 1);
+      ]
    in
    let builtin_rules =
       [true, [".PHONY"], phony_targets]
    in
    let builtin_info =
       { builtin_empty with builtin_funs  = builtin_funs;
+                           builtin_kfuns = builtin_kfuns;
                            builtin_rules = builtin_rules;
                            phony_targets = phony_targets
       }
