@@ -8,28 +8,29 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2005 Mojave Group, Caltech
+ * Copyright (C) 2005-2007 Mojave Group, California Institute of Technology and
+ * HRL Laboratories, LLC
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
  *
- * Author: Jason Hickey
- * @email{jyh@cs.caltech.edu}
+ * Author: Jason Hickey @email{jyh@cs.caltech.edu}
+ * Modified By: Aleksey Nogin @email{anogin@hrl.com}
  * @end[license]
  *)
 open Lm_printf
@@ -40,6 +41,7 @@ open Lm_string_set
 
 open Omake_ir
 open Omake_env
+open Omake_pos
 open Omake_eval
 open Omake_node
 open Omake_rule
@@ -47,14 +49,16 @@ open Omake_exec
 open Omake_value
 open Omake_state
 open Omake_symbol
+open Omake_builtin
 open Omake_node_sig
 open Omake_exec_util
 open Omake_build_type
 open Omake_cache_type
-open Omake_builtin
 open Omake_builtin_type
 open Omake_builtin_util
+open Omake_value_type
 open Omake_build
+open Omake_var
 
 module Pos = MakePos (struct let name = "Omake_builtin_target" end)
 open Pos
@@ -93,7 +97,7 @@ open Pos
  * in the current project.
  *
  * In all three functions, files that are not part of the current project are silently
- * discarded. All three functions will return phony and scanner targets along with the 
+ * discarded. All three functions will return phony and scanner targets along with the
  * ``real'' ones.
  *
  * One purpose of the \verb+dependencies-proper+ function is for ``clean'' targets.
@@ -120,9 +124,7 @@ let dependencies venv pos loc args =
    let pos = string_pos "dependencies" pos in
    let nodes =
       match args with
-         [ValRules rules] ->
-            List.fold_left (fun deps r -> NodeSet.union deps r.rule_sources) NodeSet.empty rules
-       | [arg] ->
+         [arg] ->
             let args  = values_of_value venv pos arg in
             let nodes = List.map (file_of_value venv pos) args in
             let rec find_deps deps node =
@@ -219,7 +221,7 @@ let split_command venv (values1, lines1) command =
             let v =
                match line with
                   CommandSection (_, _, e) ->
-                     ValBody (env, e)
+                     ValBody (env, e, ExportNone)
                 | CommandValue (_, v) ->
                      v
             in
@@ -257,7 +259,7 @@ let target_of_command venv pos loc command =
    let command_values, command_lines = split_commands venv commands in
 
    (* Get the default target object *)
-   let obj = venv_find_var_exn venv ScopeGlobal target_object_sym in
+   let obj = venv_find_var_exn venv target_object_var in
    let obj =
       match obj with
          ValObject obj ->
@@ -267,20 +269,20 @@ let target_of_command venv pos loc command =
    in
 
    (* Add the fields *)
-   let obj = venv_add_field obj target_sym (ValNode target) in
-   let obj = venv_add_field obj target_effects_sym effects in
-   let obj = venv_add_field obj scanner_deps_sym scanner_deps in
-   let obj = venv_add_field obj static_deps_sym static_deps in
-   let obj = venv_add_field obj build_deps_sym build_deps in
-   let obj = venv_add_field obj build_values_sym (ValArray command_values) in
-   let obj = venv_add_field obj build_commands_sym (ValArray command_lines) in
+   let obj = venv_add_field_internal obj target_sym (ValNode target) in
+   let obj = venv_add_field_internal obj target_effects_sym effects in
+   let obj = venv_add_field_internal obj scanner_deps_sym scanner_deps in
+   let obj = venv_add_field_internal obj static_deps_sym static_deps in
+   let obj = venv_add_field_internal obj build_deps_sym build_deps in
+   let obj = venv_add_field_internal obj build_values_sym (ValArray command_values) in
+   let obj = venv_add_field_internal obj build_commands_sym (ValArray command_lines) in
 
    (* Add the tee *)
    let obj =
       match tee_file tee with
          Some name ->
             let node = venv_intern venv PhonyProhibited name in
-               venv_add_field obj output_file_sym (ValNode node)
+               venv_add_field_internal obj output_file_sym (ValNode node)
        | None ->
             obj
    in
@@ -289,9 +291,7 @@ let target_of_command venv pos loc command =
 let target_core optional_flag venv pos loc args =
    let pos = string_pos "target" pos in
       match args with
-         [ValRules rules] ->
-            concat_array (List.map (fun r -> ValNode r.rule_target) rules)
-       | [arg] ->
+         [arg] ->
             let args     = values_of_value venv pos arg in
             let env      = get_env pos loc in
             let commands = env.env_commands in
@@ -453,11 +453,57 @@ let rule_fun venv pos loc args =
       match args with
          [multiple; target; pattern; source; options; body] ->
             let multiple = bool_of_value venv pos multiple in
-            let venv, v = eval_rule_exp venv pos loc multiple target pattern source options body in
-               ValEnv (venv, ExportValue v)
+               eval_rule_exp venv pos loc multiple target pattern source options body
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 6, List.length args)))
 
+(*
+ * This is called whenever a .STATIC: ... or a .MEMO: ... rule is evaluated.
+ * It isn't clear whether we want to document this.
+ *)
+let memo_rule_fun venv pos loc args =
+   let pos = string_pos "memo_rule_fun" pos in
+      match args with
+         [multiple; is_static; node; index; key; vars; source; options; body] ->
+            let multiple = bool_of_value venv pos multiple in
+            let is_static = bool_of_value venv pos is_static in
+            let key = values_of_value venv pos key in
+            let key = key_of_value venv pos (ValArray (node :: index ::key)) in
+            let vars = vars_of_value venv pos vars in
+            let target = file_of_value venv pos node in
+            let venv = eval_memo_rule_exp venv pos loc multiple is_static key vars target source options body in
+               venv, ValNone
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 9, List.length args)))
+
+(*
+ * \begin{doc}
+ * \fun{build}
+ *
+ * \begin{verbatim}
+ *     build(targets : File Array) : bool
+ * \end{verbatim}
+ *
+ * Build the given targets.  The value is true iff the build was successful.
+ * This function can be used only in \verb+osh+.
+ * \end{doc}
+ *)
+let build venv pos loc args =
+   let pos = string_pos "build" pos in
+      if not (Omake_options.opt_osh (venv_options venv)) then
+         raise (OmakeException (pos, StringError "build can be called only from osh"));
+      match args with
+         [arg] ->
+            let targets = strings_of_value venv pos arg in
+            let b =
+               try Omake_build.build_fun venv targets with
+                  exn ->
+                     raise_uncaught_exception pos exn
+            in
+               if b then val_true else val_false
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
+
 (************************************************************************
  * Hooks.
  *)
@@ -471,26 +517,28 @@ let () =
        true,  "dependencies-proper",  dependencies_proper, ArityExact 1;
        true,  "project-directories",  project_directories, ArityExact 0;
        true,  "find-build-targets",   find_build_targets,  ArityExact 1;
-
-       (* Rule definition *)
-       true,  "rule",                 rule_fun,            ArityExact 6]
+       true,  "build",                build,               ArityExact 1;
+      ]
+   in
+   let builtin_kfuns =
+      [true,  "rule",                 rule_fun,            ArityExact 6;
+       true,  "memo-rule",            memo_rule_fun,       ArityExact 9;
+      ]
    in
    let pervasives_objects =
       ["Target"]
    in
    let builtin_info =
       { builtin_empty with builtin_funs = builtin_funs;
+                           builtin_kfuns = builtin_kfuns;
                            pervasives_objects = pervasives_objects
       }
    in
       register_builtin builtin_info
 
-(*!
- * @docoff
- *
+(*
  * -*-
  * Local Variables:
- * Caml-master: "compile"
  * End:
  * -*-
  *)
