@@ -9,28 +9,29 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2005-2006 Mojave Group, Caltech
+ * Copyright (C) 2005-2007 Mojave Group, California Institute of Technology and
+ * HRL Laboratories, LLC
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
  *
- * Author: Jason Hickey
- * @email{jyh@cs.caltech.edu}
+ * Author: Jason Hickey @email{jyh@cs.caltech.edu}
+ * Modified By: Aleksey Nogin @email{anogin@hrl.com}
  * @end[license]
  *)
 open Lm_glob
@@ -40,12 +41,15 @@ open Lm_location
 
 open Omake_ir
 open Omake_env
+open Omake_pos
 open Omake_node
 open Omake_value
 open Omake_symbol
 open Omake_builtin
 open Omake_builtin_type
 open Omake_builtin_util
+open Omake_value_type
+open Omake_var
 
 module Pos = MakePos (struct let name = "Omake_builtin_test" end)
 open Pos
@@ -106,6 +110,7 @@ type token =
  | TokBinop of binop * string
  | TokIntop of intop * string
  | TokName of string
+ | TokRegex of string
 
 (*
  * Expressions.
@@ -188,6 +193,8 @@ let token_of_string s =
     | "-w"  -> TokUnop (IsWritableFileOp, s)
     | "-x"  -> TokUnop (IsExecutableFileOp, s)
     | "-name" -> TokName s
+    | "-regex" 
+    | "-regexp" -> TokRegex s
     | _     -> TokString s
 
 (*
@@ -206,6 +213,7 @@ let string_of_token arg =
     | TokUnop (_, s)
     | TokBinop (_, s)
     | TokIntop (_, s)
+    | TokRegex s
     | TokName s ->
          s
 
@@ -434,7 +442,7 @@ let older_than_file stat1 stat2 =
 let eval_string_exp venv pos arg =
    match arg with
       TokCurrentFile _ ->
-         string_of_value venv pos (venv_get_var venv ScopeGlobal pos braces_sym)
+         string_of_value venv pos (venv_get_var venv pos braces_var)
     | _ ->
          string_of_token arg
 
@@ -535,7 +543,7 @@ let eval_intop_exp venv pos op e1 e2 =
  *)
 let eval_match_exp venv pos regex =
    let basename =
-      match venv_get_var venv ScopeGlobal pos braces_sym with
+      match venv_get_var venv pos braces_var with
          ValNode node ->
             Node.tail node
        | v ->
@@ -622,6 +630,8 @@ let rec parse_term venv pos tokens =
             UnopExp (op, arg), tokens
        | TokName _ :: arg :: tokens ->
             MatchExp (regex_of_shell_pattern no_glob_options (string_of_token arg)), tokens
+       | TokRegex _ :: arg :: tokens ->
+            MatchExp (LmStr.regexp (string_of_token arg)), tokens
        | arg :: tokens ->
             UnopExp (IsNonEmptyStringOp, arg), tokens
        | [] ->
@@ -760,13 +770,13 @@ and parse_exp venv pos tokens =
  * \end{doc}
  *)
 let print_usage venv pos loc =
-   let outv = venv_find_var venv ScopeGlobal pos loc stderr_sym in
+   let outv = venv_find_var venv pos loc stderr_var in
    let outx = channel_of_value venv pos outv in
       Lm_channel.output_string outx "test <expression>\nFor usage, see the omake manual\n";
       false
 
 let print_version venv pos loc =
-   let outv = venv_find_var venv ScopeGlobal pos loc stdout_sym in
+   let outv = venv_find_var venv pos loc stdout_var in
    let outx = channel_of_value venv pos outv in
       Lm_channel.output_string outx "test version 1.0.0\n";
       false
@@ -838,6 +848,7 @@ let test_cmd venv pos loc argv =
  * \begin{itemize}
  * \item    \verb+-name+ \emph{string} : The current file matches the glob expression
  * (see Section~\ref{section:globbing}).
+ * \item    \verb+-regex+ \emph{string} : The current file matches the regular expression
  * \end{itemize}
  *
  * The \verb+find+ function performs a recursive scan of all subdirectories.
@@ -905,7 +916,7 @@ let rec find_file venv pos nodes name e =
 
    (* Add this node if the expression matches *)
    let nodes =
-      let venv = venv_add_var venv ScopeGlobal pos braces_sym v in
+      let venv = venv_add_var venv braces_var v in
          if eval_bool_exp venv pos e then
             v :: nodes
          else
@@ -965,7 +976,7 @@ let shell_find venv pos loc args =
    let pos = string_pos "find" pos in
       match args with
          [arg] ->
-            let stdout_fd = venv_find_var venv ScopeGlobal pos loc stdout_sym in
+            let stdout_fd = venv_find_var venv pos loc stdout_var in
             let outp, close_flag = out_channel_of_any_value venv pos stdout_fd in
             let outx = venv_find_channel venv pos outp in
             let nodes = find_top venv pos loc arg in
