@@ -4,24 +4,28 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2004 Jason Hickey, Caltech
+ * Copyright (C) 2004-2007 Mojave Group, Caltech and HRL Laboratories, LLC
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
+ * as published by the Free Software Foundation; version 2
+ * of the License.
+ * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * 
+ * Additional permission is given to link this library with the
+ * with the Objective Caml runtime, and to redistribute the
+ * linked executables.  See the file LICENSE.OMake for more details.
  *
- * Author: Jason Hickey
- * @email{jyh@cs.caltech.edu}
+ * Author: Jason Hickey @email{jyh@cs.caltech.edu}
+ * Modified by: Aleksey Nogin @email{nogin@metaprl.org}, @email{anogin@hrl.com}
  * @end[license]
  */
 #include <stdio.h>
@@ -30,15 +34,31 @@
 #include <caml/memory.h>
 #include <caml/fail.h>
 #include <caml/custom.h>
+#include <caml/callback.h>
+
+/*
+ * XXX: HACK (nogin 02/28/07):
+ * CAMLreturn with non-value types is wrong in 3.09.3 and later; CAMLreturnT was added in 3.09.4 and
+ * 3.10 to address this, but in 3.09.3 we are out of luck!
+ */
+#ifndef CAMLreturnT
+#define CAMLreturnT(type, result) do{ \
+   type caml__temp_result = (result); \
+   caml_local_roots = caml__frame; \
+   return (caml__temp_result); \
+}while(0)
+#endif
 
 #ifdef WIN32
+#  include <caml/signals.h>
 #  include <windows.h>
+   /* Disable some of the warnings */
+#  pragma warning( disable : 4100 4127 4505 4702 4996 4189)
 #else
 #  include <unistd.h>
 #  include <string.h>
 #  include <sys/types.h>
-#  include <sys/stat.h>
-#  include <dirent.h>
+#  include <limits.h>
 
 #  ifdef READLINE_ENABLED
 #    include <readline/readline.h>
@@ -47,101 +67,35 @@
 #endif /* !WIN32 */
 
 /*
- * Maximum number of characters in an input line.
+ * Maximum number of characters in an input line. Defined by limits.h on Unix.
  */
-#define MAX_LINE_LENGTH         256
+#ifndef LINE_MAX
+#define LINE_MAX                2048
+#endif
 
 /*
  * Prompts shouldn't be too wild.
  * In particular, they should be less than an average line length.
  */
-#define MAX_PROMPT_LENGTH       80
+#define MAX_PROMPT_LENGTH       70
 
 /*
- * Maximum length of the history filename.
+ * Maximum length of the history filename. Defined by limits.h on Unix.
  */
-#define MAX_FILENAME_LENGTH     2048
+#ifndef PATH_MAX
+#define PATH_MAX                2048
+#endif
 
-/************************************************************************
- * Compatibility.
- */
-
-#ifdef WIN32
 /*
- * Simulate opendir/readdir/closedir on Win32.
+ * Space for completions array.
  */
-struct dirent {
-    int d_isdir;
-    char d_name[MAX_FILENAME_LENGTH];
-};
+#define COMPLETION_LENGTH       10
 
-typedef struct _dir {
-    HANDLE hand;
-    struct dirent entry;
-    int full;
-} DIR;
-
-static DIR *opendir(const char *dirname)
-{
-    WIN32_FIND_DATA data;
-    char name[MAX_FILENAME_LENGTH + 2];
-    HANDLE hand;
-    DIR *dirp;
-    int len;
-
-    /* Always add the \* to the directory */
-    len = strlen(dirname);
-    memcpy(name, dirname, len);
-    name[len] = '\\';
-    name[len + 1] = '*';
-    name[len + 2] = 0;
-
-    /* Read it */
-    hand = FindFirstFile(name, &data);
-    if(hand == INVALID_HANDLE_VALUE)
-        return 0;
-
-    /* Allocate the info */
-    dirp = (DIR *) malloc(sizeof(DIR));
-    if(dirp == 0)
-        return 0;
-
-    /* Copy the entry */
-    dirp->hand = hand;
-    strncpy(dirp->entry.d_name, data.cFileName, MAX_FILENAME_LENGTH - 1);
-    dirp->entry.d_name[MAX_FILENAME_LENGTH - 1] = 0;
-    dirp->entry.d_isdir = data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
-    dirp->full = 1;
-    return dirp;
-}
-
-static struct dirent *readdir(DIR *dirp)
-{
-    WIN32_FIND_DATA data;
-
-    /* Do we already have an entry? */
-    if(dirp->full) {
-        dirp->full = 0;
-        return &dirp->entry;
-    }
-
-    /* Read the next entry */
-    if(FindNextFile(dirp->hand, &data) == 0)
-        return 0;
-
-    /* Copy it */
-    strncpy(dirp->entry.d_name, data.cFileName, MAX_FILENAME_LENGTH - 1);
-    dirp->entry.d_name[MAX_FILENAME_LENGTH - 1] = 0;
-    dirp->entry.d_isdir = data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
-    return &dirp->entry;
-}
-
-static void closedir(DIR *dirp)
-{
-    FindClose(dirp->hand);
-    free(dirp);
-}
-#endif /* WIN32 */
+/*
+ * Name of the command completion callback.
+ */
+static char omake_filename_completion[] = "omake_filename_completion";
+static char omake_command_completion[]  = "omake_command_completion";
 
 /************************************************************************
  * Completions.
@@ -150,197 +104,54 @@ static void closedir(DIR *dirp)
 /*
  * Linked list of filename completions.
  */
-typedef struct _completion {
-    char *name;
-    struct _completion *next;
-} Completion;
-
 typedef struct _completion_info {
     /* Current directory (needed for filename completion) */
-    char dir[MAX_FILENAME_LENGTH];
+    char dir[PATH_MAX];
 
     /* List of completions */
-    Completion *completions;
+    char **completions;
 } CompletionInfo;
 
 /*
- * It the file absolute?
+ * Command completions use a callback.
  */
-static int is_absolute_name(const char *name)
+static char **readline_completion(char *omake_completion, const char *text)
 {
+    CAMLparam0();
+    CAMLlocal2(request, response);
+    char *namep, **completions;
+    value *callbackp;
+    int i, length;
+
 #ifdef WIN32
-    int drive;
+    /* This is bogus but temporary */
+    (void) caml__dummy_request;
+#endif
 
-    drive = name[0];
-    return (drive >= 'a' && drive <= 'z' || drive >= 'A' && drive <= 'Z') && name[1] == ':' && (name[2] == '/' || name[2] == '\\');
-#else /* WIN32 */
-    return *name == '/';
-#endif /* !WIN32 */
-}
+    /* Find the callback, abort if it doesn't exist */
+    callbackp = caml_named_value(omake_completion);
+    if(callbackp == 0 || *callbackp == 0)
+        CAMLreturnT(char **, 0);
 
-/*
- * Find the last path separator.
- */
-static const char *last_slash(const char *name)
-{
-    int len;
-
-    len = strlen(name);
-    while(len) {
-        len--;
-
-        switch(name[len]) {
-        case '/':
-        case '\\':
-            return name + len;
-        }
+    /* The callback returns an array of strings */
+    request = copy_string(text);
+    response = caml_callback(*callbackp, request);
+    
+    /* Copy the array of strings */
+    length = Wosize_val(response);
+    if(length == 0)
+        CAMLreturnT(char **, 0);
+    completions = malloc((length + 1) * sizeof(char *));
+    if(completions == 0)
+        CAMLreturnT(char **, 0);
+    for(i = 0; i != length; i++) {
+        namep = strdup(String_val(Field(response, i)));
+        if(namep == 0)
+            break;
+        completions[i] = namep;
     }
-    return 0;
-}
-
-/*
- * Check if a file is a directory.
- */
-#ifdef WIN32
-static int readline_is_dir(struct dirent *entryp)
-{
-    return entryp->d_isdir;
-}
-
-#else /* WIN32 */
-
-static int readline_is_dir(const char *name)
-{
-    struct stat buf;
-    int code;
-
-#ifdef READLINE_GNU
-    if(rl_completion_mark_symlink_dirs)
-        code = stat(name, &buf);
-    else
-        code = lstat(name, &buf);
-#else /* READLINE_GNU */
-    code = stat(name, &buf);
-#endif /* !READLINE_GNU */
-    return code == 0 && S_ISDIR(buf.st_mode);
-}
-#endif /* !WIN32 */
-
-/*
- * Free previous completions.
- */
-static void readline_free_completions(CompletionInfo *infop)
-{
-    Completion *comp, *nextp;
-
-    for(comp = infop->completions; comp; comp = nextp) {
-        free(comp->name);
-        nextp = comp->next;
-        free(comp);
-    }
-    infop->completions = 0;
-}
-
-/*
- * Read the value to be completed.
- */
-static void readline_load_completion(CompletionInfo *infop, const char *text)
-{
-    /* The absolute pathname of the directory we are searching in */
-    char completion_dir[2 * MAX_FILENAME_LENGTH + 1];
-
-    /* The filename prefix we are looking for */
-    char completion_name[MAX_FILENAME_LENGTH];
-
-    /* The directory prefix for the completion */
-    char completion_buffer[2 * MAX_FILENAME_LENGTH + 1];
-    int buffer_offset;
-
-#ifndef WIN32
-    /* The absolute pathname prefix for directory checking */
-    char completion_absname[2 * MAX_FILENAME_LENGTH + 1];
-    int absname_offset;
-#endif /* !WIN32 */
-
-    /* Various other junk */
-    struct dirent *entryp;
-    const char *slashp;
-    Completion *comp;
-    char *namep;
-    int i, len;
-    DIR *dirp;
-
-    /* Get the directory and prefix */
-    slashp = last_slash(text);
-    if(is_absolute_name(text)) {
-        i = slashp - text + 1;
-        memcpy(completion_dir, text, i);
-        completion_dir[i] = 0;
-        strcpy(completion_name, slashp + 1);
-        strcpy(completion_buffer, completion_dir);
-        buffer_offset = i;
-    }
-    else {
-        len = strlen(infop->dir);
-        memcpy(completion_dir, infop->dir, len);
-        completion_dir[len] = '/';
-        if(slashp) {
-            i = slashp - text + 1;
-            memcpy(completion_dir + len + 1, text, i);
-            completion_dir[len + i + 1] = 0;
-            strcpy(completion_name, text + i);
-            memcpy(completion_buffer, text, i);
-            buffer_offset = i;
-        }
-        else {
-            completion_dir[len + 1] = 0;
-            strcpy(completion_name, text);
-            completion_buffer[0] = 0;
-            buffer_offset = 0;
-        }
-    }
-
-#ifndef WIN32
-    /* Absolute prefix */
-    absname_offset = strlen(completion_dir);
-    memcpy(completion_absname, completion_dir, absname_offset);
-#endif /* !WIN32 */
-
-    /* Read the entries from the directory */
-    dirp = opendir(completion_dir);
-    if(dirp == 0)
-        return;
-
-    len = strlen(completion_name);
-    while((entryp = readdir(dirp)) != NULL) {
-        if(strncmp(completion_name, entryp->d_name, len) == 0) {
-            /* Allocate the completion */
-            strcpy(completion_buffer + buffer_offset, entryp->d_name);
-#ifdef WIN32
-            if(readline_is_dir(entryp))
-                strcat(completion_buffer, "\\");
-#else /* WIN32 */
-            strcpy(completion_absname + absname_offset, entryp->d_name);
-            if(readline_is_dir(completion_absname))
-                strcat(completion_buffer, "/");
-#endif /* !WIN32 */
-            namep = strdup(completion_buffer);
-            if(namep == 0)
-                break;
-
-            /* Found a completion */
-            comp = (Completion *) malloc(sizeof(Completion));
-            if(comp == 0) {
-                free(namep);
-                break;
-            }
-            comp->next = infop->completions;
-            comp->name = namep;
-            infop->completions = comp;
-        }
-    }
-
-    closedir(dirp);
+    completions[i] = 0;
+    CAMLreturnT(char **, completions);
 }
 
 /************************************************************************
@@ -376,7 +187,7 @@ static void readline_load_completion(CompletionInfo *infop, const char *text)
  * and the line terminator, and the string terminator.
  */
 typedef struct _line_buffer {
-    char buffer[MAX_LINE_LENGTH + 3];
+    char buffer[LINE_MAX + 3];
     int length, index;
 } LineBuffer;
 
@@ -389,7 +200,7 @@ typedef struct _readline {
     int length;                         // Number of lines in the history
     int index;                          // Current line in the history
     int offset;                         // Base history offset
-    char filename[MAX_FILENAME_LENGTH]; // Where is the history saved?
+    char filename[PATH_MAX]; // Where is the history saved?
 
     /* Input processing */
     int escape;                         // Should next char be escaped?
@@ -521,7 +332,7 @@ static void LineInsertChar(LineBuffer *linep, char c)
 {
     int amount;
 
-    if(linep->index == MAX_LINE_LENGTH)
+    if(linep->index == LINE_MAX)
         return;
 
     /* Insert it */
@@ -732,8 +543,8 @@ static void UpdateCursor(ReadLine *readp, int index)
     }
 
     /* Set the cursor */
-    pos.X = x;
-    pos.Y = y;
+    pos.X = (SHORT) x;
+    pos.Y = (SHORT) y;
     SetConsoleCursorPosition(readp->console_out, pos);
 }
 
@@ -909,8 +720,8 @@ static ProcessCode processor(ReadLine *readp)
     INPUT_RECORD *event;
     KEY_EVENT_RECORD *key;
     ProcessCode code;
-    int i, status;
-    DWORD count;
+    int status;
+    DWORD count, i;
     char c;
 
     /* Input loop */
@@ -956,7 +767,7 @@ static void readline_cooked(ReadLine *readp)
     char *s;
 
     enter_blocking_section();
-    s = fgets(readp->current.buffer, MAX_LINE_LENGTH, stdin);
+    s = fgets(readp->current.buffer, LINE_MAX, stdin);
     leave_blocking_section();
     if(s == 0)
         raise_end_of_file();
@@ -968,8 +779,6 @@ static void readline_cooked(ReadLine *readp)
  */
 static void readline_raw(ReadLine *readp)
 {
-    LineBuffer *linep;
-
     /* Process in raw mode */
     ConsoleRaw(readp);
     processor(readp);
@@ -979,7 +788,7 @@ static void readline_raw(ReadLine *readp)
 /*
  * Once reading is done, reset all the buffers.
  */
-static readline_done(ReadLine *readp)
+static void readline_done(ReadLine *readp)
 {
     LineBuffer *linep;
     char *strp;
@@ -1034,7 +843,6 @@ static readline_done(ReadLine *readp)
  */
 static void readline(ReadLine *readp, const char *promptp)
 {
-    LineBuffer *linep;
     int len;
 
     /* Get the prompt */
@@ -1066,13 +874,12 @@ static void readline(ReadLine *readp, const char *promptp)
  */
 static int do_readline_load_history(ReadLine *readp, const char *filenamep)
 {
-    char line[MAX_LINE_LENGTH + 32];
+    char line[LINE_MAX + 32];
     LineBuffer *linep;
-    int i, offset;
     FILE *filep;
     char *strp;
 
-    if(strlen(filenamep) >= MAX_FILENAME_LENGTH)
+    if(strlen(filenamep) >= PATH_MAX)
         return -1;
     strcpy(readp->filename, filenamep);
     if((filep = fopen(filenamep, "r")) == NULL)
@@ -1093,8 +900,8 @@ static int do_readline_load_history(ReadLine *readp, const char *filenamep)
         linep->length = strlen(strp);
         if(linep->length == 0)
             continue;
-        if(linep->length > MAX_LINE_LENGTH)
-            linep->length = MAX_LINE_LENGTH;
+        if(linep->length > LINE_MAX)
+            linep->length = LINE_MAX;
         else
             linep->length--;
         memcpy(linep->buffer, strp, linep->length);
@@ -1325,10 +1132,19 @@ value omake_readline_init(value v_unit)
     return Val_unit;
 }
 
+#else /* !WIN32 */
+
 /************************************************************************
  * Unix readline interface.
  */
-#else /* !WIN32 */
+
+#ifdef RL_PROMPT_START_IGNORE
+#define INVIS_START RL_PROMPT_START_IGNORE
+#define INVIS_END   RL_PROMPT_END_IGNORE
+#else
+#define INVIS_START '\001'
+#define INVIS_END   '\002'
+#endif
 
 /*
  * Input state.
@@ -1339,16 +1155,18 @@ typedef struct _readline {
     int is_console;
 
     /* Prompts */
-    char prompt[MAX_PROMPT_LENGTH];
+    char * prompt;
+    int prompt_size;
     int force_prompt;
 
     /* Buffered data */
-    char buffer[MAX_LINE_LENGTH];
+    char * buffer;
+    int buffer_size;
     int index;
     int length;
 
-    /* Completion info */
-    CompletionInfo completions;
+    /* Current directory */
+    char dir[PATH_MAX];
 } ReadLine;
 
 /*
@@ -1360,14 +1178,24 @@ static ReadLine *AllocReadLine(int is_console, int console_in)
 
     /* Allocate */
     readp = (ReadLine *) malloc(sizeof(ReadLine));
-    if(readp == 0)
+    if(readp == NULL)
         failwith("AllocReadLine: out of memory");
     memset(readp, 0, sizeof(ReadLine));
 
     /* Initialize */
+    readp->buffer = malloc(LINE_MAX);
+    if (readp->buffer == NULL)
+        failwith("AllocReadLine: out of memory");
+    readp->buffer_size = LINE_MAX;
+
+    readp->prompt = malloc(MAX_PROMPT_LENGTH);
+    if (readp->prompt == NULL)
+        failwith("AllocReadLine: out of memory");
+    readp->prompt_size = MAX_PROMPT_LENGTH;
+
     readp->console_in = console_in;
     readp->is_console = is_console;
-    readp->completions.dir[0] = '/';
+    readp->dir[0] = '/';
     return readp;
 }
 
@@ -1380,7 +1208,7 @@ static void readline_cooked(ReadLine *readp)
         printf("%s", readp->prompt);
         fflush(stdout);
     }
-    if(fgets(readp->buffer, MAX_LINE_LENGTH, stdin) == NULL)
+    if(fgets(readp->buffer, readp->buffer_size, stdin) == NULL)
         readp->length = 0;
     else
         readp->length = strlen(readp->buffer);
@@ -1392,7 +1220,7 @@ static void readline_cooked(ReadLine *readp)
 #ifdef READLINE_ENABLED
 static void readline_raw(ReadLine *readp)
 {
-    char *linep, *expansion;
+    char *linep, *expansion = NULL;
     int length, result;
 
   prompt:
@@ -1431,8 +1259,14 @@ static void readline_raw(ReadLine *readp)
         /* Successful expansion */
         add_history(expansion);
         length = strlen(expansion);
-        if(length > MAX_LINE_LENGTH)
-            length = MAX_LINE_LENGTH;
+        if(length >= readp->buffer_size) {
+            char *new_buffer = malloc(length + 1);
+            if(new_buffer == NULL)
+                failwith("readline_raw: out of memory");
+            free(readp->buffer);
+            readp->buffer = new_buffer;
+            readp->buffer_size = length + 1;
+        }
         memcpy(readp->buffer, expansion, length);
         readp->buffer[length] = '\n';
         readp->length = length + 1;
@@ -1455,14 +1289,52 @@ static void readline_raw(ReadLine *readp)
  */
 static void do_readline(ReadLine *readp, const char *promptp)
 {
-    int c, i;
+    int i;
 
     /* Get the prompt */
-    for(i = 0; i < sizeof(readp->prompt) - 1; i++) {
-        c = *promptp++;
-        if(c == 0)
-            break;
-        readp->prompt[i] = c;
+    int is_vis = 1, vis_length = 0;
+    int last_vis[3] = {0,0,0}, vis_ind = 0;
+#ifdef RL_PROMPT_START_IGNORE
+    int copy_markers = readp -> is_console;
+#else
+#define copy_markers 0
+#endif
+    for(i = 0; *promptp; promptp++) {
+        if (i == readp->prompt_size - 1) {
+            /* Ran out of memory, allocate bigger buffer */
+            char * old_prompt = readp->prompt;
+            int new_size = readp->prompt_size * 2;
+            readp->prompt = malloc(new_size);
+            if (readp->prompt == NULL) {
+                readp->prompt = old_prompt;
+                failwith("do_readline: out of memory");
+            } else {
+                memcpy(readp->prompt, old_prompt, i);
+                free(old_prompt);
+                readp->prompt_size = new_size;
+            }
+        } 
+        if (*promptp == INVIS_START)
+            is_vis = 0;
+        if (is_vis) {
+            vis_length++;
+            if (vis_length == MAX_PROMPT_LENGTH + 1) {
+                /* We just ran out */
+                readp->prompt[last_vis[0]] = '.';
+                readp->prompt[last_vis[1]] = '.';
+                readp->prompt[last_vis[2]] = '.';
+            }
+        }
+
+        if (((!is_vis) || (vis_length <= MAX_PROMPT_LENGTH))
+                && (copy_markers
+                    || ((*promptp != INVIS_START) && (*promptp != INVIS_END)))) {
+            if (is_vis)
+                last_vis[vis_ind++ % 3] = i;
+            readp->prompt[i++] = *promptp;
+        }
+        if (*promptp == INVIS_END)
+            is_vis = 1;
     }
     readp->prompt[i] = 0;
 
@@ -1521,52 +1393,26 @@ static ReadLine *readp;
 
 #ifdef READLINE_ENABLED
 /*
- * Return the next completion.
+ * Flatten the list.
  */
-static char *readline_next_completion(ReadLine *readp)
+static char **readline_completion_matches(const char *text, int first, int last)
 {
-    CompletionInfo *infop;
-    Completion *comp;
-    char *namep;
+    char **matches;
+    int amount;
 
-    namep = 0;
-    infop = &readp->completions;
-    comp = infop->completions;
-    if(comp) {
-        namep = comp->name;
-        infop->completions = comp->next;
-        free(comp);
-    }
-    return namep;
+    /* Sanity checking */
+    amount = last - first;
+    if(amount <= 0 || amount >= LINE_MAX)
+        return NULL;
+
+    /* Three kinds of completion */
+    if(first == 0)
+        matches = readline_completion(omake_command_completion, text);
+    else
+        matches = readline_completion(omake_filename_completion, text);
+    return matches;
 }
 
-/*
- * Filename completion.
- */
-static char *readline_completion_function(const char *text, int not_first)
-{
-    /* Username expansion */
-    if(*text == '~') {
-#ifdef READLINE_GNU
-        return rl_username_completion_function(text, not_first);
-#else
-        return username_completion_function(text, not_first);
-#endif /* READLINE_GNU */
-    }
-
-    /* Fail if the name is already too long */
-    if(strlen(text) >= MAX_FILENAME_LENGTH)
-        return 0;
-
-    /* Read the completions from the filesystem */
-    if(not_first == 0) {
-        readline_free_completions(&readp->completions);
-        readline_load_completion(&readp->completions, text);
-    }
-
-    /* Now return the completions */
-    return readline_next_completion(readp);
-}
 #endif /* READLINE_ENABLED */
 
 /************************************************************************
@@ -1625,8 +1471,8 @@ value omake_readline_set_length(value v_length)
 value omake_readline_set_directory(value v_dir)
 {
     /* Copy the current directory */
-    strncpy(readp->completions.dir, String_val(v_dir), MAX_FILENAME_LENGTH - 1);
-    readp->completions.dir[MAX_FILENAME_LENGTH - 1] = 0;
+    strncpy(readp->dir, String_val(v_dir), PATH_MAX - 1);
+    readp->dir[PATH_MAX - 1] = 0;
     return Val_unit;
 }
 
@@ -1656,10 +1502,12 @@ value omake_readline_history(value v_unit)
      * history in one call.
      */
     for(i = 0; i < history_length; i++) {
-        HIST_ENTRY *entry;
+        HIST_ENTRY *entryp;
+        entryp = history_get(history_base + i);
+        if(entryp == 0)
+            break;
         data[i] = entries[i];
-        entry = history_get(i + history_base);
-        sprintf(data[i], "%-5d %.70s", i + history_base, entry->line);
+        sprintf(data[i], "%-5d %.70s", i + history_base, entryp->line);
         data[i][79] = 0;
     }
     data[i] = 0;
@@ -1742,16 +1590,41 @@ value omake_readline_init(value v_unit)
     readp = AllocReadLine(isatty(0), 0);
 #ifdef READLINE_ENABLED
     using_history();
-#ifdef READLINE_GNU
-    /* This is annoying.  The BSD version of rl_completion_entry_function
-     * is a function that returns an int.  The GNU version returns a char*.
-     * There are no docs for the BSD version, so for now there's no
-     * filename completion without GNU readline. */
-    rl_completion_entry_function = readline_completion_function;
+    rl_attempted_completion_function = readline_completion_matches;
     rl_completion_append_character = 0;
-#endif
 #endif
     return Val_unit;
 }
 
 #endif /* !WIN32 */
+
+value omake_rl_prompt_wrappers(value v_unit) {
+    CAMLparam1(v_unit);
+    CAMLlocal1(buf);
+
+#ifdef INVIS_START
+    {
+        char begin[2] = { INVIS_START, 0};
+        char end  [2] = { INVIS_END, 0};
+        CAMLlocal2(s1, s2);
+        s1 = caml_copy_string(begin);
+        s2 = caml_copy_string(end);
+        buf = alloc_tuple(2);
+        Field(buf, 0) = s1;
+        Field(buf, 1) = s2;
+    }
+#else /* INVIS_START */
+    {
+        CAMLlocal1(emptystr);
+        emptystr = caml_copy_string("");
+        buf = alloc_tuple(2);
+        Field(buf, 0) = emptystr;
+        Field(buf, 1) = emptystr;
+    }
+#endif /* INVIS_START */
+    CAMLreturn(buf);
+}
+
+/*
+ * vim:tw=100:ts=4:et:sw=4:cin
+ */
