@@ -9,22 +9,23 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2005-2007 Mojave Group, Caltech
+ * Copyright (C) 2005-2007 Mojave Group, California Insitute of Technology and
+ * HRL Laboratories, LLC
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -36,12 +37,16 @@
 open Lm_printf
 open Lm_symbol
 open Lm_string_set
+open Lm_hash_sig
 open Lm_hash
 
 open Omake_ir
 open Omake_env
+open Omake_pos
 open Omake_node
 open Omake_shell_type
+open Omake_value_type
+open Omake_value_print
 open Omake_command_type
 
 (*
@@ -56,6 +61,7 @@ type code =
  | CodeArrayString
  | CodeArrow
  | CodeBegin
+ | CodeObjectString
  | CodeBodyString
  | CodeCase
  | CodeCaseExp
@@ -68,6 +74,7 @@ type code =
  | CodeCommands
  | CodeNoneString
  | CodeConstString
+ | CodeVarString
  | CodeEagerApply
  | CodeEnd
  | CodeExpString
@@ -85,16 +92,26 @@ type code =
  | CodeQuoteString
  | CodeQuoteStringString
  | CodeExportExp
+ | CodeExportNone
+ | CodeExportAll
+ | CodeExportList
+ | CodeExportRules
+ | CodeExportPhonies
+ | CodeExportVar
  | CodeCancelExportExp
- | CodeReturnCatchExp
+ | CodeReturnBodyExp
  | CodeStringExp
  | CodeReturnExp
  | CodeReturnObjectExp
  | CodeReturnSaveExp
- | CodeScopeDynamic
- | CodeScopeGlobal
- | CodeScopePrivate
- | CodeScopeProtected
+ | CodeVarScopePrivate
+ | CodeVarScopeThis
+ | CodeVarScopeVirtual
+ | CodeVarScopeGlobal
+ | CodeVarPrivate
+ | CodeVarThis
+ | CodeVarVirtual
+ | CodeVarGlobal
  | CodeSectionExp
  | CodeSequenceExp
  | CodeSequenceString
@@ -111,7 +128,6 @@ type code =
  | CodeValDir
  | CodeValFloat
  | CodeValFun
- | CodeValFunValue
  | CodeValInt
  | CodeValMap
  | CodeValMethodApply
@@ -123,14 +139,16 @@ type code =
  | CodeValQuoteString
  | CodeValSequence
  | CodeValString
- | CodeValImplicit
+ | CodeValVar
+ | CodeValMaybeApply
  | CodeValSuperApply
  | CodeVarDefApply
  | CodeVarDefNormal
  | CodeLetKeyExp
- | CodeKeyString
+ | CodeKeyApplyString
  | CodeKeyExp
- | CodeValKey
+ | CodeValKeyApply
+ | CodeValStaticApply
  | CodeArg
  | CodeArgString
  | CodeArgData
@@ -158,6 +176,8 @@ type code =
  | CodeRedirectNode
  | CodeRedirectArg
  | CodeRedirectNone
+   (* ZZZ: temporary fix, don't propagate to keyword *)
+ | CodeValFunValue
 (* %%MAGICEND%% *)
 
 module type HashSig =
@@ -198,6 +218,25 @@ let rec squash_vars buf vars =
     | [] ->
          ()
 
+let squash_var_info buf v =
+   match v with
+      VarPrivate (_, v) ->
+         Hash.add_code buf CodeVarPrivate;
+         Hash.add_code buf CodeSpace;
+         squash_var buf v
+    | VarThis (_, v) ->
+         Hash.add_code buf CodeVarThis;
+         Hash.add_code buf CodeSpace;
+         squash_var buf v
+    | VarVirtual (_, v) ->
+         Hash.add_code buf CodeVarVirtual;
+         Hash.add_code buf CodeSpace;
+         squash_var buf v
+    | VarGlobal (_, v) ->
+         Hash.add_code buf CodeVarGlobal;
+         Hash.add_code buf CodeSpace;
+         squash_var buf v
+
 (*
  * File.
  *)
@@ -207,16 +246,15 @@ let squash_node buf node =
 (*
  * String representations.
  *)
-let squash_scope buf scope =
-   match scope with
-      ScopeGlobal ->
-         Hash.add_code buf CodeScopeGlobal
-    | ScopeProtected ->
-         Hash.add_code buf CodeScopeProtected
-    | ScopeDynamic ->
-         Hash.add_code buf CodeScopeDynamic
-    | ScopePrivate ->
-         Hash.add_code buf CodeScopePrivate
+let squash_var_scope buf scope =
+   let code =
+      match scope with
+         VarScopePrivate -> CodeVarScopePrivate
+       | VarScopeThis    -> CodeVarScopeThis
+       | VarScopeVirtual -> CodeVarScopeVirtual
+       | VarScopeGlobal  -> CodeVarScopeGlobal
+   in
+      Hash.add_code buf code
 
 let squash_strategy buf strategy =
    let s =
@@ -240,6 +278,27 @@ let squash_def_kind buf kind =
    in
       Hash.add_code buf s
 
+(*
+ * Export info.
+ *)
+let squash_export_info buf info =
+   match info with
+      Omake_ir.ExportNone ->
+         Hash.add_code buf CodeExportNone
+    | Omake_ir.ExportAll ->
+         Hash.add_code buf CodeExportAll
+    | Omake_ir.ExportList items ->
+         Hash.add_code buf CodeExportList;
+         List.iter (fun item ->
+               match item with
+                  Omake_ir.ExportRules ->
+                     Hash.add_code buf CodeExportRules
+                | Omake_ir.ExportPhonies ->
+                     Hash.add_code buf CodeExportPhonies
+                | Omake_ir.ExportVar v ->
+                     Hash.add_code buf CodeExportVar;
+                     squash_var_info buf v) items
+
 (*
  * Squash string expressions.
  *)
@@ -252,35 +311,34 @@ let rec squash_string_exp pos buf e =
        | ConstString (_, s) ->
             Hash.add_code buf CodeConstString;
             Hash.add_string buf s
-       | KeyString (_, strategy, s) ->
-            Hash.add_code buf CodeKeyString;
+       | VarString (_, v) ->
+            Hash.add_code buf CodeVarString;
+            squash_var_info buf v
+       | KeyApplyString (_, strategy, s) ->
+            Hash.add_code buf CodeKeyApplyString;
             squash_strategy buf strategy;
             Hash.add_string buf s
-       | ApplyString (_, strategy, scope, v, sl) ->
+       | ApplyString (_, strategy, v, sl) ->
             Hash.add_code buf CodeApplyString;
             squash_strategy buf strategy;
             Hash.add_code buf CodeSpace;
-            squash_scope buf scope;
-            Hash.add_code buf CodeSpace;
-            squash_var buf v;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
             squash_string_exp_list pos buf sl
-       | SuperApplyString (_, strategy, scope, v1, v2, sl) ->
+       | SuperApplyString (_, strategy, v1, v2, sl) ->
             Hash.add_code buf CodeSuperApplyString;
             squash_strategy buf strategy;
             Hash.add_code buf CodeSpace;
-            squash_scope buf scope;
-            Hash.add_code buf CodeSpace;
             squash_var buf v1;
             Hash.add_code buf CodeSpace;
             squash_var buf v2;
             Hash.add_code buf CodeSpace;
             squash_string_exp_list pos buf sl
-       | MethodApplyString (_, strategy, scope, vars, sl) ->
+       | MethodApplyString (_, strategy, v, vars, sl) ->
             Hash.add_code buf CodeMethodApplyString;
             squash_strategy buf strategy;
             Hash.add_code buf CodeSpace;
-            squash_scope buf scope;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
             squash_vars buf vars;
             Hash.add_code buf CodeSpace;
@@ -301,18 +359,26 @@ let rec squash_string_exp pos buf e =
             Hash.add_code buf CodeQuoteStringString;
             Hash.add_char buf c;
             squash_string_exp_list pos buf sl
-       | BodyString (_, e) ->
+       | ObjectString (_, el, export) ->
+            Hash.add_code buf CodeObjectString;
+            squash_exp_list pos buf el;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
+       | BodyString (_, el, export) ->
             Hash.add_code buf CodeBodyString;
-            squash_exp pos buf e
-       | ExpString (_, e) ->
+            squash_exp_list pos buf el;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
+       | ExpString (_, el, export) ->
             Hash.add_code buf CodeExpString;
-            squash_exp pos buf e
+            squash_exp_list pos buf el;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
        | CasesString (_, cases) ->
             Hash.add_code buf CodeCasesString;
             squash_cases_exp pos buf cases
-       | ThisString (_, scope) ->
-            Hash.add_code buf CodeThisString;
-            squash_scope buf scope
+       | ThisString _ ->
+            Hash.add_code buf CodeThisString
    end;
    Hash.add_code buf CodeEnd
 
@@ -327,13 +393,15 @@ and squash_string_exp_list pos buf sl =
     | [] ->
          ()
 
-and squash_case_exp pos buf (v, s, e) =
+and squash_case_exp pos buf (v, s, el, export) =
    Hash.add_code buf CodeCaseString;
    squash_var buf v;
    Hash.add_code buf CodeSpace;
    squash_string_exp pos buf s;
    Hash.add_code buf CodeSpace;
-   squash_exp pos buf e;
+   squash_exp_list pos buf el;
+   Hash.add_code buf CodeSpace;
+   squash_export_info buf export;
    Hash.add_code buf CodeEnd
 
 and squash_cases_exp pos buf cases =
@@ -348,11 +416,11 @@ and squash_exp pos buf e =
    Hash.add_code buf CodeBegin;
    begin
       match e with
-         LetVarExp (_, scope, v, def, s) ->
+         LetVarExp (_, v, vl, def, s) ->
             Hash.add_code buf CodeLetVarExp;
-            squash_scope buf scope;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
-            squash_var buf v;
+            squash_vars buf vl;
             Hash.add_code buf CodeSpace;
             squash_def_kind buf def;
             Hash.add_code buf CodeSpace;
@@ -367,22 +435,28 @@ and squash_exp pos buf e =
             squash_def_kind buf def;
             Hash.add_code buf CodeSpace;
             squash_string_exp pos buf s
-       | LetFunExp (_, scope, v, params, s) ->
+       | LetFunExp (_, v, vl, params, s, export) ->
             Hash.add_code buf CodeLetFunExp;
-            squash_scope buf scope;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
-            squash_var buf v;
+            squash_vars buf vl;
             Hash.add_code buf CodeSpace;
             squash_vars buf params;
             Hash.add_code buf CodeSpace;
-            squash_exp pos buf s
-       | LetObjectExp (_, scope, v, el) ->
+            squash_exp_list pos buf s;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
+       | LetObjectExp (_, v, vl, s, el, export) ->
             Hash.add_code buf CodeLetObjectExp;
-            squash_scope buf scope;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
-            squash_var buf v;
+            squash_vars buf vl;
             Hash.add_code buf CodeSpace;
-            squash_exp_list pos buf el
+            squash_string_exp pos buf s;
+            Hash.add_code buf CodeSpace;
+            squash_exp_list pos buf el;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
        | LetThisExp (_, s) ->
             Hash.add_code buf CodeLetThisExp;
             squash_string_exp pos buf s
@@ -395,11 +469,13 @@ and squash_exp pos buf e =
        | SequenceExp (_, el) ->
             Hash.add_code buf CodeSequenceExp;
             squash_exp_list pos buf el
-       | SectionExp (_, s, el) ->
+       | SectionExp (_, s, el, export) ->
             Hash.add_code buf CodeSectionExp;
             squash_string_exp pos buf s;
             Hash.add_code buf CodeArrow;
-            squash_exp_list pos buf el
+            squash_exp_list pos buf el;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
        | OpenExp (_, nodes) ->
             Hash.add_code buf CodeOpenExp;
             List.iter (fun node ->
@@ -410,37 +486,28 @@ and squash_exp pos buf e =
             squash_string_exp pos buf s;
             Hash.add_code buf CodeCommaExp;
             squash_string_exp_list pos buf sl
-       | ApplyExp (_, scope, v, sl) ->
+       | ApplyExp (_, v, sl) ->
             Hash.add_code buf CodeApplyExp;
-            squash_scope buf scope;
-            Hash.add_code buf CodeSpace;
-            squash_var buf v;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
             squash_string_exp_list pos buf sl
-       | SuperApplyExp (_, scope, v1, v2, sl) ->
+       | SuperApplyExp (_, v1, v2, sl) ->
             Hash.add_code buf CodeSuperApplyExp;
-            squash_scope buf scope;
-            Hash.add_code buf CodeSpace;
             squash_var buf v1;
             Hash.add_code buf CodeSpace;
             squash_var buf v2;
             Hash.add_code buf CodeSpace;
             squash_string_exp_list pos buf sl
-       | MethodApplyExp (_, scope, vars, sl) ->
+       | MethodApplyExp (_, v, vars, sl) ->
             Hash.add_code buf CodeMethodApplyExp;
-            squash_scope buf scope;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
             squash_vars buf vars;
             Hash.add_code buf CodeSpace;
             squash_string_exp_list pos buf sl
-       | ExportExp (_, s) ->
-            Hash.add_code buf CodeExportExp;
-            squash_string_exp pos buf s
-       | CancelExportExp _ ->
-            Hash.add_code buf CodeCancelExportExp
-       | ReturnCatchExp (_, e) ->
-            Hash.add_code buf CodeReturnCatchExp;
-            squash_exp pos buf e
+       | ReturnBodyExp (_, el) ->
+            Hash.add_code buf CodeReturnBodyExp;
+            squash_exp_list pos buf el
        | StringExp (_, s) ->
             Hash.add_code buf CodeStringExp;
             squash_string_exp pos buf s
@@ -473,11 +540,13 @@ and squash_exp_list pos buf el =
     | [] ->
          ()
 
-and squash_if_case pos buf (s, e) =
+and squash_if_case pos buf (s, el, export) =
    Hash.add_code buf CodeCaseExp;
    squash_string_exp pos buf s;
    Hash.add_code buf CodeSpace;
-   squash_exp pos buf e;
+   squash_exp_list pos buf el;
+   Hash.add_code buf CodeSpace;
+   squash_export_info buf export;
    Hash.add_code buf CodeEnd
 
 and squash_if_cases pos buf cases =
@@ -519,39 +588,35 @@ let rec squash_value pos buf v =
             Hash.add_code buf CodeValQuoteString;
             Hash.add_char buf c;
             squash_values pos buf vl
-       | ValApply (_, scope, v, vl) ->
+       | ValApply (_, v, vl) ->
             Hash.add_code buf CodeValApply;
-            squash_scope buf scope;
-            Hash.add_code buf CodeSpace;
-            squash_var buf v;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
             squash_values pos buf vl
-       | ValImplicit (_, scope, v) ->
-            Hash.add_code buf CodeValImplicit;
-            squash_scope buf scope;
-            Hash.add_code buf CodeSpace;
-            squash_var buf v;
-       | ValSuperApply (_, scope, v1, v2, vl) ->
+       | ValMaybeApply (_, v) ->
+            Hash.add_code buf CodeValMaybeApply;
+            squash_var_info buf v;
+       | ValSuperApply (_, v1, v2, vl) ->
             Hash.add_code buf CodeValSuperApply;
-            squash_scope buf scope;
-            Hash.add_code buf CodeSpace;
             squash_var buf v1;
             Hash.add_code buf CodeSpace;
             squash_var buf v2;
             Hash.add_code buf CodeSpace;
             squash_values pos buf vl
-       | ValMethodApply (_, scope, vars, vl) ->
+       | ValMethodApply (_, v, vars, vl) ->
             Hash.add_code buf CodeValMethodApply;
-            squash_scope buf scope;
+            squash_var_info buf v;
             Hash.add_code buf CodeSpace;
             squash_vars buf vars;
             Hash.add_code buf CodeSpace;
             squash_values pos buf vl
-       | ValFun (_, _, params, body) ->
+       | ValFun (_, _, params, body, export) ->
             Hash.add_code buf CodeValFun;
             squash_vars buf params;
             Hash.add_code buf CodeArrow;
-            squash_exp pos buf body
+            squash_exp_list pos buf body;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
        | ValFunValue (_, _, params, body) ->
             Hash.add_code buf CodeValFunValue;
             squash_vars buf params;
@@ -566,9 +631,11 @@ let rec squash_value pos buf v =
        | ValDir dir ->
             Hash.add_code buf CodeValDir;
             Hash.add_string buf (Dir.fullname dir)
-       | ValBody (_, e) ->
+       | ValBody (_, e, export) ->
             Hash.add_code buf CodeValBody;
-            squash_exp pos buf e
+            squash_exp_list pos buf e;
+            Hash.add_code buf CodeSpace;
+            squash_export_info buf export
        | ValObject obj ->
             Hash.add_code buf CodeValObject;
             squash_object pos buf obj
@@ -577,10 +644,19 @@ let rec squash_value pos buf v =
             squash_map pos buf obj
        | ValCases cases ->
             squash_cases pos buf cases
-       | ValKey (_, v) ->
-            Hash.add_code buf CodeValKey;
+       | ValKeyApply (_, v) ->
+            Hash.add_code buf CodeValKeyApply;
             Hash.add_string buf v
-       | ValEnv _
+       | ValVar (_, v) ->
+            Hash.add_code buf CodeValVar;
+            squash_var_info buf v;
+       | ValDelayed { contents = ValValue v } ->
+            squash_value pos buf v
+       | ValDelayed { contents = ValStaticApply (node, v) } ->
+            Hash.add_code buf CodeValStaticApply;
+            squash_value pos buf node;
+            Hash.add_code buf CodeSpace;
+            squash_var buf v
        | ValRules _
        | ValChannel _
        | ValClass _
@@ -619,13 +695,15 @@ and squash_map pos buf map =
          squash_value pos buf v;
          Hash.add_code buf CodeEnd) map
 
-and squash_case pos buf (x, v1, x2) =
+and squash_case pos buf (x, v1, x2, export) =
    Hash.add_code buf CodeCase;
    squash_var buf x;
    Hash.add_code buf CodeSpace;
    squash_value pos buf v1;
    Hash.add_code buf CodeSpace;
    squash_value pos buf v1;
+   Hash.add_code buf CodeSpace;
+   squash_export_info buf export;
    Hash.add_code buf CodeEnd
 
 and squash_cases pos buf cases =
@@ -812,7 +890,7 @@ let squash_command_line pos buf (command : arg_command_inst) =
          squash_pipe pos buf argv;
          Hash.add_code buf CodeEnd
     | CommandEval e ->
-         squash_exp pos buf e
+         squash_exp_list pos buf e
     | CommandValues values ->
          squash_values pos buf values
 
@@ -834,6 +912,13 @@ let squash_commands pos buf commands =
 (*
  * Get the digest of some commands.
  *)
+let digest_of_exp pos values e =
+   let buf = Hash.create () in
+      squash_values pos buf values;
+      Hash.add_code buf CodeSpace;
+      squash_exp pos buf e;
+      Some (Hash.digest buf)
+
 let digest_of_commands pos commands =
    match commands with
       [] ->
