@@ -4,7 +4,8 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2003-2007 Mojave Group, Caltech
+ * Copyright (C) 2003-2007 Mojave Group, California Institute of Technology, and
+ * HRL Laboratories, LLC
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +26,7 @@
  * linked executables.  See the file LICENSE.OMake for more details.
  *
  * Author: Jason Hickey @email{jyh@cs.caltech.edu}
- * Modified By: Aleksey Nogin @email{nogin@metaprl.org}
+ * Modified By: Aleksey Nogin @email{nogin@metaprl.org}, @email{anogin@hrl.com}
  * @end[license]
  *)
 open Lm_printf
@@ -36,17 +37,21 @@ open Lm_location
 open Lm_symbol
 
 open Omake_ir
+open Omake_pos
+open Omake_var
 open Omake_node
 open Omake_exec
 open Omake_cache
 open Omake_lexer
 open Omake_parser
+open Omake_options
 open Omake_node_sig
 open Omake_exec_type
 open Omake_shell_type
-open Omake_options
+open Omake_value_type
 open Omake_command_type
 open Omake_ir_free_vars
+open Omake_handle_table
 
 (*
  * Debugging.
@@ -54,146 +59,18 @@ open Omake_ir_free_vars
 val debug_scanner  : bool ref
 val debug_implicit : bool ref
 
-(*
- * A source string.
- *)
-type 'a source = node_kind * 'a
-
-(*
- * A target value that represents a node in a rule.
- *)
-type target =
-   TargetNode of Node.t
- | TargetString of string
-
-(*
- * Type of objects.
- *)
-type obj
-
-(*
- * Type of maps.
- *)
-type map
-
-(*
- * Static scopes.
- *)
-type env
-
 (*
  * Type of environments.
  *)
 type venv
 
-(*
- * Primitive values.
- *)
-type prim_fun
-
-(*
- * Exception positions.
- *)
-type pos
-
-(*
- * IoChannels.
- *)
-type prim_channel
-
-type channel_mode = Lm_channel.mode =
-   InChannel
- | OutChannel
- | InOutChannel
-
-(*
- * Kinds of rules.
- *)
-type rule_multiple =
-   RuleSingle
- | RuleMultiple
- | RuleScannerSingle
- | RuleScannerMultiple
-
-type rule_kind =
-   RuleNormal
- | RuleScanner
-
-(*
- * Export kinds.
- *    ExportFile: this is the result of file evaluation
- *    ExportList: export just the listed items (variable values, implicit rules/dependencies, phony target set)
- *    ExportAll: export the entire environment.
- *    ExportValue v: just like Exportall, but return the value too
- *)
-and export =
-   ExportFile
- | ExportAll
- | ExportDir
- | ExportList of export_elt list
- | ExportValue of value
-
-and export_elt =
-   ExportSymbol of symbol
- | ExportRules
- | ExportPhonies
-
-(*
- * Possible values.
- * For the function, the obj is the static scope.
- *)
-and value =
-   ValNone
- | ValInt         of int
- | ValFloat       of float
- | ValSequence    of value list
- | ValArray       of value list
- | ValString      of string
- | ValData        of string
- | ValQuote       of value list
- | ValQuoteString of char * value list
- | ValApply       of loc * scope_kind * var * value list
- | ValSuperApply  of loc * scope_kind * var * var * value list
- | ValMethodApply of loc * scope_kind * var list * value list
- | ValImplicit    of loc * scope_kind * var
- | ValFun         of arity * env * var list * exp
- | ValFunValue    of arity * env * var list * value
- | ValPrim        of arity * bool * prim_fun
- | ValRules       of erule list
- | ValNode        of Node.t
- | ValDir         of Dir.t
- | ValEnv         of venv * export
- | ValBody        of env * exp
- | ValObject      of obj
- | ValMap         of map
- | ValChannel     of channel_mode * prim_channel
- | ValClass       of obj SymbolTable.t
- | ValCases       of (var * value * value) list
- | ValKey         of loc * string
- | ValOther       of value_other
-
-(*
- * Put all the other stuff here, to keep the primary value type
- * smaller.
- *)
-and value_other =
-   ValLexer       of Lexer.t
- | ValParser      of Parser.t
- | ValLocation    of loc
- | ValPosition    of pos
- | ValExitCode    of int
-
 (*
  * Command lists are used for rule bodies.
  * They have their environment, a list of sources,
  * and the actual body.  The body is polymorphic
  * for various kinds of commands.
  *)
-and command =
-   CommandSection of value * free_vars * exp    (* Name of the section, its free variables, and the expression *)
- | CommandValue of loc * value
-
-and command_info =
+type command_info =
    { command_env       : venv;
      command_sources   : Node.t list;
      command_values    : value list;
@@ -231,6 +108,20 @@ and erule_info =
      explicit_directories     : venv DirTable.t
    }
 
+type srule =
+   { srule_loc      : loc;
+     srule_static   : bool;
+     srule_env      : venv;
+     srule_key      : value;
+     srule_deps     : NodeSet.t;
+     srule_vals     : value list;
+     srule_exp      : exp
+   }
+
+type static_info =
+   StaticRule of srule
+ | StaticValue of obj
+
 (*
  * Command lines.
  *)
@@ -240,7 +131,7 @@ and arg_command_line = (venv, exp, arg_pipe, value) poly_command_line
 and string_command_inst = (exp, string_pipe, value) poly_command_inst
 and string_command_line = (venv, exp, string_pipe, value) poly_command_line
 
-and apply        = venv -> Unix.file_descr -> Unix.file_descr -> Unix.file_descr -> (symbol * string) list -> value list -> int * value
+and apply        = venv -> Unix.file_descr -> Unix.file_descr -> Unix.file_descr -> (symbol * string) list -> value list -> int * venv * value
 
 and value_cmd    = (unit, value list, value list) poly_cmd
 and value_apply  = (value list, value list, apply) poly_apply
@@ -273,7 +164,7 @@ type tok =
 type pid =
    InternalPid of int
  | ExternalPid of int
- | ResultPid of int * value
+ | ResultPid of int * venv * value
 
 type exec = (arg_command_line, pid, value) Exec.t
 
@@ -290,11 +181,6 @@ type include_scope =
    IncludePervasives
  | IncludeAll
 
-(*
- * Intermediate code include the names of the classes.
- *)
-type ir = symbol list * scope_kind SymbolTable.t * Omake_ir.exp
-
 (*
  * Check if command list does not contain anything to execute.
  *)
@@ -319,18 +205,20 @@ val venv_get_pervasives : venv -> Node.t -> venv
 (*
  * Variables in scope.
  *)
-val venv_include_scope : venv -> include_scope -> scope_kind SymbolTable.t
+val venv_include_scope : venv -> include_scope -> senv
 
 (*
  * Fork, so that a thread can work on a private copy in peace.
  *)
 val venv_fork       : venv -> venv
+val venv_unfork     : venv -> venv -> venv
 
 (*
  * Global values.
  *)
 val venv_exec       : venv -> exec
 val venv_cache      : venv -> Omake_cache.t
+val venv_add_cache  : venv -> Omake_cache.t -> venv
 
 (*
  * Add values to environment.
@@ -352,17 +240,18 @@ val venv_nodename         : venv -> Node.t -> string
 
 val venv_mount       : venv -> mount_option list -> Dir.t -> Dir.t -> venv
 
-val venv_add_var     : venv -> scope_kind -> pos -> var -> value -> venv
-val venv_add_args    : venv -> pos -> loc -> env -> var list -> value list -> venv
-val venv_add_assoc   : venv -> (var * string) list -> venv
+val venv_add_var     : venv -> var_info -> value -> venv
 val venv_add_phony   : venv -> loc -> target list -> venv
 
+val venv_add_args    : venv -> pos -> loc -> env -> var list -> value list -> venv
 val venv_add_args_hack : venv -> pos -> loc -> env -> var list -> value list -> venv
 
 val venv_add_wild_match  : venv -> value -> venv
+val venv_add_match_values : venv -> value list -> venv
 val venv_add_match_args  : venv -> string list -> venv
 val venv_add_match       : venv -> string -> string list -> venv
 val venv_explicit_target : venv -> Node.t -> venv
+val venv_explicit_find   : venv -> pos -> Node.t -> erule
 
 val venv_add_rule : venv -> pos -> loc ->
    rule_multiple ->                     (* multiple, scanner, etc *)
@@ -373,7 +262,20 @@ val venv_add_rule : venv -> pos -> loc ->
    target source list ->                (* scanners *)
    value list ->                        (* additional values the target depends on *)
    command list ->                      (* commands *)
-   venv * erule list
+   venv * Node.t list
+
+val venv_add_memo_rule : venv -> pos -> loc ->
+   bool ->                              (* multiple *)
+   bool ->                              (* static flag *)
+   value ->                             (* key *)
+   var_info list ->                     (* variables to be defined *)
+   target source list ->                (* sources *)
+   value list ->                        (* additional values the target depends on *)
+   exp ->                               (* commands *)
+   venv
+
+val venv_set_static_info  : venv -> value -> static_info -> unit
+val venv_find_static_info : venv -> pos -> value -> static_info
 
 (*
  * System environment.
@@ -387,18 +289,27 @@ val venv_defined_env : venv -> var -> bool
 (*
  * Handle options.
  *)
-val venv_options : venv -> omake_options
-val venv_set_options : venv -> loc -> pos -> string -> venv
+val venv_options          : venv -> omake_options
+val venv_set_options      : venv -> loc -> pos -> string list -> venv
+
+(*
+ * Values represented with handles.
+ *)
+val venv_add_environment      : venv -> handle_env
+val venv_find_environment     : venv -> pos -> handle_env -> venv
 
 (*
  * Find values.
  *)
-val venv_dir          : venv -> Dir.t
-val venv_defined      : venv -> scope_kind -> var -> bool
-val venv_get_var      : venv -> scope_kind -> pos -> var -> value
-val venv_find_var     : venv -> scope_kind -> pos -> loc -> var -> value
-val venv_find_var_exn : venv -> scope_kind -> var -> value
-val venv_find_object_or_empty : venv -> scope_kind -> symbol -> obj
+val venv_dir                  : venv -> Dir.t
+val venv_defined              : venv -> var_info -> bool
+val venv_defined_field        : venv -> obj -> var -> bool
+
+val venv_get_var              : venv -> pos -> var_info -> value
+
+val venv_find_var             : venv -> pos -> loc -> var_info -> value
+val venv_find_var_exn         : venv -> var_info -> value
+val venv_find_object_or_empty : venv -> var_info -> obj
 
 (*
  * Static environments.
@@ -423,8 +334,10 @@ val venv_save_static_values    : venv -> unit
 (*
  * Primitive functions.
  *)
-val venv_add_prim_fun    : venv -> var -> (venv -> pos -> loc -> value list -> value) -> prim_fun
-val venv_apply_prim_fun  : prim_fun -> venv -> pos -> loc -> value list -> value
+type prim_fun_data = venv -> pos -> loc -> value list -> venv * value
+
+val venv_add_prim_fun    : venv -> var -> prim_fun_data -> prim_fun
+val venv_apply_prim_fun  : prim_fun -> prim_fun_data
 
 (*
  * Channels.
@@ -444,23 +357,33 @@ val venv_add_formatter_channel : venv -> Format.formatter -> prim_channel
  *)
 val venv_empty_object    : obj
 val venv_this            : venv -> obj
-val venv_this_object     : venv -> scope_kind -> obj
 val venv_current_object  : venv -> symbol list -> obj
-val venv_current_objects : venv -> scope_kind -> obj list
 val venv_define_object   : venv -> venv
 val venv_with_object     : venv -> obj -> venv
 val venv_include_object  : venv -> obj -> venv
 val venv_flatten_object  : venv -> obj -> venv
-val venv_find_super      : venv -> pos -> loc -> symbol -> obj
+val venv_find_super_field : venv -> pos -> loc -> symbol -> symbol -> value
+
+(* ZZZ: this doesn't exist in 0.9.9 *)
+val venv_current_objects : venv -> pos -> var_info -> value list
 
-val venv_add_field       : obj -> var -> value -> obj
-val venv_find_field_exn  : obj -> var -> value
-val venv_find_field      : obj -> pos -> var -> value
-val venv_object_mem      : obj -> var -> bool
-val venv_object_length   : obj -> int
-val venv_object_fold     : ('a -> var -> value -> 'a) -> 'a -> obj -> 'a
-val venv_instanceof      : obj -> symbol -> bool
 val venv_add_class       : obj -> symbol -> obj
+val venv_instanceof      : obj -> symbol -> bool
+
+val venv_find_field_path_exn  : venv -> path -> obj -> pos -> var -> path * value
+val venv_find_field_path      : venv -> path -> obj -> pos -> var -> path * value
+val venv_find_field_exn       : venv -> obj -> pos -> var -> value
+val venv_find_field           : venv -> obj -> pos -> var -> value
+val venv_add_field            : venv -> obj -> pos -> var -> value -> venv * obj
+val venv_defined_field        : venv -> obj -> var -> bool
+val venv_object_length        : obj -> int
+
+(* Internal hacks when we don't care about checking *)
+val venv_add_field_internal       : obj -> var -> value -> obj
+val venv_find_field_internal      : obj -> pos -> var -> value
+val venv_find_field_internal_exn  : obj -> var -> value
+val venv_defined_field_internal   : obj -> var -> bool
+val venv_object_fold_internal     : ('a -> var -> value -> 'a) -> 'a -> obj -> 'a
 
 val venv_add_included_file    : venv -> Node.t -> venv
 val venv_is_included_file     : venv -> Node.t -> bool
@@ -512,20 +435,10 @@ val venv_get_ordering_deps : venv -> ordering_info -> NodeSet.t -> NodeSet.t
 (*
  * Update the environment with a result.
  *)
-val add_exports : venv -> pos -> value -> venv * value
-val add_include : venv -> pos -> value -> venv * value
-
-(*
- * Remove exports from a value
- *)
-val export_none : value -> value
-
-(*
- * In case a value is an export, return the given variables in that
- * export to their state in the original environment.
- * (XXX: variables are updated in all 3 scope classes).
- *)
-val unexport : venv -> value -> var list -> value
+val add_exports      : venv -> venv -> pos -> export -> venv
+val add_path_exports : venv -> venv -> venv -> pos -> path -> export -> venv
+val hoist_path       : venv -> path -> obj -> venv
+val hoist_this       : venv -> venv -> path -> venv
 
 (*
  * Cached buildable flags.
@@ -556,97 +469,16 @@ val pp_print_arg_command_lines : formatter -> arg_command_line list -> unit
 val squash_prim_fun : prim_fun -> var
 val squash_object : obj -> value SymbolTable.t
 
-(************************************************************************
- * Exceptions.
- *)
-
-(*
- * Errors.
- *)
-type omake_error =
-   SyntaxError       of string
- | StringError       of string
- | StringStringError of string * string
- | StringDirError    of string * Dir.t
- | StringNodeError   of string * Node.t
- | StringVarError    of string * var
- | StringIntError    of string * int
- | StringMethodError of string * var list
- | StringValueError  of string * value
- | StringTargetError of string * target
- | LazyError         of (formatter -> unit)
- | UnboundVar        of var
- | UnboundFun        of var
- | UnboundMethod     of var list
- | ArityMismatch     of arity * int
- | NotImplemented    of string
- | UnboundKey        of string
- | UnboundValue      of value
- | NullCommand
-
 (*
  * General exception includes debugging info.
  *)
-exception OmakeException    of pos * omake_error
-exception UncaughtException of pos * exn
-exception RaiseException    of pos * obj
-exception ExitException     of pos * int
-exception Return            of loc * value
 exception Break             of loc * venv
 
-(*
- * Omake's internal version of the Invalid_argument
- *)
-exception OmakeFatal of string
-exception OmakeFatalErr of pos * omake_error
-
-(*
- * Module for creating positions.
- * You have to specify the name of the module
- * where the exception are being created: use
- * MakePos in each file where Name.name is set
- * to the name of the module.
- *)
-module type PosSig =
-sig
-   val loc_exp_pos    : loc -> pos
-   val loc_pos        : loc -> pos -> pos
-
-   val ast_exp_pos    : Omake_ast.exp -> pos
-   val ir_exp_pos     : Omake_ir.exp -> pos
-   val var_exp_pos    : var -> pos
-   val string_exp_pos : string -> pos
-   val value_exp_pos  : value -> pos
-
-   val string_pos     : string -> pos -> pos
-   val pos_pos        : pos -> pos -> pos
-   val int_pos        : int -> pos -> pos
-   val var_pos        : var -> pos -> pos
-   val error_pos      : omake_error -> pos -> pos
-
-   val del_pos        : (formatter -> unit) -> loc -> pos
-   val del_exp_pos    : (formatter -> unit) -> pos -> pos
-
-   (* Utilities *)
-   val loc_of_pos     : pos -> loc
-   val pp_print_pos   : formatter -> pos -> unit
-end
-
-module MakePos (Name : sig val name : string end) : PosSig
-
-(*
- * Exception printing.
- *)
-val pp_print_exn : formatter -> omake_error -> unit
-
 (*
  * For debugging.
  *)
-val pp_print_env            : formatter -> env -> unit
-val pp_print_value          : formatter -> value -> unit
-val pp_print_value_list     : formatter -> value list -> unit
 val pp_print_explicit_rules : formatter -> venv -> unit
-val pp_print_export         : formatter -> export -> unit
+val pp_print_rule           : formatter -> erule -> unit
 
 (*
  * Static values.
@@ -666,38 +498,26 @@ sig
     * not the .omc file.  We'll figure out where the .omc file
     * goes on our own.  Raises Not_found if the source file
     * can't be found.
+    * The implementation will make sure all the locking/unlocking is done properly.
     *)
-   val create_in    : venv -> Node.t -> in_handle
-   val close_in     : in_handle -> unit
-
-   val create_out   : venv -> Node.t -> out_handle
-   val recreate_out : in_handle -> out_handle
-   val close_out    : out_handle -> unit
+   val read        : venv -> Node.t -> (in_handle -> 'a) -> 'a
+   val rewrite     : in_handle -> (out_handle -> 'a) -> 'a
 
    (*
-    * Unfortunately, the IR type is delayed because it
-    * has type (Omake_ir_ast.senv * Omake_ir.ir), and
-    * Omake_ir_ast depends on this file.
-    *)
-
-   (*
-    * Fetch the three kinds of entries.
+    * Fetch the two kinds of entries.
     *)
    val find_ir     : in_handle -> ir
    val find_object : in_handle -> obj
-   val find_values : in_handle -> obj SymbolTable.t
 
-   (*
-    * Add the three kinds of entries.
-    *)
    val get_ir      : out_handle -> ir
    val get_object  : out_handle -> obj
-   val get_values  : out_handle -> obj SymbolTable.t
 
+   (*
+    * Add the two kinds of entries.
+    *)
    val add_ir      : out_handle -> ir -> unit
    val add_object  : out_handle -> obj -> unit
-   val add_values  : out_handle -> obj SymbolTable.t -> unit
-end;;
+end
 
 module Static : StaticSig;;
 
