@@ -9,16 +9,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -33,9 +33,12 @@ open Lm_location
 
 open Omake_ast
 open Omake_env
+open Omake_pos
 open Omake_symbol
+open Omake_value_type
+open Omake_value_print
 
-module Pos = MakePos (struct let name = "Omake_exn_print" end)
+module Pos = MakePos (struct let name = "Omake_exn_print" end);;
 open Pos
 
 (*
@@ -59,8 +62,8 @@ let pp_print_other_exn buf exn =
             (Printexc.to_string exn)
 
 let pp_print_obj_err buf obj =
-   if venv_object_mem obj message_sym then
-      match venv_find_field_exn obj message_sym with
+   if venv_defined_field_internal obj message_sym then
+      match venv_find_field_internal_exn obj message_sym with
          ValString s
        | ValData s ->
             begin match Lm_string_util.split "\n" s with
@@ -73,6 +76,7 @@ let pp_print_obj_err buf obj =
             pp_print_value buf v
    else
       pp_print_value buf (ValObject obj)
+
 (*
  * Exception printer.
  *)
@@ -96,6 +100,7 @@ let pp_print_exn buf exn =
             pp_print_obj_err obj
     | OmakeFatal s ->
          fprintf buf "@[<v 3>*** omake fatal error:@ %s@]" s
+    | ExitParentException (pos, code)
     | ExitException (pos, code) ->
          fprintf buf "@[<v 3>*** omake %s:@ %a@ early exit(%i) requested by an omake file@]" (**)
             (if code = 0 then "warning" else "error")
@@ -142,6 +147,7 @@ let catch f x =
     | Return _ as exn ->
          eprintf "%a@." pp_print_exn exn;
          exit Omake_state.exn_error_code
+    | ExitParentException (_, code)
     | ExitException (_, code) as exn ->
          eprintf "%a@." pp_print_exn exn;
          exit code
