@@ -16,16 +16,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -36,10 +36,14 @@
  *)
 open Lm_symbol
 open Lm_printf
+open Lm_location
 
 open Omake_ir
 open Omake_env
+open Omake_pos
+open Omake_options
 open Omake_ir_util
+open Omake_value_type
 
 module Pos = MakePos (struct let name = "Omake_ir_semant" end)
 open Pos
@@ -62,7 +66,8 @@ type renv =
  *    env_is_tail     : the current expression is in final position
  *)
 type env =
-   { env_in_function   : bool;
+   { env_warnings      : loc option ref;
+     env_in_function   : bool;
      env_in_cond       : bool;
      env_section_tail  : bool;
      env_function_tail : bool
@@ -98,15 +103,25 @@ let renv_return =
 (*
  * Normal environment, not in a function.
  *)
-let env_empty =
-   { env_in_function   = false;
+let env_empty () =
+   { env_warnings      = ref None;
+     env_in_function   = false;
      env_in_cond       = false;
      env_section_tail  = false;
      env_function_tail = false
    }
 
-let env_object_tail =
-   { env_in_function   = false;
+let env_object env =
+   { env_warnings      = env.env_warnings;
+     env_in_function   = false;
+     env_in_cond       = false;
+     env_section_tail  = false;
+     env_function_tail = false
+   }
+
+let env_object_tail env =
+   { env_warnings      = env.env_warnings;
+     env_in_function   = false;
      env_in_cond       = false;
      env_section_tail  = true;
      env_function_tail = true
@@ -115,8 +130,9 @@ let env_object_tail =
 (*
  * Fresh environment for a function body.
  *)
-let env_fun =
-   { env_in_function   = true;
+let env_fun env =
+   { env_warnings      = env.env_warnings;
+     env_in_function   = true;
      env_in_cond       = false;
      env_section_tail  = true;
      env_function_tail = true
@@ -129,13 +145,17 @@ let update_return renv has_return =
  * Error checkers.
  *)
 let check_return_placement env loc =
-   if not (env.env_in_function && (env.env_function_tail || env.env_section_tail && env.env_in_cond)) then
+   if not env.env_in_function then
       let pos = string_pos "check_in_function" (loc_exp_pos loc) in
       let print_error buf =
          fprintf buf "@[<v 0>Misplaced return statement.";
-         fprintf buf "@ Use the 'value' function if you just want the expression value.@]"
+         fprintf buf "@ The return is not within a function.@]"
       in
          raise (OmakeException (pos, LazyError print_error))
+   else if not (env.env_function_tail || env.env_section_tail && env.env_in_cond) then begin
+      eprintf "@[<v 3>*** omake warning: %a@ statements after this return are not reached@]@." pp_print_location loc;
+      env.env_warnings := Some loc
+   end
 
 let check_section_tail env loc =
    if not env.env_section_tail then
@@ -156,17 +176,18 @@ let rec build_string env s =
       NoneString _
     | ConstString _
     | ThisString _
-    | KeyString _ ->
+    | KeyApplyString _
+    | VarString _ ->
          false, s
-    | ApplyString (loc, strategy, scope, v, args) ->
+    | ApplyString (loc, strategy, v, args) ->
          let has_return, args = build_string_list env args in
-            has_return, ApplyString (loc, strategy, scope, v, args)
-    | SuperApplyString (loc, strategy, scope, v1, v2, args) ->
+            has_return, ApplyString (loc, strategy, v, args)
+    | SuperApplyString (loc, strategy, v1, v2, args) ->
          let has_return, args = build_string_list env args in
-            has_return, SuperApplyString (loc, strategy, scope, v1, v2, args)
-    | MethodApplyString (loc, strategy, scope, vl, args) ->
+            has_return, SuperApplyString (loc, strategy, v1, v2, args)
+    | MethodApplyString (loc, strategy, v, vl, args) ->
          let has_return, args = build_string_list env args in
-            has_return, MethodApplyString (loc, strategy, scope, vl, args)
+            has_return, MethodApplyString (loc, strategy, v, vl, args)
     | SequenceString (loc, sl) ->
          let has_return, sl = build_string_list env sl in
             has_return, SequenceString (loc, sl)
@@ -182,20 +203,24 @@ let rec build_string env s =
     | QuoteStringString (loc, c, sl) ->
          let has_return, sl = build_string_list env sl in
             has_return, QuoteStringString (loc, c, sl)
-    | BodyString (loc, e) ->
-         let renv, e = build_exp env e in
-            renv.renv_has_return, BodyString (loc, e)
-    | ExpString (loc, e) ->
-         let renv, e = build_exp env e in
-            renv.renv_has_return, ExpString (loc, e)
+    | ObjectString (loc, el, export) ->
+         let el = build_object_exp env el in
+            (* XXX: we should handle the case when an object contains a return *)
+            false, ObjectString (loc, el, export)
+    | BodyString (loc, el, export) ->
+         let renv, el = build_sequence_exp env el in
+            renv.renv_has_return, BodyString (loc, el, export)
+    | ExpString (loc, el, export) ->
+         let renv, el = build_sequence_exp env el in
+            renv.renv_has_return, ExpString (loc, el, export)
     | CasesString (loc, cases) ->
          let env = { env with env_in_cond = true } in
          let has_return, cases =
-            List.fold_left (fun (has_return, cases) (v, s, e) ->
+            List.fold_left (fun (has_return, cases) (v, s, el, export) ->
                   let has_return2, s = build_string env s in
-                  let renv, e = build_exp env e in
+                  let renv, e = build_sequence_exp env el in
                   let has_return = has_return || has_return2 || renv.renv_has_return in
-                     has_return, (v, s, e) :: cases) (false, []) cases
+                     has_return, (v, s, e, export) :: cases) (false, []) cases
          in
             has_return, CasesString (loc, List.rev cases)
 
@@ -212,22 +237,23 @@ and build_string_list env sl =
  *)
 and build_exp env e =
    match e with
-      LetFunExp (loc, kind, v, vars, e) ->
-         let renv, e = build_exp env_fun e in
-         let e =
+      LetFunExp (loc, v, vl, vars, el, export) ->
+         let renv, el = build_sequence_exp (env_fun env) el in
+         let el =
             if renv.renv_has_return then
-               ReturnCatchExp (loc, e)
+               [ReturnBodyExp (loc, el)]
             else
-               e
+               el
          in
-         let e = LetFunExp (loc, kind, v, vars, e) in
-            renv_empty, e
-    | LetObjectExp (loc, kind, v, el) ->
-         let el = build_object_exp el in
-         let e = LetObjectExp (loc, kind, v, el) in
+         let e = LetFunExp (loc, v, vl, vars, el, export) in
             renv_empty, e
+    | LetObjectExp (loc, v, vl, s, el, export) ->
+         let el = build_object_exp env el in
+         let has_return, s = build_string env s in
+         let e = LetObjectExp (loc, v, vl, s, el, export) in
+            update_return renv_empty has_return, e
     | StaticExp (loc, node, v, el) ->
-         let el = build_object_exp el in
+         let el = build_object_exp env el in
          let e = StaticExp (loc, node, v, el) in
             renv_empty, e
     | IfExp (loc, cases) ->
@@ -238,35 +264,35 @@ and build_exp env e =
          let renv, el = build_sequence_exp env el in
          let e = SequenceExp (loc, el) in
             renv, e
-    | SectionExp (loc, s, el) ->
+    | SectionExp (loc, s, el, export) ->
          let has_return, s = build_string env s in
          let renv, el = build_sequence_exp env el in
-         let e = SectionExp (loc, s, el) in
+         let e = SectionExp (loc, s, el, export) in
             update_return renv has_return, e
-    | ReturnCatchExp (loc, e) ->
-         let renv, e = build_exp env e in
-         let e = ReturnCatchExp (loc, e) in
-            renv, e
-    | LetVarExp (loc, scope, v, kind, s) ->
+    | ReturnBodyExp (loc, el) ->
+         let renv, el = build_sequence_exp env el in
+         let el = ReturnBodyExp (loc, el) in
+            renv, el
+    | LetVarExp (loc, v, vl, kind, s) ->
          let has_return, s = build_string env s in
-         let e = LetVarExp (loc, scope, v, kind, s) in
+         let e = LetVarExp (loc, v, vl, kind, s) in
             update_return renv_empty has_return, e
     | IncludeExp (loc, s, sl) ->
          let has_return1, s = build_string env s in
          let has_return2, sl = build_string_list env sl in
          let e = IncludeExp (loc, s, sl) in
             update_return renv_empty (has_return1 || has_return2), e
-    | ApplyExp (loc, scope, v, args) ->
+    | ApplyExp (loc, v, args) ->
          let has_return, args = build_string_list env args in
-         let e = ApplyExp (loc, scope, v, args) in
+         let e = ApplyExp (loc, v, args) in
             update_return renv_empty has_return, e
-    | SuperApplyExp (loc, scope, v1, v2, args) ->
+    | SuperApplyExp (loc, v1, v2, args) ->
          let has_return, args = build_string_list env args in
-         let e = SuperApplyExp (loc, scope, v1, v2, args) in
+         let e = SuperApplyExp (loc, v1, v2, args) in
             update_return renv_empty has_return, e
-    | MethodApplyExp (loc, scope, vl, args) ->
+    | MethodApplyExp (loc, v, vl, args) ->
          let has_return, args = build_string_list env args in
-         let e = MethodApplyExp (loc, scope, vl, args) in
+         let e = MethodApplyExp (loc, v, vl, args) in
             update_return renv_empty has_return, e
     | LetKeyExp (loc, v, kind, s) ->
          let has_return, s = build_string env s in
@@ -287,12 +313,6 @@ and build_exp env e =
          let has_return, s = build_string env s in
          let e = StringExp (loc, s) in
             update_return renv_value has_return, e
-    | ExportExp (loc, s) ->
-         let has_return, s = build_string env s in
-         let e = ExportExp (loc, s) in
-            update_return renv_value has_return, e
-    | CancelExportExp _ ->
-         renv_value, e
     | ReturnExp (loc, s) ->
          check_return_placement env loc;
          let has_return, s = build_string env s in
@@ -309,18 +329,21 @@ and build_exp env e =
  * An object expression is an expression sequence,
  * but it is not in a function.
  *)
-and build_object_exp el =
+and build_object_exp_aux env el =
    match el with
       [e] ->
-         let _, e = build_exp env_object_tail e in
+         let _, e = build_exp (env_object_tail env) e in
             [e]
     | e :: el ->
-         let _, e = build_exp env_empty e in
-         let el = build_object_exp el in
+         let _, e = build_exp env e in
+         let el = build_object_exp_aux env el in
             e :: el
     | [] ->
          []
 
+and build_object_exp env el =
+   build_object_exp_aux (env_object env) el
+
 (*
  * A new sequence expression.
  * It should be terminated with a final statement.
@@ -337,13 +360,6 @@ and build_sequence_exp env el =
             let env_tail = { env with env_section_tail = true } in
             let renv, e = build_exp env_tail e in
             let rel = e :: rel in
-            let rel =
-               if renv.renv_is_final || renv.renv_is_value then
-                  rel
-               else
-                  (* Add a dummy value *)
-                  CancelExportExp (loc_of_exp e) :: rel
-            in
                update_return renv has_return, rel
        | e :: el ->
             let renv, e = build_exp env_non_tail e in
@@ -369,15 +385,15 @@ and build_cases_exp env cases =
       }
    in
    let has_return, cases =
-      List.fold_left (fun (has_return, cases) (s, e) ->
-            let renv, e = build_exp env e in
+      List.fold_left (fun (has_return, cases) (s, el, export) ->
+            let renv, el = build_sequence_exp env el in
             let has_return = has_return || renv.renv_has_return in
-               has_return, (s, e) :: cases) (false, []) cases
+               has_return, (s, el, export) :: cases) (false, []) cases
    in
    let cases = List.rev cases in
    let renv =
-      { renv_is_final = false;
-        renv_is_value = true;
+      { renv_is_final   = false;
+        renv_is_value   = true;
         renv_has_return = has_return
       }
    in
@@ -386,8 +402,17 @@ and build_cases_exp env cases =
 (************************************************************************
  * Main function
  *)
-let build_prog e =
-   let _, e = build_exp env_empty e in
+let build_prog venv e =
+   let env = env_empty () in
+   let _, e = build_exp env e in
+   let count = !(env.env_warnings) in
+   let () =
+      match count with
+         Some loc when opt_warn_error (venv_options venv) ->
+            raise (OmakeException (loc_exp_pos loc, StringError "warnings treated as errors"))
+       | _ ->
+            ()
+   in
       e
 
 (*!
