@@ -31,12 +31,15 @@
 open Lm_printf
 
 open Omake_ir
+open Omake_var
 open Omake_env
+open Omake_pos
 open Omake_node_sig
 open Omake_node
 open Omake_lexer
 open Omake_parser
 open Omake_symbol
+open Omake_value_type
 
 module Pos = MakePos (struct let name = "Omake_value" end)
 open Pos
@@ -65,12 +68,12 @@ let bool_of_value     = Omake_eval.bool_of_value
  *)
 let eval_object_value venv pos obj =
    let pos = string_pos "eval_object_value" pos in
-      try venv_find_field_exn obj builtin_sym with
+      try venv_find_field_internal_exn obj builtin_sym with
          Not_found ->
             raise (OmakeException (pos, StringError "not a primitive object"))
 
 let add_object_value obj x =
-   venv_add_field obj builtin_sym x
+   venv_add_field_internal obj builtin_sym x
 
 (*
  * Concatenate.
@@ -140,6 +143,20 @@ let number_of_value venv pos v =
                         Failure _ ->
                            raise (OmakeException (pos, StringStringError ("not a number", s)))
 
+(*
+ * Variables.
+ *)
+let var_of_value venv pos v =
+   let v = eval_prim_value venv pos v in
+      match v with
+         ValVar (_, v) ->
+            v
+       | _ ->
+            raise (OmakeException (pos, StringValueError ("not a var", v)))
+
+let vars_of_value venv pos v =
+   List.map (var_of_value venv pos) (values_of_value venv pos v)
+
 (*
  * Maps.
  *)
@@ -165,7 +182,7 @@ let rec key_of_value venv pos v =
        | ValFloat _
        | ValOther (ValExitCode _)
        | ValOther (ValLocation _)
-       | ValOther (ValPosition _) ->
+       | ValVar _ ->
             v
        | ValQuote _
        | ValQuoteString _
@@ -176,14 +193,13 @@ let rec key_of_value venv pos v =
             let values = values_of_value venv pos v in
             let values = List.map (key_of_value venv pos) values in
                ValArray values
-       | ValKey _
+       | ValKeyApply _
        | ValApply _
-       | ValImplicit _
+       | ValMaybeApply _
        | ValFun _
        | ValFunValue _
        | ValPrim _
        | ValRules _
-       | ValEnv _
        | ValBody _
        | ValMap _
        | ValObject _
@@ -192,7 +208,8 @@ let rec key_of_value venv pos v =
        | ValChannel _
        | ValClass _
        | ValCases _
-       | ValOther _ ->
+       | ValOther _
+       | ValDelayed _ ->
             raise (OmakeException (pos, StringValueError ("bad map key", v)))
 
 
@@ -218,14 +235,14 @@ let dir_of_value venv pos dir =
        | ValFloat _ ->
             venv_intern_dir venv (string_of_value venv pos dir)
        | ValNone
-       | ValKey _
+       | ValVar _
+       | ValKeyApply _
        | ValApply _
-       | ValImplicit _
+       | ValMaybeApply _
        | ValFun _
        | ValFunValue _
        | ValPrim _
        | ValRules _
-       | ValEnv _
        | ValBody _
        | ValMap _
        | ValObject _
@@ -234,7 +251,8 @@ let dir_of_value venv pos dir =
        | ValChannel _
        | ValClass _
        | ValCases _
-       | ValOther _ ->
+       | ValOther _
+       | ValDelayed _ ->
             raise (OmakeException (pos, StringError "not a directory"))
 
 let node_value_of_value venv pos v =
@@ -250,9 +268,9 @@ let node_value_of_value venv pos v =
        | ValString _
        | ValSequence _
        | ValArray _
-       | ValKey _
+       | ValKeyApply _
        | ValApply _
-       | ValImplicit _
+       | ValMaybeApply _
        | ValSuperApply _
        | ValMethodApply _
        | ValBody _
@@ -266,7 +284,7 @@ let node_value_of_value venv pos v =
                else
                   ValNode node
        | ValNone
-       | ValEnv _
+       | ValVar _
        | ValFun _
        | ValFunValue _
        | ValPrim _
@@ -276,7 +294,8 @@ let node_value_of_value venv pos v =
        | ValChannel _
        | ValClass _
        | ValCases _
-       | ValOther _ ->
+       | ValOther _
+       | ValDelayed _ ->
             raise (OmakeException (pos, StringValueError("not a file", v)))
 
 let dir_value_of_value venv pos v =
@@ -292,9 +311,9 @@ let dir_value_of_value venv pos v =
        | ValString _
        | ValSequence _
        | ValArray _
-       | ValKey _
+       | ValKeyApply _
        | ValApply _
-       | ValImplicit _
+       | ValMaybeApply _
        | ValSuperApply _
        | ValMethodApply _
        | ValBody _
@@ -303,7 +322,7 @@ let dir_value_of_value venv pos v =
             let name = string_of_value venv pos v in
                ValDir (venv_intern_dir venv name)
        | ValNone
-       | ValEnv _
+       | ValVar _
        | ValFun _
        | ValFunValue _
        | ValPrim _
@@ -313,7 +332,8 @@ let dir_value_of_value venv pos v =
        | ValChannel _
        | ValClass _
        | ValCases _
-       | ValOther _ ->
+       | ValOther _
+       | ValDelayed _ ->
             raise (OmakeException (pos, StringValueError("not a file", v)))
 
 let filename_of_value venv pos v =
@@ -374,14 +394,14 @@ let prim_channel_of_value venv pos v =
             prim_channel_of_string venv pos (string_of_value venv pos arg)
        | ValInt _
        | ValFloat _
-       | ValKey _
+       | ValKeyApply _
        | ValApply _
-       | ValImplicit _
+       | ValMaybeApply _
        | ValSuperApply _
        | ValMethodApply _
+       | ValVar _
        | ValBody _
        | ValNone
-       | ValEnv _
        | ValFun _
        | ValFunValue _
        | ValPrim _
@@ -391,11 +411,12 @@ let prim_channel_of_value venv pos v =
        | ValObject _
        | ValClass _
        | ValCases _
-       | ValOther _ ->
-            raise (OmakeException (pos, StringError "not an input channel"))
+       | ValOther _
+       | ValDelayed _ ->
+            raise (OmakeException (pos, StringError "not a channel"))
 
 let prim_channel_of_var venv pos loc v =
-   prim_channel_of_value venv pos (venv_find_var venv ScopeGlobal pos loc v)
+   prim_channel_of_value venv pos (venv_find_var venv pos loc v)
 
 let channel_of_var venv pos loc v =
    let channel = prim_channel_of_var venv pos loc v in
@@ -420,9 +441,9 @@ let in_channel_of_any_value venv pos v =
        | ValQuoteString _
        | ValString _
        | ValSequence _
-       | ValKey _
+       | ValKeyApply _
        | ValApply _
-       | ValImplicit _
+       | ValMaybeApply _
        | ValSuperApply _
        | ValMethodApply _
        | ValBody _
@@ -444,7 +465,6 @@ let in_channel_of_any_value venv pos v =
                      pc, true
        | ValChannel (OutChannel, _)
        | ValNone
-       | ValEnv _
        | ValFun _
        | ValFunValue _
        | ValPrim _
@@ -454,7 +474,9 @@ let in_channel_of_any_value venv pos v =
        | ValObject _
        | ValClass _
        | ValCases _
-       | ValOther _ ->
+       | ValOther _
+       | ValVar _
+       | ValDelayed _ ->
             raise (OmakeException (pos, StringError "not an input channel"))
 
 let out_channel_of_any_value venv pos v =
@@ -471,9 +493,9 @@ let out_channel_of_any_value venv pos v =
        | ValString _
        | ValQuoteString _
        | ValSequence _
-       | ValKey _
+       | ValKeyApply _
        | ValApply _
-       | ValImplicit _
+       | ValMaybeApply _
        | ValSuperApply _
        | ValMethodApply _
        | ValBody _
@@ -495,7 +517,6 @@ let out_channel_of_any_value venv pos v =
                      prim, true
        | ValChannel (InChannel, _)
        | ValNone
-       | ValEnv _
        | ValFun _
        | ValFunValue _
        | ValPrim _
@@ -505,7 +526,9 @@ let out_channel_of_any_value venv pos v =
        | ValObject _
        | ValClass _
        | ValCases _
-       | ValOther _ ->
+       | ValOther _
+       | ValVar _
+       | ValDelayed _ ->
             raise (OmakeException (pos, StringError "not an output channel"))
 
 (*
@@ -524,16 +547,15 @@ let rec is_glob_value options v =
     | ValDir _
     | ValData _
     | ValQuote _
-    | ValKey _
+    | ValKeyApply _
     | ValApply _
-    | ValImplicit _
+    | ValMaybeApply _
     | ValSuperApply _
     | ValMethodApply _
     | ValBody _
     | ValInt _
     | ValFloat _
     | ValNone
-    | ValEnv _
     | ValFun _
     | ValFunValue _
     | ValPrim _
@@ -542,7 +564,9 @@ let rec is_glob_value options v =
     | ValObject _
     | ValClass _
     | ValCases _
-    | ValOther _ ->
+    | ValOther _
+    | ValVar _
+    | ValDelayed _ ->
          false
 
 and is_glob_value_list options vl =
@@ -554,7 +578,7 @@ and is_glob_value_list options vl =
 let current_lexer venv pos =
    let pos = string_pos "current_lexer" pos in
       try
-         match venv_find_var_exn venv ScopeProtected builtin_sym with
+         match venv_find_var_exn venv builtin_field_var with
             ValOther (ValLexer lexer) ->
                lexer
           | v ->
@@ -566,7 +590,7 @@ let current_lexer venv pos =
 let current_parser venv pos =
    let pos = string_pos "current_parser" pos in
       try
-         match venv_find_var_exn venv ScopeProtected builtin_sym with
+         match venv_find_var_exn venv builtin_field_var with
             ValOther (ValParser parser) ->
                parser
           | v ->
@@ -579,8 +603,6 @@ let loc_of_value venv pos v =
    match eval_prim_value venv pos v with
       ValOther (ValLocation loc) ->
          loc
-    | ValOther (ValPosition pos) ->
-         loc_of_pos pos
     | _ ->
          raise (OmakeException (pos, StringValueError ("not a location", v)))
 
