@@ -4,21 +4,25 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2003 Mojave Group, Caltech
+ * Copyright (C) 2003-2006 Mojave Group, Caltech
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
+ * as published by the Free Software Foundation; version 2
+ * of the License.
+ * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * 
+ * Additional permission is given to link this library with the
+ * with the Objective Caml runtime, and to redistribute the
+ * linked executables.  See the file LICENSE.OMake for more details.
  *
  * Author: Jason Hickey
  * @email{jyh@cs.caltech.edu}
@@ -38,6 +42,11 @@ type id
  *)
 module IdTable : Lm_map_sig.LmMap with type key = id
 
+(*
+ * "Null" id for the "master" process.
+ *)
+val null_id : id
+
 (*
  * Get a new id.
  *)
@@ -54,12 +63,9 @@ val pp_print_pid : formatter -> id -> unit
 val marshal_id : id -> msg
 val unmarshal_id : msg -> id
 
-(*!
- * @docoff
- *
+(*
  * -*-
  * Local Variables:
- * Caml-master: "compile"
  * End:
  * -*-
  *)
