@@ -41,10 +41,12 @@ open Lm_debug
 open Lm_printf
 open Lm_string_set
 
+open Omake_options
 open Omake_node
 open Omake_command
 open Omake_node_sig
 open Omake_cache_type
+open Omake_value_type
 
 let debug_cache =
    create_debug (**)
@@ -63,7 +65,24 @@ struct
    let compare = (-)
 end
 
-module IndexMTable = Lm_map.LmMakeList (IndexCompare);;
+module IndexTable = Lm_map.LmMakeList (IndexCompare);;
+
+module IndexNodeTable =
+struct
+   type t = NodeSet.t IndexTable.t
+
+   let empty = IndexTable.empty
+
+   let add table i node =
+      IndexTable.filter_add table i (fun nodes ->
+            match nodes with
+               Some nodes ->
+                  NodeSet.add nodes node
+             | None ->
+                  NodeSet.singleton node)
+
+   let find = IndexTable.find
+end;;
 
 (*
  * Directory entry is a directory or node.
@@ -99,12 +118,12 @@ type exe_listing = exe_listing_item list
  * care what their digest is.
  *)
 (* %%MAGICBEGIN%% *)
-type memo =
+type 'a memo =
    { memo_index    : int;
      memo_targets  : digest NodeTable.t;
      memo_deps     : digest NodeTable.t;
      memo_commands : digest;
-     memo_result   : memo_result
+     memo_result   : 'a memo_result
    }
 (* %%MAGICEND%% *)
 
@@ -112,6 +131,7 @@ type memo =
  * Stats of a file.
  * FreshStats: we have taken the stats during this program run
  * OldStats: we took the stats during a prior run of this program
+ * PartialStats: we took the stats, but not the digest
  * NoStats: we tried to take stats, but an error occurred (usually
  *    because the file did not exist).
  *)
@@ -139,9 +159,9 @@ type node_memo =
  *)
 type key = int
 
-type cache_info =
-   { mutable cache_memos     : memo NodeTable.t;
-     mutable cache_index     : Node.t IndexMTable.t
+type 'a cache_info =
+   { mutable cache_memos     : 'a memo NodeTable.t;
+     mutable cache_index     : IndexNodeTable.t
    }
 
 (*
@@ -152,7 +172,9 @@ type stat = Unix.LargeFile.stats
 type cache =
    { (* State *)
      mutable cache_nodes             : node_memo NodeTable.t;
-     mutable cache_info              : cache_info array;
+     mutable cache_info              : memo_deps cache_info array;
+     mutable cache_static_values     : obj memo ValueTable.t;
+     mutable cache_memo_values       : obj memo ValueTable.t;
      mutable cache_file_stat_count   : int;  (* only succeessful stats are counted *)
      mutable cache_digest_count      : int;
 
@@ -166,10 +188,13 @@ type cache =
  * The version of the cache that is saved in the file also
  * contains a table of all nodes.
  *)
+(* %%MAGICBEGIN%% *)
 type cache_save =
    { save_cache_nodes        : node_memo NodeTable.t;
-     save_cache_info         : memo NodeTable.t array
+     save_cache_info         : memo_deps memo NodeTable.t array;
+     save_cache_value        : obj memo ValueTable.t
    }
+(* %%MAGICEND%% *)
 
 (*
  * Squash stat code.
@@ -222,12 +247,10 @@ let pp_print_node_digest_table buf deps =
  *)
 let pp_print_memo_result pp_print_result buf result =
    match result with
-      MemoSuccess ->
-         ()
+      MemoSuccess _ ->
+         fprintf buf "@ success"
     | MemoFailure code ->
          fprintf buf "@ failed(%d)" code
-    | MemoResult result ->
-         pp_print_result buf result
 
 (*
  * Print a memo.
@@ -264,6 +287,8 @@ let output_magic = Omake_magic.output_magic
 let create () =
    { cache_nodes           = NodeTable.empty;
      cache_info            = [||];
+     cache_static_values   = ValueTable.empty;
+     cache_memo_values     = ValueTable.empty;
      cache_file_stat_count = 0;
      cache_digest_count    = 0;
      cache_dirs            = DirTable.empty;
@@ -281,23 +306,14 @@ let stats { cache_file_stat_count = stat_count;
     } =
    stat_count, digest_count
 
-(*
- * Clear one of the tables.
- *)
-let clear cache index =
-   let info = cache.cache_info in
-      if index < Array.length info then
-         let info = info.(index) in
-            info.cache_memos <- NodeTable.empty;
-            info.cache_index <- IndexMTable.empty
-
 (*
  * When the file is saved, remove all stat information,
  * as well as any files that don't exist.
  *)
 let save_of_cache cache =
-   let { cache_nodes = cache_nodes;
-         cache_info  = cache_info
+   let { cache_nodes  = cache_nodes;
+         cache_info   = cache_info;
+         cache_static_values = cache_values
        } = cache
    in
    let cache_nodes =
@@ -313,7 +329,8 @@ let save_of_cache cache =
    in
    let cache_info = Array.map (fun info -> info.cache_memos) cache_info in
       { save_cache_nodes = cache_nodes;
-        save_cache_info  = cache_info
+        save_cache_info  = cache_info;
+        save_cache_value = cache_values
       }
 
 (*
@@ -326,44 +343,59 @@ let create_index memos =
              } = memo
          in
             match result with
-               MemoSuccess
-             | MemoResult _ ->
-                  let index = IndexMTable.add index hash target in
+               MemoSuccess _ ->
+                  let index = IndexNodeTable.add index hash target in
                      memos, index
              | MemoFailure _ ->
                   let memos = NodeTable.remove memos target in
-                     memos, index) (memos, IndexMTable.empty) memos
+                     memos, index) (memos, IndexNodeTable.empty) memos
 
 (*
  * Rebuild the cache from the saved version.
  *)
-let cache_of_save save =
+let cache_of_save options save =
    let { save_cache_nodes = cache_nodes;
-         save_cache_info  = cache_info
+         save_cache_info  = cache_info;
        } = save
    in
+   let flush_scanner = opt_flush_dependencies options in
    let cache_info =
-      Array.map (fun memos ->
-            let memos, index = create_index memos in
-               { cache_memos = memos;
-                 cache_index = index
-               }) cache_info
+      Array.mapi (fun idx memos ->
+            if flush_scanner && idx = scanner_fun then
+               { cache_memos = NodeTable.empty;
+                 cache_index = IndexNodeTable.empty
+               }
+            else
+               let memos, index = create_index memos in
+                  { cache_memos = memos;
+                    cache_index = index
+                  }) cache_info
    in
-      { (create ()) with cache_nodes = cache_nodes;
-                         cache_info  = cache_info;
+   let cache_values =
+      if opt_flush_static options then
+         ValueTable.empty
+      else
+         save.save_cache_value
+   in
+      { (create ()) with cache_nodes  = cache_nodes;
+                         cache_info   = cache_info;
+                         cache_static_values = cache_values
       }
 
 (*
  * Load the old cache from a file.
  *)
-let from_channel inx =
-   let magic = input_magic inx in
-   let _ =
-      if magic <> magic_number then
-         raise (Sys_error "bad magic number")
-   in
-   let save = Marshal.from_channel inx in
-      cache_of_save save
+let from_channel options inx =
+   if opt_flush_cache options then
+      create ()
+   else
+      let magic = input_magic inx in
+      let _ =
+         if magic <> magic_number then
+            raise (Sys_error "bad magic number")
+      in
+      let save = Marshal.from_channel inx in
+         cache_of_save options save
 
 (*
  * Save the cache to the file.
@@ -474,17 +506,15 @@ let digest_sample_exn fd file_size =
 let digest_large name file_size =
    try
       let fd = Unix.openfile name [Unix.O_RDONLY] 0o000 in
-      let digest =
          try
-            digest_sample_exn fd file_size
+            let digest = digest_sample_exn fd file_size in
+               Unix.close fd;
+               digest
          with
             Unix.Unix_error _
           | End_of_file ->
                Unix.close fd;
                raise (Sys_error name)
-      in
-         Unix.close fd;
-         digest
    with
       Unix.Unix_error _ ->
          raise (Sys_error name)
@@ -624,7 +654,44 @@ let stat_unix_simple stats =
     | OldStats _ ->
          raise (Invalid_argument "Omake_cache.stat_unix_simple")
 
-let stat_unix cache node =
+let do_stat_unix cache node =
+   let name = Node.fullname node in
+   let nmemo =
+      try
+         let stats = Unix.LargeFile.stat name in
+            if stats.Unix.LargeFile.st_kind = Unix.S_DIR then
+               { nmemo_stats = FreshStats stats;
+                 nmemo_digest = squash_stat
+               }
+            else begin
+               cache.cache_file_stat_count <- succ cache.cache_file_stat_count;
+               { nmemo_stats = PartialStats stats;
+                 nmemo_digest = None
+               }
+            end
+      with
+         Unix.Unix_error _
+       | Sys_error _ ->
+            { nmemo_stats  = NoStats;
+              nmemo_digest = None
+            }
+   in
+      cache.cache_nodes <- NodeTable.add cache.cache_nodes node nmemo;
+      stat_unix_simple nmemo.nmemo_stats
+
+let stat_unix cache ?(force=false) node =
+   let node =
+      match Node.kind node with
+         NodePhony
+       | NodeScanner ->
+            raise Not_found
+       | NodeNormal ->
+            node
+       | NodeOptional
+       | NodeSquashed
+       | NodeExists ->
+            Node.core node
+   in
    let nodes = cache.cache_nodes in
    let stats =
       try Some (NodeTable.find nodes node) with
@@ -636,7 +703,10 @@ let stat_unix cache node =
        | Some { nmemo_stats = PartialStats stats } ->
             stats
        | Some { nmemo_stats = NoStats } ->
-            raise Not_found
+            if force then
+               do_stat_unix cache node
+            else
+               raise Not_found
        | Some ({ nmemo_stats = OldStats old_stats } as nmemo) ->
             let name = Node.fullname node in
             let nmemo =
@@ -663,35 +733,13 @@ let stat_unix cache node =
                cache.cache_nodes <- NodeTable.add nodes node nmemo;
                stat_unix_simple nmemo.nmemo_stats
        | None ->
-            let name = Node.fullname node in
-            let nmemo =
-               try
-                  let stats = Unix.LargeFile.stat name in
-                     if stats.Unix.LargeFile.st_kind = Unix.S_DIR then
-                        { nmemo_stats = FreshStats stats;
-                          nmemo_digest = squash_stat
-                        }
-                     else begin
-                        cache.cache_file_stat_count <- succ cache.cache_file_stat_count;
-                        { nmemo_stats = PartialStats stats;
-                          nmemo_digest = None
-                        }
-                     end
-               with
-                  Unix.Unix_error _
-                | Sys_error _ ->
-                     { nmemo_stats  = NoStats;
-                       nmemo_digest = None
-                     }
-            in
-               cache.cache_nodes <- NodeTable.add nodes node nmemo;
-               stat_unix_simple nmemo.nmemo_stats
+            do_stat_unix cache node
 
 (*
  * Check if a file is a directory.
  *)
-let is_dir cache node =
-   try (stat_unix cache node).Unix.LargeFile.st_kind = Unix.S_DIR with
+let is_dir cache ?(force=false) node =
+   try (stat_unix cache ~force node).Unix.LargeFile.st_kind = Unix.S_DIR with
       Not_found ->
          false
 
@@ -768,13 +816,14 @@ let stat_changed cache node =
 (*
  * Check if a file exists.
  *)
-let exists cache node =
-   match stat cache node with
-      Some _ -> true
-    | None -> Node.always_exists node
+let exists cache ?(force=false) node =
+   try
+      ignore (stat_unix cache ~force node); true
+   with
+      Not_found -> Node.always_exists node
 
-let exists_dir cache dir =
-   exists cache (Node.node_of_dir dir)
+let exists_dir cache ?(force=false) dir =
+   exists cache ~force (Node.node_of_dir dir)
 
 (************************************************************************
  * Adding to the cache.
@@ -804,7 +853,7 @@ let get_info cache key =
             let cache_info' =
                Array.init (succ key) (fun _ ->
                      { cache_memos = NodeTable.empty;
-                       cache_index = IndexMTable.empty
+                       cache_index = IndexNodeTable.empty
                      })
             in
                Array.blit cache_info 0 cache_info' 0 len;
@@ -831,7 +880,7 @@ let add cache key target targets deps commands result =
    in
    let info = get_info cache key in
       info.cache_memos <- NodeTable.add info.cache_memos target memo;
-      info.cache_index <- IndexMTable.add info.cache_index index target
+      info.cache_index <- IndexNodeTable.add info.cache_index index target
 
 (*
  * Check the target digest.
@@ -877,7 +926,9 @@ let find_memo cache key deps commands =
     | [] ->
          raise Not_found
    in
-      search (IndexMTable.find_all index hash)
+   let targets = NodeSet.to_list (IndexNodeTable.find index hash) in
+      (* eprintf "Targets: %d@." (List.length targets); *)
+      search targets
 
 (*
  * A memo has not changed if all the deps and the target
@@ -888,8 +939,7 @@ let up_to_date cache key deps commands =
       let memo = find_memo cache key deps commands in
          if targets_equal cache memo.memo_targets then
             match memo.memo_result with
-               MemoSuccess
-             | MemoResult _ ->
+               MemoSuccess _ ->
                   true
              | MemoFailure _ ->
                   false
@@ -904,8 +954,7 @@ let up_to_date_status cache key deps commands =
       let memo = find_memo cache key deps commands in
          if targets_equal cache memo.memo_targets then
             match memo.memo_result with
-               MemoSuccess
-             | MemoResult _ ->
+               MemoSuccess _ ->
                   StatusSuccess
              | MemoFailure code ->
                   StatusFailure code
@@ -923,10 +972,8 @@ let target_results results =
    match results with
       MemoFailure _ ->
          raise Not_found
-    | MemoSuccess ->
-         NodeTable.empty
-    | MemoResult results ->
-         results
+    | MemoSuccess deps ->
+         deps
 
 let find_result cache key deps commands =
    let memo = find_memo cache key deps commands in
@@ -948,10 +995,60 @@ let find_result_sloppy cache key target =
       match memo.memo_result with
          MemoFailure _ ->
             raise Not_found
-       | MemoSuccess ->
-            NodeTable.empty
-       | MemoResult result ->
-            result
+       | MemoSuccess deps ->
+            deps
+
+(************************************************************************
+ * Values.  In this case, we use the key to find the memo.
+ *)
+
+(*
+ * Get the memo entry from the key.
+ *)
+let get_value cache key is_static =
+   ValueTable.find (if is_static then cache.cache_static_values else cache.cache_memo_values) key
+
+(*
+ * Find a memo from the deps and commands.
+ *)
+let find_value_memo cache key is_static deps1 commands1 =
+   let memo = get_value cache key is_static in
+   let hash1 = hash_index deps1 commands1 in
+   let { memo_index = hash2;
+         memo_deps  = deps2;
+         memo_commands = commands2
+       } = memo
+   in
+      if hash1 = hash2 && deps_equal cache deps1 deps2 && commands1 = commands2 then
+         memo
+      else
+         raise Not_found
+
+let find_value cache key is_static deps commands =
+   let memo = find_value_memo cache key is_static deps commands in
+   let { memo_targets = targets;
+         memo_result  = result
+       } = memo
+   in
+      if targets_equal cache targets then
+         target_results result
+      else
+         raise Not_found
+
+let add_value cache key is_static deps commands result =
+   let index = hash_index deps commands in
+   let memo =
+      { memo_index    = index;
+        memo_targets  = NodeTable.empty;
+        memo_deps     = stat_set cache deps;
+        memo_result   = result;
+        memo_commands = commands
+      }
+   in
+      if is_static then
+         cache.cache_static_values <- ValueTable.add cache.cache_static_values key memo
+      else
+         cache.cache_memo_values <- ValueTable.add cache.cache_memo_values key memo
 
 (************************************************************************
  * Directory listings.
