@@ -106,9 +106,16 @@ struct
          raise Not_found
       else
          match name.[i] with
-            'A'..'Z' -> String.lowercase name
-          | 'a'..'z' -> String.uppercase name
-          | _ -> toggle_name_case name len (succ i)
+            'A'..'Z'
+          | '\192' .. '\214'
+          | '\216' .. '\222' ->
+               String.lowercase name
+          | 'a'..'z'
+          | '\224' .. '\246'
+          | '\248' .. '\254' ->
+               String.uppercase name
+          | _ ->
+               toggle_name_case name len (succ i)
 
    (*
     * Stat, does not fail.
@@ -178,7 +185,10 @@ struct
           | ".." ->
                dir_test_all_entries_exn absdir dir_handle
           | _ ->
-               try stat_with_toggle_case absdir name
+               try 
+                  let result = stat_with_toggle_case absdir name in
+                     Unix.closedir dir_handle;
+                     result
                with Not_found ->
                      dir_test_all_entries_exn absdir dir_handle
 
@@ -186,6 +196,7 @@ struct
     * Check for sensativity by creating a dummy file.
     *)
    let dir_test_new_entry_exn absdir =
+      Unix.access absdir [Unix.W_OK];
       let name = sprintf "OM%06x.tmp" (Random.State.bits fs_random land 0xFFFFFF) in
       let absname = Filename.concat absdir name in
       let () = do_create absname in
@@ -202,27 +213,52 @@ struct
     * stopping when one is successful.
     *    1. Try the name being created itself.
     *    2. Try looking at the directory entries.
-    *    3. Create a dummy file, and test.
+    *    3. Create a dummy file, and test (unless the attept to do step 2 made
+    *       us realize there is no such directory).
     *    4. Test the parent.
+    *
+    * Steps 2-4 will only be performed when we really need them (the name contains
+    * uppercase letters),
     *)
-   let rec dir_test_sensitivity dir absdir name =
+   exception Not_a_usable_directory
+   exception Already_lowercase
+
+   let rec check_already_lowercase name len i =
+      if i = len then
+         raise Already_lowercase
+      else
+         match name.[i] with
+            'A'..'Z'
+          | '\192' .. '\214'
+          | '\216' .. '\222' -> ()
+          | _ -> check_already_lowercase name len (i+1)
+
+   let rec dir_test_sensitivity shortcircuit dir absdir name =
       try stat_with_toggle_case absdir name
       with Not_found ->
-         try dir_test_all_entries_exn absdir (Unix.opendir absdir)
-         with Unix.Unix_error _ | Not_found | End_of_file ->
-            try dir_test_new_entry_exn absdir
-            with Unix.Unix_error _ | Not_found | End_of_file ->
-               match DirHash.get dir with
-                  DirRoot _ ->
-                     (* Nothing else we can do, assume sensitive *)
-                     true
-                | DirSub (_, name, parent) ->
-                     dir_is_sensitive parent name
+         if shortcircuit then
+            check_already_lowercase name (String.length name) 0;
+         try
+            let dir_handle =
+               try Unix.opendir absdir with
+                  Unix.Unix_error ((Unix.ENOENT | Unix.ENOTDIR | Unix.ELOOP | Unix.ENAMETOOLONG), _, _) ->
+                     raise Not_a_usable_directory
+            in
+               try dir_test_all_entries_exn absdir dir_handle
+               with Unix.Unix_error _ | Not_found | End_of_file ->
+                     dir_test_new_entry_exn absdir
+         with Unix.Unix_error _ | Not_found | End_of_file | Not_a_usable_directory ->
+            match DirHash.get dir with
+               DirRoot _ ->
+                  (* Nothing else we can do, assume sensitive *)
+                  true
+             | DirSub (_, name, parent) ->
+                  dir_is_sensitive false parent name
 
    (*
     * This is the caching version of the case-sensitivity test.
     *)
-   and dir_is_sensitive dir name =
+   and dir_is_sensitive shortcircuit dir name =
       try DirTable.find !case_table dir with
          Not_found ->
             let absdir = abs_dir_name dir in
@@ -232,9 +268,9 @@ struct
                      Lm_fs_case_sensitive.case_sensitive absdir
                   with
                      Failure _ ->
-                        dir_test_sensitivity dir absdir name
+                        dir_test_sensitivity shortcircuit dir absdir name
                else
-                  dir_test_sensitivity dir absdir name
+                  dir_test_sensitivity shortcircuit dir absdir name
             in
                case_table := DirTable.add !case_table dir sensitive;
                sensitive
@@ -250,7 +286,11 @@ struct
        | "Cygwin" ->
             (fun _ name -> String.lowercase name)
        | _ ->
-            (fun dir name -> if dir_is_sensitive dir name then name else String.lowercase name)
+            (fun dir name ->
+               try
+                  if dir_is_sensitive true dir name then name else String.lowercase name
+               with Already_lowercase ->
+                  name)
 
    let compare = Lm_string_util.string_compare
    let equal (s1: string) s2 = (s1 = s2)
