@@ -10,16 +10,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -81,12 +81,17 @@ type omake_options =
      opt_cd_root              : bool;
      opt_project              : bool;
      opt_poll                 : bool setting;
+     opt_osh                  : bool;
      opt_poll_on_done         : bool;
      opt_flush_include        : bool;
      opt_flush_static         : bool;
      opt_allow_exceptions     : bool;
      opt_absname              : bool;
      opt_output               : (output_flag * bool) list;
+
+     (* Warnings *)
+     opt_warn_declare        : bool;
+     opt_warn_error          : bool
    }
 
 let opt_job_count opts =
@@ -267,6 +272,12 @@ let opt_poll opts =
 let set_poll_opt opts b =
    { opts with opt_poll = Set b }
 
+let opt_osh opts =
+   opts.opt_osh
+
+let set_osh_opt opts =
+   { opts with opt_osh = true }
+
 let opt_terminate_on_error opts =
    match opts.opt_terminate_on_error with
       Set v  -> v
@@ -299,6 +310,18 @@ let opt_absname opts =
 let set_absname_opt opts flag =
    { opts with opt_absname = flag }
 
+let opt_warn_declare opts =
+   opts.opt_warn_declare
+
+let set_warn_declare_opt opts flag =
+   { opts with opt_warn_declare = flag }
+
+let opt_warn_error opts =
+   opts.opt_warn_error
+
+let set_warn_error_opt opts flag =
+   { opts with opt_warn_error = flag }
+
 (*
  * Output control.
  *)
@@ -410,11 +433,14 @@ let default_options =
      opt_project              = false;
      opt_poll                 = Default;
      opt_poll_on_done         = false;
+     opt_osh                  = false;
      opt_flush_include        = false;
      opt_flush_static         = false;
      opt_allow_exceptions     = false;
      opt_absname              = false;
      opt_output               = [];
+     opt_warn_declare         = false;
+     opt_warn_error           = false;
    }
 
 (*
@@ -458,7 +484,12 @@ let options_spec =
     "--verbose-dependencies", Lm_arg.SetFold set_verbose_dependencies_opt, (**)
        "For --print-dependencies and --show-dependencies, print all dependencies too";
     "--absname", Lm_arg.SetFold set_absname_opt, (**)
-       "Filenames are always displayed as absolute paths"]
+       "Filenames are always displayed as absolute paths";
+    "-Wdeclare", Lm_arg.SetFold set_warn_declare_opt, (**)
+       "Warn about undeclared variables";
+    "-warn-error", Lm_arg.SetFold set_warn_error_opt, (**)
+       "Treat warnings as errors"
+   ]
 
 let progress_usage =
    match Sys.os_type with
