@@ -1,25 +1,25 @@
 (*
- * Virtual identifiers for verious things in files.
+ * Virtual identifiers for various things in files.
  *
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2005 Mojave Group, Caltech
+ * Copyright (C) 2005-2007 Mojave Group, Caltech
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -28,15 +28,6 @@
  * @email{jyh@cs.caltech.edu}
  * @end[license]
  *)
-open Lm_hash
-open Lm_printf
-open Lm_symbol
-
-open Omake_node
-open Omake_util
-open Omake_state
-open Omake_symbol
-open Omake_print_util
 
 (*!
  * @docoff
