@@ -6,19 +6,24 @@
  * @begin[license]
  * Copyright (C) 2004-2006 Mojave group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * 
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
  *
  * Author: Jason Hickey @email{jyh@cs.caltech.edu}
  * Modified by: Aleksey Nogin @email{nogin@cs.caltech.edu}
@@ -33,6 +38,12 @@
 #include <caml/fail.h>
 #include <caml/custom.h>
 
+#ifdef WIN32 
+#include <windows.h>
+/* Disable some of the warnings */
+#pragma warning( disable : 4100 )
+#endif /* WIN32 */
+
 /*
  * Character classes.
  */
@@ -46,7 +57,7 @@ static value get_chars(int (*f)(int))
     p = buf;
     for(i = 0; i != 256; i++) {
         if(f(i))
-            *p++ = i;
+            *p++ = (char) i;
     }
     s = alloc_string(p - buf);
     memcpy(String_val(s), buf, p - buf);
