@@ -3,21 +3,26 @@
  *
  * ----------------------------------------------------------------
  *
- * Copyright (C) 1999-2005 Jason Hickey, Caltech
+ * Copyright (C) 1999-2005 Mojave Group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
  *
  * Author: Jason Hickey
  * jyh@cs.caltech.edu
@@ -28,6 +33,31 @@
  *)
 exception Unchanged
 
+(*
+ * Array-style operations.
+ *)
+let sub =
+   let rec skip l off len =
+      if off = 0 then
+         collect [] l len
+      else
+         match l with
+            _ :: l ->
+               skip l (off - 1) len
+          | [] ->
+               raise (Invalid_argument "Lm_list_util.sub")
+   and collect l1 l2 len =
+      if len = 0 then
+         List.rev l1
+      else
+         match l2 with
+            x :: l2 ->
+               collect (x :: l1) l2 (len - 1)
+          | [] ->
+               raise (Invalid_argument "Lm_list_util.sub")
+   in
+      skip
+
 (*
  * Iterate over two lists, but stop at the end of the
  * shorter one.
@@ -311,6 +341,26 @@ let rec split i l =
        | [] ->
             raise (Invalid_argument "split")
 
+(*
+ * Split into fragments.
+ *)
+let splitup n l =
+   let rec split i vll vl1 vl2 =
+      match vl2 with
+         v :: vl2 ->
+            if i = 0 then
+               split n (List.rev vl1 :: vll) [v] vl2
+            else
+               split (pred i) vll (v :: vl1) vl2
+       | [] ->
+            List.rev vl1 :: vll
+   in
+      match l with
+         [] ->
+            []
+       | h :: t ->
+            split n [] [h] t
+
 (*
  * Get the last element.
  *)
