@@ -179,6 +179,20 @@ struct
    external ext_print_int : string -> int -> string       = "ml_print_int"
    external ext_print_float : string -> float -> string   = "ml_print_float"
    external ext_print_string : string -> string -> string = "ml_print_string"
+   external ext_print_string2 : int -> string -> string -> string = "ml_print_string2"
+
+   (*
+    * Maximum width specified in the format.
+    *)
+   let max_width info =
+      match info with
+         { field_width = Some width; field_precision = Some pre } ->
+            max width pre
+       | { field_width = Some width; field_precision = None }
+       | { field_width = None; field_precision = Some width } ->
+            width
+       | { field_width = None; field_precision = None } ->
+            0
 
    (*
     * Next arg should be an int.
@@ -215,9 +229,9 @@ struct
       in
          Obj.magic print
 
-   and print_string buf i len s fmt _info =
+   and print_string buf i len s fmt info =
       let print str =
-         let str = ext_print_string fmt str in
+         let str = ext_print_string2 (max_width info) fmt str in
             Args.print_string buf str;
             print_loop buf i len s
       in
