@@ -20,21 +20,26 @@
  *
  * ----------------------------------------------------------------
  *
- * Copyright (C) 1999-2005 Jason Hickey, Caltech
+ * Copyright (C) 1999-2005 Mojave Group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
  *
  * Author: Jason Hickey
  * jyh@cs.caltech.edu
@@ -1228,7 +1233,7 @@ struct
    (*
     * Search without reorganizing the tree.
     *)
-   let rec mem_aux tree key =
+   let rec mem tree key =
       match tree with
          Black (key', _, left, right, _)
        | Red (key', _, left, right, _) ->
@@ -1236,15 +1241,27 @@ struct
                if comp = 0 then
                   true
                else if comp < 0 then
-                  mem_aux left key
+                  mem left key
                else
-                  mem_aux right key
+                  mem right key
 
        | Leaf ->
             false
 
-   let mem tree key =
-      mem_aux tree key
+   let rec find_key tree key =
+      match tree with
+         Black (key', _, left, right, _)
+       | Red (key', _, left, right, _) ->
+            let comp = Base.compare key key' in
+               if comp = 0 then
+                  Some key'
+               else if comp < 0 then
+                  find_key left key
+               else
+                  find_key right key
+
+       | Leaf ->
+            None
 
    (*
     * An empty tree is just a leaf.
@@ -1464,6 +1481,8 @@ struct
 
    let mem = MMap.mem
 
+   let find_key = MMap.find_key
+
    let iter f t =
       MMap.iter (fun i l ->
             match l with
@@ -1583,7 +1602,7 @@ struct
       if MMap.cardinality set1 = MMap.cardinality set2 then
          let list1 = MMap.to_list set1 in
          let list2 = MMap.to_list set2 in
-            List.for_all2 (fun (x1, x2) (y1, y2) ->
+            List.for_all2 (fun (_x1, x2) (_y1, y2) ->
                   List.length x2 = List.length y2 && List.for_all2 eq x2 y2) list1 list2
       else
          false
