@@ -3,21 +3,26 @@
  *
  * ----------------------------------------------------------------
  *
- * Copyright (C) 2000-2005 Jason Hickey, Caltech
+ * Copyright (C) 2000-2006 Mojave Group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
  *
  * Author: Jason Hickey <jyh@cs.caltech.edu>
  * Modified By: Aleksey Nogin <nogin@cs.cornell.edu>
@@ -59,6 +64,17 @@ let string_compare s1 s2 =
          0 -> string_compare_aux s1 s2 len1 0
        | i -> i
 
+(*
+ * String prefix.
+ *)
+let rec is_string_prefix_aux s1 s2 i len =
+   i = len || (String.unsafe_get s1 i = String.unsafe_get s2 i && is_string_prefix_aux s1 s2 (i + 1) len)
+
+let is_string_prefix s1 s2 =
+   let len1 = String.length s1 in
+   let len2 = String.length s2 in
+      len1 <= len2 && is_string_prefix_aux s1 s2 0 len1
+
 (*
  * Compare a substring.
  *)
@@ -248,6 +264,42 @@ let js_escaped s =
 (*
  * Escape a string using the HTML conventions.
  *)
+let html_pre_escaped s =
+   let len = String.length s in
+   let buf = Buffer.create len in
+   let rec loop i =
+      if i = len then
+         Buffer.contents buf
+      else
+         let c = s.[i] in
+         let _ =
+            match c with
+               '<' ->
+                  Buffer.add_string buf "&lt;"
+             | '>' ->
+                  Buffer.add_string buf "&gt;"
+             | '&' ->
+                  Buffer.add_string buf "&amp;"
+             | '"' ->
+                  Buffer.add_string buf "&quot;"
+             | ' '
+             | '\r'
+             | '\n'
+             | '\t' ->
+                  Buffer.add_char buf c
+             | _ ->
+                  if c < ' ' || c >= '\127' then begin
+                      Buffer.add_string buf "&#";
+                      Buffer.add_string buf (string_of_int (Char.code c));
+                      Buffer.add_char buf ';'
+                  end
+                  else
+                      Buffer.add_char buf c
+         in
+            loop (succ i)
+   in
+      loop 0
+
 let html_escaped s =
    let len = String.length s in
    let buf = Buffer.create len in
@@ -582,6 +634,12 @@ let tokens_data info s =
 let tokens_string info s =
    let len = String.length s in
    let wrap = info.tokens_wrap_string in
+   let wrap_prefix prefix s off len =
+      if len <> 0 then
+         wrap (String.sub s off len) :: prefix
+      else
+         prefix
+   in
 
    (* Scanning whitespace *)
    let rec scan_white tokens i =
@@ -603,9 +661,9 @@ let tokens_string info s =
    (* Scanning a quoted word *)
    and scan_quote tokens prefix delim start i =
       if i >= len then
-         let head = wrap (String.sub s start (len - start)) in
+         let head = wrap_prefix prefix s start (len - start) in
             { info with tokens_list = tokens;
-                        tokens_prefix = QuotePrefix (delim, (head :: prefix))
+                        tokens_prefix = QuotePrefix (delim, head)
             }
       else
          let c = String.unsafe_get s i in
@@ -620,14 +678,15 @@ let tokens_string info s =
    (* Scanning a word *)
    and scan_word tokens prefix start i =
       if i >= len then
-         { info with tokens_list = tokens;
-                     tokens_prefix = WordPrefix ((wrap (String.sub s start (len - start))) :: prefix)
-         }
+         let prefix = wrap_prefix prefix s start (len - start) in
+            { info with tokens_list = tokens;
+                        tokens_prefix = WordPrefix prefix
+            }
       else
          match String.unsafe_get s i with
             ' ' | '\t' | '\n' | '\r' | '\012' ->
-               let head = wrap (String.sub s start (i - start)) in
-               let head_tok = info.tokens_group (head :: prefix) in
+               let head = wrap_prefix prefix s start (i - start) in
+               let head_tok = info.tokens_group head in
                   scan_white (head_tok :: tokens) (succ i)
           | '"' | '\'' as c ->
                scan_quote tokens prefix c start (succ i)
@@ -700,7 +759,22 @@ let tokens_lex info s =
    in
    let len = String.length s in
 
-   (* Scanning whitespace *)
+   (* Don't add empty strings *)
+   let wrap_data_prefix prefix s off len =
+      if len <> 0 then
+         wrap_data (String.sub s off len) :: prefix
+      else
+         prefix
+   in
+
+   let wrap_string_prefix prefix s off len =
+      if len <> 0 then
+         wrap_string (String.sub s off len) :: prefix
+      else
+         prefix
+   in
+
+  (* Scanning whitespace *)
    let rec scan_white tokens i =
       if i = len then
          { info with tokens_list   = tokens;
@@ -723,15 +797,14 @@ let tokens_lex info s =
    (* Scanning a quoted word *)
    and scan_quote tokens prefix delim start i =
       if i >= len then
-         let head = wrap_data (String.sub s start (len - start)) in
+         let head = wrap_data_prefix prefix s start (len - start) in
             { info with tokens_list   = tokens;
-                        tokens_prefix =  QuotePrefix (delim, head :: prefix)
+                        tokens_prefix =  QuotePrefix (delim, head)
             }
       else
          match buffer_get_quoted s i with
             BufQuote c when c = delim ->
-               eprintf "Got '%s'@." (String.sub s start (i - start));
-               let prefix = wrap_data (String.sub s start (i - start)) :: prefix in
+               let prefix = wrap_data_prefix prefix s start (i - start) in
                   scan_word tokens prefix (succ i) (succ i)
           | BufBackslash ->
                scan_quote tokens prefix delim start (i + 2)
@@ -745,23 +818,21 @@ let tokens_lex info s =
    (* Scanning a word *)
    and scan_word tokens prefix start i =
       if i >= len then
-         let head = wrap_string (String.sub s start (len - start)) in
+         let head = wrap_string_prefix prefix s start (len - start) in
             { info with tokens_list   = tokens;
-                        tokens_prefix = WordPrefix (head :: prefix)
+                        tokens_prefix = WordPrefix head
             }
       else
          match buffer_get_token lexer s i len with
             BufWhite ->
-               let head = wrap_string (String.sub s start (i - start)) in
-               let head = group (head :: prefix) in
+               let head = group (wrap_string_prefix prefix s start (i - start)) in
                   scan_white (head :: tokens) (succ i)
           | BufToken len ->
-               let head1 = wrap_string (String.sub s start (i - start)) in
-               let head1 = group (head1 :: prefix) in
+               let head1 = group (wrap_string_prefix prefix s start (i - start)) in
                let head2 = wrap_token (String.sub s i len) in
                   scan_white (head2 :: head1 :: tokens) (i + len)
           | BufQuote c ->
-               let prefix = wrap_string (String.sub s start (i - start)) :: prefix in
+               let prefix = wrap_string_prefix prefix s start (i - start) in
                   scan_quote tokens prefix c (succ i) (succ i)
           | BufBackslash ->
                scan_word tokens prefix start (i + 2)
@@ -1246,39 +1317,24 @@ let rec needs_quotes mode s i len =
    if i >= len then
       mode <> ModeNormal
    else
-      match mode with
-         ModeNormal ->
-            (match s.[i] with
-               ' '
-             | '\t'
-             | '\012'
-             | '\n'
-             | '\r' ->
-                  true
-             | '"' ->
-                  needs_quotes ModeDouble s (succ i) len
-             | '\'' ->
-                  needs_quotes ModeSingle s (succ i) len
-             | '\\' ->
-                  needs_quotes ModeNormal s (i + 2) len
-             | _ ->
-                  needs_quotes ModeNormal s (succ i) len)
-       | ModeSingle ->
-            (match s.[i] with
-                '\'' ->
+      match mode, s.[i] with
+         _, '\\' ->
+            needs_quotes mode s (i + 2) len
+       | ModeNormal, ' '
+       | ModeNormal, '\t'
+       | ModeNormal, '\012'
+       | ModeNormal, '\n'
+       | ModeNormal, '\r' ->
+            true
+       | ModeNormal, '"' ->
+            needs_quotes ModeDouble s (succ i) len
+       | ModeNormal, '\'' ->
+            needs_quotes ModeSingle s (succ i) len
+       | ModeSingle, '\''
+       | ModeDouble, '"' ->
                   needs_quotes ModeNormal s (succ i) len
-              | '\\' ->
-                  needs_quotes ModeSingle s (i + 2) len
-              | _ ->
-                  needs_quotes ModeSingle s (succ i) len)
-       | ModeDouble ->
-           (match s.[i] with
-               '"' ->
-                  needs_quotes ModeNormal s (succ i) len
-             | '\\' ->
-                  needs_quotes ModeDouble s (i + 2) len
-             | _ ->
-                  needs_quotes ModeDouble s (succ i) len)
+       | _ ->
+            needs_quotes mode s (succ i) len
 
 let needs_quotes s =
    let len = String.length s in
@@ -1311,6 +1367,11 @@ let quotify buf s =
       copy 0;
       Buffer.add_char buf dquote
 
+let shell_quotes s =
+   let buf = Buffer.create 32 in
+      quotify buf s;
+      Buffer.contents buf
+
 let quote buf s =
    if needs_quotes s then
       quotify buf s
@@ -1444,7 +1505,7 @@ let encode_hex_name uri =
          String.sub buf 0 j
       else
          match uri.[i] with
-            ('0'..'9' | 'A'..'Z' | 'a'..'z') as c ->
+            ('0'..'9' | 'A'..'Z' | 'a'..'z' | '/' | '_' | '-' | '.') as c ->
                buf.[j] <- c;
                convert (succ i) (succ j)
           | c ->
@@ -1459,7 +1520,6 @@ let encode_hex_name uri =
 (*
  * -*-
  * Local Variables:
- * Caml-master: "compile"
  * End:
  * -*-
  *)
