@@ -3,24 +3,29 @@
  *
  * ----------------------------------------------------------------
  *
- * Copyright (C) 2000-2005 Jason Hickey, Caltech
+ * Copyright (C) 2000-2006 Mojave Group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * Author: Jason Hickey
- * jyh@cs.caltech.edu
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
+ *
+ * Author: Jason Hickey <jyh@cs.caltech.edu>
+ * Modified By: Aleksey Nogin <nogin@cs.caltech.edu>
  *)
 
 (*
@@ -38,6 +43,11 @@ val string_compare : string -> string -> int
  *)
 val equal_substring : string -> int -> string -> bool
 
+(*
+ * Check whether the first string is a prefix of the second.
+ *)
+val is_string_prefix : string -> string -> bool
+
 (*
  * Hex representation of a string.
  *)
@@ -119,6 +129,7 @@ val c_escaped : string -> string
  *)
 val js_escaped : string -> string
 val html_escaped : string -> string
+val html_pre_escaped : string -> string
 val html_escaped_nonwhite : string -> string
 
 (*
@@ -188,6 +199,9 @@ val tokens_flush  : 'a tokens -> 'a list
 val parse_args_list : string -> string list list
 val parse_args : string -> string list
 
+(* Add outer quotes to a string, and escape all the inner quotes. *)
+val shell_quotes : string -> string
+
 (*
  * Reconstruct an argv string from a list of strings.
  * The strings are concatenated with intervening whitespace.
@@ -262,7 +276,6 @@ val encode_hex_name : string -> string
 (*
  * -*-
  * Local Variables:
- * Caml-master: "compile"
  * End:
  * -*-
  *)
