@@ -7,25 +7,30 @@
  *
  * ----------------------------------------------------------------
  *
- * Copyright (C) 2002, Justin David Smith, Caltech
- * Based on original code, Copyright (C) 2000-2005 Jason Hickey, Caltech
+ * Copyright (C) 2000-2006 Mojave Group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * Author: Jason Hickey
- * jyh@cs.caltech.edu
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
+ *
+ * Authors: Jason Hickey <jyh@cs.caltech.edu>
+ *          Justin David Smith
+ * Modified By: Aleksey Nogin <nogin@cs.caltech.edu>
  *)
 open Lm_printf
 
@@ -192,11 +197,39 @@ let char_table_lookup table ch =
             None
 
 
+(* lookup_option
+   We also allow --no-* prefixes on Boolean options.
+
+   JYH: this is perhaps not the simplest way to deal
+   with inversion, but the implementation is simple.  *)
+let is_invert_prefix name =
+   String.length name > 5
+   && String.unsafe_get name 0 = '-'
+   && String.unsafe_get name 1 = '-'
+   && String.unsafe_get name 2 = 'n'
+   && String.unsafe_get name 3 = 'o'
+   && String.unsafe_get name 4 = '-'
+
+let strip_invert_prefix name =
+   String.sub name 4 (String.length name - 4)
+
+let is_invertable_option opt = function
+   Set _
+ | Clear _
+ | SetFold _
+ | ClearFold _ ->
+      String.length opt > 1 && opt.[0] = '-'
+ | _ ->
+      false
+
+
 (* add_option
    Add a new option name to the option tree.  If the exact option already
    exists, then an exception is thrown.  If a prefix or suffix of this
    option is already defined, then no error occurs.  *)
 let add_option options name spec =
+   if is_invert_prefix name then
+      raise (BogusArg ("Option contains an invertion prefix: " ^ name));
    let length = String.length name in
 
    (* deconstruct_name
@@ -238,7 +271,7 @@ let add_option options name spec =
       deconstruct_name options 0
 
 
-(* lookup_option
+(* lookup_option_core
    Lookup the option with the indicated name in the options tree.  If there
    is an exact option match in the tree, we return the option spec and an
    empty string.  If we hit end up at a node without a spec, but we are an
@@ -252,7 +285,7 @@ let add_option options name spec =
    space.  Note that any option that is a prefix of another option cannot
    take a value in this way.
  *)
-let lookup_option options name =
+let lookup_option_core options name =
    let length = String.length name in
 
    (* find_branch
@@ -293,6 +326,9 @@ let lookup_option options name =
                None ->
                   (* No option with this prefix was defined *)
                   raise (BogusArg ("No such option: " ^ name))
+             | Some (SpecNode (Unit _ | Set _ | Clear _ | UnitFold _ | SetFold _ | ClearFold _ | Usage )) ->
+                  (* Name was too long; can not assume a name/value pair *)
+                  raise (BogusArg ("No such option: " ^ name ^ " (option " ^ (String.sub name 0 offset) ^ " does not take arguments)"))
              | Some (SpecNode spec) ->
                   (* Name was too long; assume it was a name/value pair *)
                   spec, String.sub name offset (length - offset)
@@ -319,6 +355,30 @@ let lookup_option options name =
    in
       lookup_name options 0
 
+let lookup_option options name =
+   if is_invert_prefix name then
+      let orig_name = strip_invert_prefix name in
+         try
+            match lookup_option_core options orig_name with
+               Set f, "" ->
+                  Clear f, ""
+             | SetFold f, "" ->
+                  ClearFold f, ""
+             | Clear f, "" ->
+                  Set f, ""
+             | ClearFold f, "" ->
+                  SetFold f, ""
+             | _ ->
+                  raise (Failure "invert")
+         with
+            BogusArg _
+          | Not_found ->
+               raise (BogusArg ("No such option: " ^ orig_name ^ " (extracted from inverted: " ^ name ^ ")"))
+          | Failure "invert" ->
+               raise (BogusArg ("Not an invertable option: " ^ orig_name ^ " (extracted from inverted: " ^ name ^ ")"))
+   else
+      lookup_option_core options name
+
 
 (* compute_option_tree
    Convert a sections spec into an option tree.  Can raise an exception
@@ -334,58 +394,90 @@ let compute_option_tree spec =
 
 (***  Help System  ***)
 
+(* Wraps at terminal width *)
+let rec print_doc_string opt_width s =
+   let width = Lm_termsize.stdout_width - opt_width in
+   let margin = String.make (opt_width + 1) ' ' in
+   let () = margin.[0] <- '\n' in
+   let len = String.length s in
+      if len <= width then
+         print_string s
+      else
+         if String.rcontains_from s width ' ' then begin
+            let i = String.rindex_from s width ' ' in
+               print_string (String.sub s 0 i);
+               print_string margin;
+               print_doc_string opt_width (String.sub s (i+1) (len - i - 1))
+         end else begin
+            print_string (String.sub s 0 width);
+            print_string margin;
+            print_doc_string opt_width (String.sub s width (len - width))
+         end
+
+let usage_arg = function
+   Unit _
+ | Set _
+ | Clear _
+ | UnitFold _
+ | SetFold _
+ | ClearFold _
+ | Usage ->
+      ""
+ | String _
+ | StringFold _ ->
+      " <string>"
+ | Int _
+ | IntFold _ ->
+      " <number>"
+ | Float _
+ | FloatFold _ ->
+      " <float>"
+ | Rest _
+ | RestFold _ ->
+      " ..."
 
 (* usage
    Display the usage message and help text for the options.  *)
-let usage spec =
+let usage opt_width spec =
    List.iter (fun (opt, spec, doc) ->
          (* Descriptive text for the option argument *)
-         let arg =
-            match spec with
-               Unit _
-             | Set _
-             | Clear _
-             | UnitFold _
-             | SetFold _
-             | ClearFold _
-             | Usage ->
-                  ""
-             | String _
-             | StringFold _ ->
-                  " <string>"
-             | Int _
-             | IntFold _ ->
-                  " <number>"
-             | Float _
-             | FloatFold _ ->
-                  " <float>"
-             | Rest _
-             | RestFold _ ->
-                  " ..."
-         in
-         let opt = opt ^ arg in
+         let opt = opt ^ (usage_arg spec) in
 
             (* Display information on a single option. *)
-            if String.length opt > 20 then
+            (if String.length opt > opt_width then
                (* option name too long to fit on one line *)
-               printf "@ %s@ %20s" opt ""
+               printf "@ %s@ %*s" opt opt_width ""
             else
-               printf "@ %-20s" opt;
-            printf ":  ";
-            print_string doc) spec
+               printf "@ %-*s" opt_width opt);
+            (if is_invertable_option opt spec then
+               printf "*:  "
+            else
+               printf " :  ");
+            print_doc_string (opt_width + 7) doc) spec
+
+let usage_length (opt, spec, _) =
+   String.length opt + String.length (usage_arg spec)
 
 let usage (mode, spec) usage_msg =
    (* Display help for all sections. *)
+   let opt_max_length =
+      List.fold_left (fun i (_, spec) ->
+         (List.fold_left (fun i opt -> max i (usage_length opt))) i spec)
+      0 spec
+   in
+   let opt_width = min opt_max_length ((max 80 Lm_termsize.stdout_width) / 3 - 7) in
    printf "@[<v 0>%s." usage_msg;
+   List.iter (fun (section, spec) ->
+      printf "@ @ @[<v 3>%s:" section;
+      usage opt_width spec;
+      printf "@]") spec;
    (match mode with
        StrictOptions ->
           ()
      | MultiLetterOptions ->
           printf "@ Single-letter options may be concatenated as part of a single option.");
-   List.iter (fun (section, spec) ->
-      printf "@ @ @[<v 3>%s:" section;
-      usage spec;
-      printf "@]") spec;
+   (if List.exists (fun (_, spec) -> List.exists (fun (opt, spec, _) -> is_invertable_option opt spec) spec) spec then
+      printf "@ @ (*) Prefix the option with \"--no\" to disable.");
    printf "@]@."
 
 
@@ -395,7 +487,7 @@ let usage (mode, spec) usage_msg =
    Query for pending arguments or options.  Advances the parser for
    the current mode, and returns a pair (mode, found), where found
    is true iff there are options or arguments left to process.  *)
-let advance_options mode argv argv_length current =
+let advance_options mode _argv argv_length current =
    match mode with
       StrictMode
     | MultiLetterMode ->
@@ -409,11 +501,14 @@ let advance_options mode argv argv_length current =
 (* get_next_arg
    Get the next argument in the argument stream.  Returns
    the argument string, as well as the new current marker.  *)
-let get_next_arg argv argv_length current =
+let get_next_arg opt argv argv_length current =
    if current < argv_length then
       argv.(current), current + 1
    else
-      raise (BogusArg "Missing argument")
+      if (opt <> "") then
+         raise (BogusArg ("Option " ^ opt ^ " requires an argument"))
+      else
+         raise (Invalid_argument "Lm_arg: internal error")
 
 (* get_next_option
    In StrictMode, this is the same as get_next_arg.
@@ -422,11 +517,11 @@ let get_next_arg argv argv_length current =
 let rec get_next_option mode argv argv_length current =
    match mode with
       StrictMode ->
-         let opt, current = get_next_arg argv argv_length current in
+         let opt, current = get_next_arg "" argv argv_length current in
             opt, current, mode
     | MultiLetterMode ->
          (* See if the next argument is an option *)
-         let opt, current = get_next_arg argv argv_length current in
+         let opt, current = get_next_arg "" argv argv_length current in
             if String.length opt >= 2 && opt.[0] = '-' && is_alnum opt.[1] then
                get_next_option (MultiLetterPending (opt, 1)) argv argv_length current
             else
@@ -487,7 +582,7 @@ let fold_argv argv (mode_info, spec_info) arg default usage_msg =
                       | StringFold _, ""
                       | IntFold _,    ""
                       | FloatFold _,  "" ->
-                           let s, current = get_next_arg argv argv_length current in
+                           let s, current = get_next_arg opt argv argv_length current in
                               s, current, arg
 
                       | Unit _,       ""
@@ -525,7 +620,7 @@ let fold_argv argv (mode_info, spec_info) arg default usage_msg =
                            in
                               rest_function arg current
                       | _ ->
-                           raise (BogusArg "Option cannot accept an argument")
+                           raise (Invalid_argument "Lm_arg: internal error")
                   in
 
                   (* Actually process the option. *)
@@ -540,7 +635,7 @@ let fold_argv argv (mode_info, spec_info) arg default usage_msg =
                            x := true;
                            arg
                       | SetFold f ->
-                           f arg true
+                           f arg true;
                       | Clear x ->
                            x := false;
                            arg
