@@ -23,19 +23,24 @@
  * @begin[license]
  * Copyright (C) 2004 Mojave Group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
  *
  * Author: Jason Hickey
  * @email{jyh@cs.caltech.edu}
@@ -136,6 +141,8 @@ let getpwnam user =
       tilde_insert dir user;
       dir
 
+let gethomedir = getpwnam
+
 (*
  * Try to figure out the home directory as best as possible.
  *)
@@ -144,6 +151,19 @@ let home_dir =
       tilde_insert home "";
       home
 
+(*
+ * Get a list of all the users.
+ *)
+let getusers () =
+   let users = Lm_unix_util.getpwents () in
+      List.map (fun entry ->
+            let { Unix.pw_name = name;
+                  Unix.pw_dir  = dir
+                } = entry
+            in
+               tilde_insert dir name;
+               name) users
+
 (************************************************************************
  * Glob expansion.
  *)
@@ -154,6 +174,7 @@ type glob_option =
  | GlobNoCheck                        (* If an expansion fails, return the expansion literally *)
  | GlobIgnoreCheck                    (* If an expansion fails, it expands to nothing *)
  | GlobDot                            (* Allow wildcards to match filenames with a leading . *)
+ | GlobOnlyFiles                      (* Return only non-directories in the result *)
  | GlobOnlyDirs                       (* Return only directories in the result *)
  | GlobCVSIgnore                      (* Ignore files as specified by .cvsignore files *)
  | GlobIgnore of string list          (* Ignore the files that match the pattern *)
@@ -174,6 +195,7 @@ type glob_options =
      glob_escape    : bool;
      glob_check     : glob_check;
      glob_dot       : bool;
+     glob_files     : bool;
      glob_dirs      : bool;
      glob_cvs       : bool;
      glob_ignore    : (string -> bool);
@@ -189,6 +211,7 @@ let default_glob_options =
      glob_escape    = true;
      glob_check     = NoMatchError;
      glob_dot       = false;
+     glob_files     = false;
      glob_dirs      = false;
      glob_cvs       = false;
      glob_ignore    = (fun _ -> false);
@@ -522,6 +545,7 @@ let create_options l =
                 | GlobNoCheck     -> { options with glob_check = NoMatchPreserve }
                 | GlobIgnoreCheck -> { options with glob_check = NoMatchIgnore }
                 | GlobDot         -> { options with glob_dot = true }
+                | GlobOnlyFiles   -> { options with glob_files = true }
                 | GlobOnlyDirs    -> { options with glob_dirs = true }
                 | GlobCVSIgnore   -> { options with glob_cvs = true }
                 | GlobIgnoreFun f -> { options with glob_ignore = f }
@@ -627,6 +651,7 @@ let glob_dir_pattern options root dirs names dir pattern =
                let dir_flag = is_dir root_name in
                let dirs, names =
                   if (options.glob_dot || name.[0] <> '.')
+                     && (not options.glob_files || not dir_flag)
                      && (not options.glob_dirs || dir_flag)
                      && LmStr.string_match pattern name 0
                      && not (options.glob_ignore name)
