@@ -6,19 +6,24 @@
  * @begin[license]
  * Copyright (C) 2004 Mojave Group, Caltech
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation,
+ * version 2.1 of the License.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Additional permission is given to link this library with the
+ * OpenSSL project's "OpenSSL" library, and with the OCaml runtime,
+ * and you may distribute the linked executables.  See the file
+ * LICENSE.libmojave for more details.
  *
  * Author: Jason Hickey
  * @email{jyh@cs.caltech.edu}
@@ -33,6 +38,7 @@ type glob_option =
  | GlobNoCheck          (* If an expansion fails, return the expansion literally *)
  | GlobIgnoreCheck      (* If an expansion fails, it expands to nothing *)
  | GlobDot              (* Allow wildcards to match filenames with a leading . *)
+ | GlobOnlyFiles        (* Return only non-directories in the result *)
  | GlobOnlyDirs         (* Return only directories in the result *)
  | GlobCVSIgnore        (* Ignore files as specified by .cvsignore files *)
  | GlobIgnore of string list  (* Ignore the files that match the pattern *)
@@ -53,6 +59,17 @@ val default_glob_options : glob_options
  *)
 val home_dir : string
 
+(*
+ * Get a list of all the users in the system.
+ * On non-unix systems, returns the empty list.
+ *)
+val getusers : unit -> string list
+
+(*
+ * Get the home directory for a user.
+ *)
+val gethomedir : string -> string
+
 (*
  * Try to collapse a filename.
  * Tilde-expansion will invert this process.
