@@ -4,24 +4,28 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2003 Jason Hickey, Caltech
+ * Copyright (C) 2003-2006 Mojave Group, Caltech
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
+ * as published by the Free Software Foundation; version 2
+ * of the License.
+ * 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * 
+ * Additional permission is given to link this library with the
+ * with the Objective Caml runtime, and to redistribute the
+ * linked executables.  See the file LICENSE.OMake for more details.
  *
- * Author: Jason Hickey
- * @email{jyh@cs.caltech.edu}
+ * Author: Jason Hickey @email{jyh@cs.caltech.edu}
+ * Modified by: Aleksey Nogin @email{nogin@cs.caltech.edu}
  * @end[license]
  *)
 open Printf
@@ -42,8 +46,9 @@ let output_file = ref None
 let cache_files = ref []
 let omc_files   = ref []
 let omo_files   = ref []
-let libdir      = ref "/unknown"
+let libdir      = ref None
 let version_txt = ref "version.txt"
+let default_save_interval = ref 15.0
 
 let anon s =
    let p =
@@ -62,12 +67,14 @@ let anon s =
 
 let spec =
    ["--magic",       Arg.Set make_magic, "generate the omake_magic.ml file";
-    "--lib",         Arg.String (fun s -> libdir := s), "specify the location of the library directory";
+    "--lib",         Arg.String (fun s -> libdir := Some s), "specify the location of the library directory";
     "--root",        Arg.String (fun s -> make_root := Some s),  "generate the OMakeroot file";
     "--cache-files", Arg.Unit (fun () -> mode := CacheFiles), "specify the magic files for the cache magic number";
     "--omc-files",   Arg.Unit (fun () -> mode := OmcFiles), "specify the files to scan for the IR magic number";
     "--omo-files",   Arg.Unit (fun () -> mode := OmoFiles), "specify the files to scan for the object magic number";
     "--version",     Arg.String (fun s -> version_txt := s), "specify the version.txt file";
+    "--default_save_interval", 
+         Arg.Float (fun f -> default_save_interval := f), "specify the default .omakedb save interval";
     "-o",            Arg.String (fun s -> output_file := Some s), "set the output file"]
 
 let usage = "Generate special files"
@@ -96,7 +103,14 @@ let read_version () =
             exit 1
    in
       close_in inx;
-      version
+      if Lm_string_util.contains_any version "()$\" " then
+         version
+      else if (String.contains version '-') then
+         let dash = String.index version '-' in
+         let release = String.sub version (dash + 1) ((String.length version) - dash - 1) in
+            sprintf "%s (release %s)" (String.sub version 0 dash) release
+      else
+         version ^ " (development snapshot)"
 
 (************************************************************************
  * File copying.
@@ -224,8 +238,18 @@ let mon_names =
 
 let digest_len = 4 + String.length (Digest.string "hello world")
 
+let shorten_version s =
+   if Lm_string_util.contains_any s " -" then
+      String.sub s 0 (Lm_string_util.index_set s " -")
+   else
+      s
+
 let omake_magic buf =
-   let s = Filename.concat !libdir "omake" in
+   let libdir = 
+      match !libdir with 
+         Some s -> Filename.concat s "omake"
+       | None -> Filename.concat (Filename.dirname (Unix.getcwd ())) "lib"
+   in
    let version = read_version () in
    let now = Unix.time () in
    let { Unix.tm_year = year;
@@ -237,15 +261,16 @@ let omake_magic buf =
          Unix.tm_sec = sec
        } = Unix.localtime now
    in
+      fprintf buf "let default_save_interval = %F\n" !default_save_interval;
       fprintf buf "let input_magic inx = let s = String.make %d ' ' in really_input inx s 0 %d; s\n" digest_len digest_len;
       fprintf buf "let output_magic = output_string\n";
       fprintf buf "let cache_magic = \"%s\"\n" (digest_files ".cache.magic" ".odb" !cache_files);
       fprintf buf "let ir_magic = \"%s\"\n"    (digest_files ".omc.magic" ".omc" !omc_files);
       fprintf buf "let obj_magic = \"%s\"\n"   (digest_files ".omo.magic" ".omo" !omo_files);
-      fprintf buf "let lib_dir = \"%s\"\n" (String.escaped s);
-      fprintf buf "let version = \"%s\"\n" (String.escaped version);
+      fprintf buf "let lib_dir = \"%s\"\n" (String.escaped libdir);
+      fprintf buf "let version = \"%s\"\n" (String.escaped (shorten_version version));
       fprintf buf "let version_message = \"OMake %s:\\n\\tbuild [%s %s %d %02d:%02d:%02d %d]\\n\\ton %s\"\n"
-         version
+         (String.escaped version)
          wday_names.(wday)
          mon_names.(mon)
          mday
@@ -253,14 +278,20 @@ let omake_magic buf =
          min
          sec
          (year + 1900)
-         (Unix.gethostname ());
+         (String.escaped (Unix.gethostname ()));
       flush buf
 
 (************************************************************************
  * OMakeroot file.
  *)
 let omake_root buf name =
-   let version = read_version () in
+   let version = shorten_version (read_version ()) in
+   let version =
+      if Lm_string_util.contains_any version "()$\" " then
+         sprintf "$'''%s'''" version
+      else
+         version
+   in
    let inx = open_in name in
       fprintf buf "#\n# Required version of omake\n#\nOMakeVersion(%s, %s)\n\n" version version;
       copy_file buf inx;
@@ -293,12 +324,9 @@ let main () =
 let _ =
    Printexc.catch main ()
 
-(*!
- * @docoff
- *
+(*
  * -*-
  * Local Variables:
- * Caml-master: "compile"
  * End:
  * -*-
  *)
