@@ -10,16 +10,16 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; version 2
  * of the License.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * 
+ *
  * Additional permission is given to link this library with the
  * with the Objective Caml runtime, and to redistribute the
  * linked executables.  See the file LICENSE.OMake for more details.
@@ -33,6 +33,7 @@ open Lm_location
 
 open Omake_ir
 open Omake_env
+open Omake_pos
 open Omake_eval
 open Omake_node
 open Omake_exec
@@ -40,7 +41,9 @@ open Omake_rule
 open Omake_state
 open Omake_symbol
 open Omake_exec_type
+open Omake_value_type
 open Omake_options
+open Omake_var
 
 module Pos = MakePos (struct let name = "Omake_main" end);;
 open Pos
@@ -90,7 +93,7 @@ let add_unknown options s =
          let l = String.length s in
          let v = String.sub s 0 i in
          let x = String.sub s (succ i) (l - i - 1) in
-            Omake_builtin.add_command_def (Lm_symbol.add v) x
+            Omake_builtin.add_command_def v x
       with
          Not_found ->
             add_target s
@@ -198,6 +201,8 @@ let spec =
               "Debug the file database";
            "-debug-hash", Lm_arg.Set debug_hash, (**)
               "Show Lm_hash statistics";
+           "-debug-thread", Lm_arg.Set Lm_thread_pool.debug_thread, (**)
+              "Show thread operations";
            "-allow-exceptions", Lm_arg.SetFold set_allow_exceptions_opt, (**)
               "Do not catch top-level exceptions (for use with OCAMLRUNPARAM=b)"];
        "Internal flags", (**)
@@ -301,10 +306,9 @@ let main_remote cwd options =
    let cwd   = Dir.cwd () in
    let exec  = Exec.create cwd options in
    let cache = Omake_cache.create () in
-   let pos   = string_exp_pos "main_remote" in
    let venv  = Omake_env.create options "." exec cache in
    let venv  = Omake_builtin.venv_add_command_defs venv in
-   let venv  = Omake_env.venv_add_var venv ScopeGlobal pos targets_sym (ValString (String.concat " " !targets)) in
+   let venv  = Omake_env.venv_add_var venv targets_var (ValString (String.concat " " !targets)) in
    let venv  = Omake_builtin.venv_add_builtins venv in
 
    (* Shell evaluator *)
@@ -341,6 +345,8 @@ let _ =
             eprintf "@\n@[<hv 3>*** omake fatal error:@ %s@]@." s;
             exit 3
    in
+      Lm_thread_core.debug_mutex := !Lm_thread_pool.debug_thread;
+      Lm_thread.debug_lock := !Lm_thread_pool.debug_thread;
       (* Run it *)
       match !server_flag with
          Some cwd ->
