@@ -34,13 +34,16 @@ open Lm_location
 open Lm_string_set
 
 open Omake_ir
+open Omake_var
 open Omake_env
+open Omake_pos
 open Omake_node_sig
 open Omake_node
-open Omake_symbol
 open Omake_value
+open Omake_symbol
 open Omake_shell_sys
 open Omake_shell_type
+open Omake_value_type
 open Omake_shell_sys_type
 
 module Pos = MakePos (struct let name = "Omake_shell_job" end)
@@ -200,7 +203,7 @@ let print_exit_code venv force pid code =
          if force then
             eprintf "- %d: done@." pid
     | JobExited code ->
-         if force || venv_defined venv ScopeGlobal printexitvalue_sym then
+         if force || venv_defined venv printexitvalue_var then
             eprintf "- %d: exited with code %d@." pid code
     | JobSignaled code ->
          eprintf "- %d: terminated with signal %d@." pid code
@@ -211,31 +214,6 @@ let print_exit_code venv force pid code =
  * Utilities
  *)
 
-(*
- * Create a pipe.
- *)
-let with_pipe f =
-   let read, write = Unix.pipe () in
-      try f read write with
-         exn ->
-            close_fd read;
-            close_fd write;
-            raise exn
-
-(*
- * Test for background jobs.
- *)
-let is_background_pipe pipe =
-   match pipe with
-      PipeBackground _ ->
-         true
-    | PipeApply _
-    | PipeCommand _
-    | PipeCond _
-    | PipeCompose _
-    | PipeGroup _ ->
-         false
-
 (*
  * Get an array representation of the environment.
  *)
@@ -426,20 +404,29 @@ let create_apply_top venv stdin stdout stderr apply =
       (* The function will close its files on its own *)
       try
          if !debug_shell then
-            eprintf "create_apply_top: %i duplicating channels@." (Unix.getpid ());
+            eprintf "create_apply_top pid=%i: duplicating channels@." (Unix.getpid ());
          let stdin  = Unix.dup stdin in
          let stdout = Unix.dup stdout in
          let stderr = Unix.dup stderr in
-         let code, v = f venv stdin stdout stderr env args in
-         let v = unexport venv v restore_vars in
+         let info = f venv stdin stdout stderr env args in
             if !debug_shell then
-               eprintf "create_apply_top %i: done@." (Unix.getpid ());
+               eprintf "create_apply_top pid=%i: done@." (Unix.getpid ());
             cleanup ();
-            code, v
+            info
       with
-         exn ->
+         ExitException (_, code) ->
             if !debug_shell then
-               eprintf "create_apply_top %i: error: %a@." (Unix.getpid ()) Omake_exn_print.pp_print_exn exn;
+               eprintf "create_apply_top pid=%i: exit exception: %i@." (Unix.getpid ()) code;
+            cleanup ();
+            code, venv, ValOther (ValExitCode code)
+       | ExitParentException (pos, code) ->
+            if !debug_shell then
+               eprintf "create_apply_top pid=%i: exit from parent exception: %i@." (Unix.getpid ()) code;
+            cleanup ();
+            raise (ExitException (pos, code))
+       | exn ->
+            if !debug_shell then
+               eprintf "create_apply_top pid=%i: error: %a@." (Unix.getpid ()) Omake_exn_print.pp_print_exn exn;
             cleanup ();
             raise exn
 
@@ -465,7 +452,8 @@ let create_apply venv pgrp bg stdin stdout stderr apply =
 
    (* The actual function call *)
    let apply_fun stdin stdout stderr pgrp =
-      fst (f venv stdin stdout stderr env args)
+      let code, _, _ = f venv stdin stdout stderr env args in
+         code
    in
    let thread_info =
       { create_thread_stdin = stdin;
@@ -510,7 +498,7 @@ let find_executable_string venv pos loc exe =
          in
             resolve_exe Omake_cache.exe_suffixes
       else
-         let path = venv_find_var venv ScopeGlobal pos loc path_sym in
+         let path = venv_find_var venv pos loc path_var in
          let path = Omake_eval.path_of_values venv pos (values_of_value venv pos path) "." in
          let path = Omake_cache.ls_exe_path cache path in
             try Omake_cache.exe_find cache path exe with
@@ -524,7 +512,7 @@ let find_executable venv pos loc exe =
        | ExeString exe ->
             find_executable_string venv pos loc exe
        | ExeNode node ->
-            if Omake_cache.exe_suffixes = ["."] then
+            if Omake_cache.exe_suffixes = [""] || Omake_cache.exists (venv_cache venv) ~force:true node then
                node
             else
                find_executable_string venv pos loc (Node.absname node)
@@ -601,6 +589,27 @@ let create_command venv pgrp bg stdin stdout stderr command =
                   raise exn
             end
 
+(*
+ * Evaluate a conditional, to see if the conditional operation should be performed.
+ *)
+let cond_continue op = function
+   JobExited 0 ->
+      (match op with
+          PipeAnd
+        | PipeSequence ->
+             true
+        | PipeOr ->
+             false)
+ | JobExited _ ->
+      (match op with
+          PipeOr
+        | PipeSequence ->
+             true
+        | PipeAnd ->
+             false)
+ | _ ->
+      false
+
 (*
  * Create a conditional.
  *)
@@ -620,20 +629,39 @@ let rec create_cond venv pgrp stdin stdout stderr op pipe1 pipe2 =
  * Create an actual pipe.
  *)
 and create_compose venv pgrp stdin stdout stderr divert_stderr pipe1 pipe2 =
-   with_pipe (fun stdin' stdout' ->
-         let stderr' =
-            if divert_stderr then
-               stdout'
-            else
-               stderr
-         in
-         let () = set_close_on_exec stdout' in
-         let exp2 = create_pipe_aux venv pgrp true stdin' stdout stderr pipe2 in
-         let () = close_fd stdin' in
-         let () = clear_close_on_exec stdout' in
-         let exp1 = create_pipe_aux venv pgrp true stdin stdout' stderr' pipe1 in
+   let stdin', stdout' = Unix.pipe () in
+   let stderr' =
+      if divert_stderr then
+         stdout'
+      else
+         stderr
+   in
+   let () = set_close_on_exec stdout' in
+   let exp2 = 
+      try create_pipe_aux venv pgrp true stdin' stdout stderr pipe2 with
+         exn ->
+            close_fd stdin';
             close_fd stdout';
-            SubjobPipe (exp1, exp2))
+            raise exn
+   in
+   let () = close_fd stdin' in
+   let () = clear_close_on_exec stdout' in
+   let exp1 = 
+      try
+         create_pipe_aux venv pgrp true stdin stdout' stderr' pipe1
+      with 
+         OmakeException _
+       | Unix.Unix_error _
+       | Failure _ as exn ->
+            eprintf "%a@." Omake_exn_print.pp_print_exn exn;
+            SubjobFinished (JobExited Omake_state.exn_error_code, venv)
+       | exn ->
+            close_fd stdout';
+            ignore(wait_exp pgrp exp2);
+            raise exn
+   in
+      close_fd stdout';
+      SubjobPipe (exp1, exp2)
 
 (*
  * Create a subshell.
@@ -643,8 +671,12 @@ and create_shell venv pgrp bg stdin stdout stderr pipe =
       eprintf "create_shell@.";
    let create_fun stdin stdout stderr pgrp =
       let exp =
-         try create_pipe_aux venv pgrp false stdin stdout stderr pipe with
-            exn ->
+         try
+            create_pipe_aux venv pgrp false stdin stdout stderr pipe
+         with
+            ExitException (_, code) ->
+               SubjobFinished (JobExited code, venv)
+          | exn ->
                eprintf "@[<v 0>%a@ Process group exception.@]@." Omake_exn_print.pp_print_exn exn;
                raise exn
       in
@@ -683,8 +715,12 @@ and create_group venv pgrp stdin stdout stderr group =
    in
    let create_fun stdin stdout stderr pgrp =
       let exp =
-         try create_pipe_aux venv pgrp false stdin stdout stderr pipe with
-            exn ->
+         try 
+            create_pipe_aux venv pgrp false stdin stdout stderr pipe
+         with
+            ExitException (_, code) ->
+               SubjobFinished (JobExited code, venv)
+          | exn ->
                eprintf "@[<v 0>%a@ Process group exception.@]@." Omake_exn_print.pp_print_exn exn;
                raise exn
       in
@@ -722,9 +758,7 @@ and create_pipe_aux venv pgrp fork stdin stdout stderr pipe =
          if fork then
             SubjobProcess (create_apply venv pgrp true stdin stdout stderr apply, venv)
          else
-            let code, v = create_apply_top venv stdin stdout stderr apply in
-            let pos = string_pos "Omake_shell_job.create_pipe_aux" (loc_exp_pos loc) in
-            let venv, _ = add_exports venv pos v in
+            let code, venv, _ = create_apply_top venv stdin stdout stderr apply in
                SubjobFinished (JobExited code, venv)
     | PipeCommand (_, command) ->
          SubjobProcess (create_command venv pgrp true stdin stdout stderr command, venv)
@@ -740,27 +774,6 @@ and create_pipe_aux venv pgrp fork stdin stdout stderr pipe =
     | PipeBackground (_, pipe) ->
          create_pipe_aux venv pgrp true stdin stdout stderr pipe
 
-(*
- * Create a pipe.
- * If this is a simple job, do not
- * monitor the pipe.
- *)
-and create_pipe_exn venv bg stdin stdout stderr pipe =
-   let rec create bg pipe =
-      match pipe with
-         PipeApply (_, apply) ->
-            create_apply venv 0 bg stdin stdout stderr apply
-       | PipeCommand (_, command) ->
-            create_command venv 0 bg stdin stdout stderr command
-       | PipeCond _
-       | PipeCompose _
-       | PipeGroup _ ->
-            create_shell venv 0 bg stdin stdout stderr pipe
-       | PipeBackground (_, pipe) ->
-            create true pipe
-   in
-      create bg pipe
-
 (*
  * Create a thread.  This may actually be a separate
  * process.
@@ -773,7 +786,7 @@ and create_thread venv f stdin stdout stderr =
    let pgrp = create_top_thread venv f stdin stdout stderr in
    let job  = new_job pgrp None in
       if !debug_shell then
-         eprintf "Started thread with pgrg %i, internal id %i@." job.job_pgrp job.job_id;
+         eprintf "Started thread with pgrp %i, internal id %i@." job.job_pgrp job.job_id;
       job.job_state <- JobBackground;
       InternalPid job.job_id
 
@@ -795,38 +808,31 @@ and wait_exp pgrp exp =
 
 and wait_exp2 pgrp exp =
    (* Wait for a job to complete; ignore stopped processes *)
-   let rec wait () =
-      if !debug_shell then
-         eprintf "wait_exp2: %i waiting for pgrp %i@." (Unix.getpid()) pgrp;
-      let code =
-         try Some (Omake_shell_sys.wait pgrp false false) with
-            Unix.Unix_error (Unix.EINTR, _, _) ->
-               None
-      in
-         if !debug_shell then
-            eprintf "wait_exp: some event happened@.";
-         match code with
+   if !debug_shell then
+      eprintf "wait_exp2: %i waiting for pgrp %i@." (Unix.getpid()) pgrp;
+   let code =
+      try Some (Omake_shell_sys.wait pgrp false false) with
+         Unix.Unix_error (Unix.EINTR, _, _) ->
             None
-          | Some (_, Unix.WSTOPPED _) ->
-               wait ()
-          | Some (pid, Unix.WEXITED code) ->
-               pid, JobExited code
-          | Some (pid, Unix.WSIGNALED code) ->
-               pid, JobSignaled code
    in
-   let pid, code = wait () in
-      (* Evaluate the expression *)
       if !debug_shell then
-         eprintf "wait_exp2: %i handling event: pid=%d@." (Unix.getpid()) pid;
-      let exp = eval_exp pgrp exp pid code in
-         match exp with
-            SubjobFinished (JobExited code, _)
-          | SubjobFinished (JobSignaled code, _) ->
+         eprintf "wait_exp: some event happened@.";
+      match code with
+         None
+       | Some (_, Unix.WSTOPPED _) ->
+            wait_exp2 pgrp exp
+       | Some (pid, (Unix.WEXITED _| Unix.WSIGNALED _ as code)) ->
+            let code =
+               match code with
+                  Unix.WEXITED code -> JobExited code
+                | Unix.WSIGNALED code -> JobSignaled code
+                | Unix.WSTOPPED _ -> raise (Invalid_argument "Omake_shell_job.wait_exp2: internal error")
+            in
+               (* Evaluate the expression *)
                if !debug_shell then
-                  eprintf "wait_exp2: %i exiting %d@." (Unix.getpid()) code;
-               code
-          | exp ->
-               wait_exp2 pgrp exp
+                  eprintf "wait_exp2: %i handling event: pid=%d@." (Unix.getpid()) pid;
+               let exp = eval_exp pgrp exp pid code in
+                  wait_exp pgrp exp
 
 (*
  * Evaluate the expression.
@@ -861,26 +867,7 @@ and eval_exp pgrp e pid code =
                          cond_stderr = stderr
                        } = cond
                    in
-                   let cont =
-                      match code with
-                         JobExited 0 ->
-                            (match op with
-                                PipeAnd
-                              | PipeSequence ->
-                                   true
-                              | PipeOr ->
-                                   false)
-                       | JobExited _ ->
-                            (match op with
-                                PipeOr
-                              | PipeSequence ->
-                                   true
-                              | PipeAnd ->
-                                   false)
-                       | _ ->
-                            false
-                   in
-                      if cont then
+                      if cond_continue op code then
                          eval_exp_top pgrp (create_pipe_aux venv pgrp false stdin stdout stderr pipe)
                       else
                          SubjobFinished (code, venv)
@@ -891,6 +878,20 @@ and eval_exp pgrp e pid code =
    in
       eval e
 
+(*
+ * Utility function for wait_top_aux and cleanup_top_aux
+ *)
+let finalize_job job = function
+   Unix.WEXITED code ->
+      remove_job job;
+      JobExited code
+ | Unix.WSIGNALED code ->
+      remove_job job;
+      JobSignaled code
+ | Unix.WSTOPPED code ->
+      job.job_state <- JobSuspended;
+      JobStopped code
+
 (*
  * Wait for a job to finish.
  * This is executed in the main process.
@@ -906,24 +907,9 @@ let rec wait_top_aux job =
       if pid <> pgrp then
          wait_top_aux job
       else
-         let code =
-            match status with
-               Unix.WEXITED code ->
-                  if !debug_shell then
-                     eprintf "wait_top_aux: exited %d@." code;
-                  remove_job job;
-                  JobExited code
-             | Unix.WSIGNALED code ->
-                  if !debug_shell then
-                     eprintf "wait_top_aux: signaled %d@." code;
-                  remove_job job;
-                  JobSignaled code
-             | Unix.WSTOPPED code ->
-                  if !debug_shell then
-                     eprintf "wait_top_aux: stopped %d@." code;
-                  job.job_state <- JobSuspended;
-                  JobStopped code
-         in
+         let code = finalize_job job status in
+            if !debug_shell then
+               eprintf "wait_top_aux: %a@." pp_print_status code;
             code, status
 
 let wait_top venv job =
@@ -937,47 +923,90 @@ let wait_pid venv job =
       Omake_shell_sys.set_tty_pgrp 0;
       status
 
+(*
+ * Create a pipe.
+ * If this is a simple job, do not monitor the pipe.
+ *)
+let rec create_pipe_exn venv bg stdin stdout stderr = function
+   PipeApply (_, apply) ->
+      create_apply venv 0 bg stdin stdout stderr apply
+ | PipeCommand (_, command) ->
+      create_command venv 0 bg stdin stdout stderr command
+ | PipeCond _
+ | PipeCompose _
+ | PipeGroup _ as pipe ->
+      create_shell venv 0 bg stdin stdout stderr pipe
+ | PipeBackground (_, pipe) ->
+      create_pipe_exn venv true stdin stdout stderr pipe
+
 (*
  * When the pipe is created:
  * If the pipe is in the background, the terminal remains attached.
  * If the pipe is not in the background, we retain control of the terminal.
+ *
+ * WARNING: this function should not be called if
+ *    1. the pipeline is an alias, and
+ *    2. the output is a pipe connected internally.
+ * The reason is that the alias is not processed in a thread.
+ * If it generates a lot of output, it will block, causing
+ * deadlock because the output processor is not being run.
+ *
+ * Remember that rules pass their output to the output
+ * processor through a pipe like this.  However, commands
+ * in rules are processed by create_process, not create_job.
  *)
-let create_job venv pipe stdin stdout stderr =
+let rec create_job_aux venv pipe stdin stdout stderr =
    if !debug_shell then
       eprintf "Creating pipe: %a@." pp_print_string_pipe pipe;
 
-   (* Evaluate applications eagerly *)
    match pipe with
-      PipeApply (loc, apply) when stdout = Unix.stdout && stderr = Unix.stderr ->
-         let _, value = create_apply_top venv stdin stdout stderr apply in
-            value
-    | _ ->
-         (* Otherwise, create the pipeline *)
-         let bg   = is_background_pipe pipe in
-         let pgrp = create_pipe_exn venv bg stdin stdout stderr pipe in
+      PipeApply (loc, apply) ->
+         (* Evaluate applications eagerly *)
+         create_apply_top venv stdin stdout stderr apply
+    | PipeBackground (_, pipe) ->
+         (* Create a background job *)
+         let pgrp = create_pipe_exn venv true stdin stdout stderr pipe in
          let job  = new_job pgrp (Some pipe) in
-            if not bg then begin
-               if !debug_shell then
-                  eprintf "Running pgrp %d (my pid = %d)@." pgrp (Unix.getpid ());
-               (*
-                * On Mac OSX this call fails with EPERM.
-                * I believe this is because the sub-process
-                * sets the controlling terminal itself (see
-                * Omake_shell_sys_unix.create_process).
-                *
-                * This means that the sub-process takes over the terminal,
-                * and we can't set it anymore.
-                *
-                * This seems like a bogus explanation, because we have
-                * to get the terminal back on suspend...
-               Omake_shell_sys.set_tty_pgrp pgrp;
-                *)
-               ValOther (ValExitCode (int_of_code (wait_top venv job)))
-            end
-            else begin
-               job.job_state <- JobBackground;
-               ValNone
-            end
+            job.job_state <- JobBackground;
+            0, venv, ValNone
+    | PipeCompose _
+      (*
+       * XXX: TODO (Aleksey 2007/06/26) 
+       * PipeCompose should be handled similar to PipeCond, where only the left hand
+       * side should be forked, while the right hand side should be evaluated in the current process.
+       *)
+    | PipeGroup _
+    | PipeCommand _ ->
+         (* Otherwise, fork a foreground job *)
+         let pgrp = create_pipe_exn venv false stdin stdout stderr pipe in
+         let job  = new_job pgrp (Some pipe) in
+            if !debug_shell then
+               eprintf "Running pgrp %d (my pid = %d)@." pgrp (Unix.getpid ());
+            (*
+             * On Mac OSX this call fails with EPERM.
+             * I believe this is because the sub-process
+             * sets the controlling terminal itself (see
+             * Omake_shell_sys_unix.create_process).
+             *
+             * This means that the sub-process takes over the terminal,
+             * and we can't set it anymore.
+             *
+             * This seems like a bogus explanation, because we have
+             * to get the terminal back on suspend...
+            Omake_shell_sys.set_tty_pgrp pgrp;
+             *)
+            let code = int_of_code (wait_top venv job) in
+               code, venv, ValOther (ValExitCode code)
+    | PipeCond (_, op, pipe1, pipe2) ->
+         let (code, venv, _) as info = create_job_aux venv pipe1 stdin stdout stderr in
+            if cond_continue op (JobExited code) then
+               create_job_aux venv pipe2 stdin stdout stderr
+            else
+               info
+
+let create_job venv pipe stdin stdout stderr =
+   let _, venv, value = create_job_aux venv pipe stdin stdout stderr in
+      venv, value
 
 (*
  * This is a variation: create the process and return the pid.
@@ -992,10 +1021,10 @@ let create_process venv pipe stdin stdout stderr =
        * prevent possible blocking on I/O.
        *)
       PipeApply (loc, apply) when stdout = Unix.stdout && stderr = Unix.stderr ->
-         let code, value =
+         let code, venv, value =
             create_apply_top venv stdin stdout stderr apply
          in
-            ResultPid (code, value)
+            ResultPid (code, venv, value)
     | _ ->
          let pgrp = create_pipe_exn venv true stdin stdout stderr pipe in
          let job  = new_job pgrp (Some pipe) in
@@ -1035,22 +1064,10 @@ let cleanup_top_aux () =
    let pid, status = Omake_shell_sys.wait 0 true true in
    let job = find_job_by_pgrp pid in
    let pid = job.job_id in
-      match status with
-         Unix.WEXITED code ->
-            if !debug_shell then
-               eprintf "cleanup_top_aux: exited %d@." code;
-            remove_job job;
-            pid, JobExited code
-       | Unix.WSIGNALED code ->
-            if !debug_shell then
-               eprintf "cleanup_top_aux: signaled %d@." code;
-            remove_job job;
-            pid, JobSignaled code
-       | Unix.WSTOPPED code ->
-            if !debug_shell then
-               eprintf "cleanup_top_aux: stopped %d@." code;
-            job.job_state <- JobSuspended;
-            pid, JobStopped code
+   let code = finalize_job job status in
+      if !debug_shell then
+         eprintf "cleanup_top_aux: %a@." pp_print_status code;
+      pid, code
 
 let rec cleanup venv =
    let code =
@@ -1151,7 +1168,7 @@ let waitpid venv pos pid =
                eprintf "Omake_shell_job.waitpid: internal id %i@." pid;
             let status = wait_pid venv (job_of_pid pos pid) in
                pid, status, ValNone
-       | ResultPid (code, value) ->
+       | ResultPid (code, _, value) ->
             0, Unix.WEXITED code, value
 
 (*
