@@ -34,7 +34,9 @@ open Lm_symbol
 open Lm_location
 
 open Omake_env
+open Omake_pos
 open Omake_shell_type
+open Omake_value_type
 
 (*
  * Create a thread or process running the function.
@@ -45,7 +47,7 @@ val create_thread : venv -> (Unix.file_descr -> Unix.file_descr -> Unix.file_des
 (*
  * Start a job given a pipe specification.
  *)
-val create_job : venv -> string_pipe -> Unix.file_descr -> Unix.file_descr -> Unix.file_descr -> value
+val create_job : venv -> string_pipe -> Unix.file_descr -> Unix.file_descr -> Unix.file_descr -> venv * value
 
 (*
  * Create a process in the background.
